// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include "ModelConstraintManager.h"

namespace MAd {

  // -------------------------------------------------------------------
  void ModelConstraintManager::initialize(pGModel _model)
  {
    model = _model;
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::finalize()
  {
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::setModel(pGModel _model)
  {
    model = _model;
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::clear()
  {
    constrEntities.clear();
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::constrain(int type, int id)
  {
    pGEntity pGE = GM_entityByTag(model, type, id);
    constrain(pGE);
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::constrain(pGEntity pGE)
  {
    // constrain entity
    constrEntities.insert(pGE);
  
#ifdef _HAVE_GMSH_
    // constrain lower geometrical levels
    pPGList closure = GEN_closure(pGE);
    void * temp = NULL;
    while (pGEntity pGE_it = PGList_next(closure, &temp))
        constrain(pGE_it);
    PGList_delete(closure);
#endif
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::unconstrain(int type, int id)
  {
    pGEntity pGE = GM_entityByTag(model, type, id);
    unconstrain(pGE);
  }

  // -------------------------------------------------------------------
  void ModelConstraintManager::unconstrain(pGEntity pGE)
  {
    std::set<pGEntity>::iterator eIter = constrEntities.find(pGE);
    if ( eIter != constrEntities.end() ) constrEntities.erase(eIter);
  }

  // -------------------------------------------------------------------
  bool ModelConstraintManager::constrained(pGEntity pGE)
  {
    //GCTODO: check upper-level geometric entities: need model with connectivity

    if ( constrEntities.find(pGE) != constrEntities.end() ) return true;
    return false;
  }

  // -------------------------------------------------------------------

}
