// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_CURVEDGESPLIT
#define _H_CURVEDGESPLIT

#include "MAdOperatorBase.h"
#include "MeshSizeBase.h"
namespace MAd{

  // -----------------------------------------------------------------
  class curvedEdgeSplit: public MAdOperatorBase
  {
  public:
   curvedEdgeSplit(pMesh, DiscreteSF *);
   curvedEdgeSplit(const curvedEdgeSplit &);
   ~curvedEdgeSplit() 
   {
    if (xyzSize) delete xyzSize;
    if(midCoordList)
     {
      for(int i=0; i<cavitySize; i++) delete[] midCoordList[i];
      delete[] midCoordList;
     }
   }

  public:
  
  operationType type() const { return MAd_ESPLIT; }
  
  bool checkConstraints() const;
  bool checkGeometry();
  void getCavity(pPList *) const;
  void setCavitySize(pPList);
  void getMidCoord3d(pRegion, double, double[3]);
  void getMidCoord2d(pFace,double, double[3]);
  void getMidCoord(double);
  void createSplitCavity2d(double);
  void createSplitCavity3d(double);
//
  void splitCreateEdges(pEdge,double,int,pVertex,pEdge,pEdge);
  void createMedianEdge(pFace,pEdge,pVertex,double,int,int,pEdge);
  void createFaceHOVertices(pFace,pEdge,double,int,pVertex[],pVertex[]);
  void splitCavity2d(double,int);
  void splitElement(pRegion,pEdge,double,int);
  void splitCavity3d(pEdge,double,int);
  
  void apply() {};
  private:

    bool evaluateShapes() {return true;};
    bool evaluateShapes2D() {return true;};
    void evaluateLengths() const {};
  
  private:
   pEdge edge;
   double xyz[3];
   double** midCoordList;
   int cavitySize;
   double u;
   pMSize xyzSize;  
   void controlPointsTriangle(double uv1[2], double uv2[2], double uv3[2], int o, double *uf, double *vf){
//    int nb = (o+1)*(o+2)/2; // nber of control points in a triangle for a given order
    if (o ==0) {uf[0]=uv1[0];vf[0]=uv1[1]; return;}     
   
    uf[0]= uv1[0]; vf[0]= uv1[1];
    uf[1]= uv2[0]; vf[1]= uv2[1];
    uf[2]= uv3[0]; vf[2]= uv3[1];
    // edges
    for (int k=0; k< (o-1); k++){
     uf[3+k] = uv1[0]+k*(uv2[0]-uv1[0])/o; vf[3+k]=uv1[1]+k*(uv2[1]-uv1[1])/o;
     uf[3+o-1+k] = uv2[0]+k*(uv3[0]-uv2[0])/o ; vf[3+o-1+k] = uv2[1]+k*(uv3[1]-uv2[1])/o;
     uf[3+2*(o-1)+k] = uv3[0]+ k*(uv1[0]-uv3[0])/o; vf[3+2*(o-1)+k] = uv3[1]+ k*(uv1[1]-uv3[1])/o;
    }
    // if order greater than 2: control points in the face
    // these points are defined by an homothetic triangle of order o-3
    // translated from original element by a vector 1/o*vec(u)+1/o*vec(v)
    // where vec(u)=(1;0;0) and vec(v)=(0;1;0)
    if (o >2){
     double *uin;
     double *vin;
     int nbin = ((o-3)+1)*((o-3)+2)/2;
     uin = new double[nbin];
     vin = new double[nbin];
     double uv1temp[2];
     double uv2temp[2];
     double uv3temp[2];
     for(int k=0; k<2; k++) {
      uv1temp[k]=uv1[k]/o; 
      uv2temp[k]=uv2[k]/o; 
      uv3temp[k]=uv3[k]/o;
     }
     controlPointsTriangle(uv1temp,uv2temp,uv3temp,o-3,uin,vin);
     for (int k=0; k< nbin; k++){
      uf[3+k+3*(o-1)] = uin[k]+uv3temp[0]-2*uv1temp[0]+uv2temp[0];
      vf[3+k+3*(o-1)] = vin[k]+uv3temp[0]-2*uv1temp[0]+uv2temp[0];
     }
     delete []uin;
     delete []vin;
    }
    return;     
   }
  };

  // -------------------------------------------------------------------
  inline curvedEdgeSplit::curvedEdgeSplit(const curvedEdgeSplit & _es):
    MAdOperatorBase(_es)
  {
    edge = _es.edge;
    xyz[0] = _es.xyz[0];
    xyz[1] = _es.xyz[1];
    xyz[2] = _es.xyz[2];
    u = _es.u;
    xyzSize = MS_copy(_es.xyzSize);
  }

  // -------------------------------------------------------------------
  inline curvedEdgeSplit::curvedEdgeSplit(pMesh _m, DiscreteSF * _sf):
    MAdOperatorBase(_m,_sf), edge(NULL)
  {
    xyz[0] = 0.;
    xyz[1] = 0.;
    xyz[2] = 0.;
    u = -1.;
    xyzSize = NULL;
  }

  // -------------------------------------------------------------------
}
#endif
