// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_CURVEDEDGESPLITOP
#define _H_CURVEDEDGESPLITOP

#include "MAdOperatorBase.h"
#include "MeshSizeBase.h"
#include "edgeSplitOp.h"


namespace MAd {

  // -------------------------------------------------------------------
  //   class CurvededgeSplitOp: public MAdOperatorBase
  class CurvededgeSplitOp: public edgeSplitOp
  {
  public:

    CurvededgeSplitOp(pMesh, DiscreteSF *);
    CurvededgeSplitOp(const CurvededgeSplitOp &);
    ~CurvededgeSplitOp() { if (xyzSize) delete xyzSize; }

    operationType type() const { return MAd_ESPLIT; }

    // stores the edge to be splitted, computes the location of the split 
    // and returns the (unique) adimensional square length of the resulting edges
    double setSplitEdge(pEdge _edge);

    // Not implemented for the curved operator
    // stores the edge to be splitted and parametric location of the split 
    // _t is the parametric location and must be in [0,1]
    void setSplitEdge(pEdge _edge, double _t);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    bool evaluateShapes2D();
    void evaluateLengths() const;

  private:

    pEdge edge;
    double xyz[3], u; // new vertex location: euclidian and edge parameter
    pMSize xyzSize;   // size at the new vertex
  };

  // -------------------------------------------------------------------
  inline CurvededgeSplitOp::CurvededgeSplitOp(const CurvededgeSplitOp & _es):
    edgeSplitOp(_es)
//     MAdOperatorBase(_es)
  {
    edge = _es.edge;
    xyz[0] = _es.xyz[0];
    xyz[1] = _es.xyz[1];
    xyz[2] = _es.xyz[2];
    u = _es.u;
    xyzSize = MS_copy(_es.xyzSize);
  }

  // -------------------------------------------------------------------
  inline CurvededgeSplitOp::CurvededgeSplitOp(pMesh _m, DiscreteSF * _sf):
    edgeSplitOp(_m,_sf), edge(NULL)
//     MAdOperatorBase(_m,_sf), edge(NULL)
  {
    xyz[0] = 0.;
    xyz[1] = 0.;
    xyz[2] = 0.;
    u = -1.;
    xyzSize = NULL;
  }

  // -------------------------------------------------------------------
  
  inline double CurvededgeSplitOp::setSplitEdge(pEdge _edge)
  {
      newVertex = NULL;
      if (xyzSize) delete xyzSize;
      edge = _edge;
      double lenReduc;
      u = sizeField->SF_E_center(_edge, xyz, &lenReduc, &xyzSize);
      return lenReduc;
  }

  // -------------------------------------------------------------------
  inline void CurvededgeSplitOp::setSplitEdge(pEdge _edge, double _t)
  {
      newVertex = NULL;
      MAdMsgSgl::instance().error(__LINE__, __FILE__,
          "Split edge at parametric location is not implemented for curved operator");
  }

  // -------------------------------------------------------------------

}

#endif
