// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_CURVEDSLIVERFACEHANDLER
#define _H_CURVEDSLIVERFACEHANDLER

#include "MAdOperatorBase.h"
#include "sliverFaceHandler.h"

#include <string>

namespace MAd {

  // -------------------------------------------------------------------
  class CurvedsliverFaceHandler : public sliverFaceHandler {

  public:

    CurvedsliverFaceHandler(pMesh, DiscreteSF *);
    ~CurvedsliverFaceHandler() {}

  public:

    void removeSliverFaces(int * nbIn=NULL, int * nbOut=NULL);

    void setSizeField(DiscreteSF *);

    // whether we can use operations on boundaries
    void collapseOnBoundary(bool, double);

    // whether we can use operations that add nodes
    void newVertexPermission(bool);
    bool getNewVertexPermission() const { return addNodes; }

    void enableReport(std::string);

  private:

    int findOperation(pFace, pMAdOperator *);

    bool F_isSliver(pFace);
    bool F_isSliver(pFace, double *);

    void reportSliver(pFace);

  private:

    pMesh mesh;
    DiscreteSF * sizeField;
    MeshQualityManager& mqm;

    bool collapseOnBdry;
    double dATol;

    bool addNodes; // whether or not we can use operators that add nodes 
                   // (edge split)

    bool   reportFailures;
    int    reportId;
    std::string reportPrefix;

  };

  // -------------------------------------------------------------------
  inline CurvedsliverFaceHandler::CurvedsliverFaceHandler(pMesh m, DiscreteSF * sf):
    sliverFaceHandler(m,sf),
    mesh(m),sizeField(sf), mqm(MeshQualityManagerSgl::instance()), 
    collapseOnBdry(false), dATol(MAdTOL), addNodes(true), 
    reportFailures(false),reportId(0),reportPrefix("")
  {}

  // -------------------------------------------------------------------
  inline void CurvedsliverFaceHandler::setSizeField(DiscreteSF * sf)
  {
    sizeField = sf;
  }

  // -------------------------------------------------------------------
  inline void CurvedsliverFaceHandler::collapseOnBoundary(bool cob, double tolerance)
  {
    collapseOnBdry = cob;
    dATol = tolerance;
  }

  // -------------------------------------------------------------------
  inline void CurvedsliverFaceHandler::newVertexPermission(bool allow)
  {
    addNodes = allow;
  }

  // -------------------------------------------------------------------
  inline void CurvedsliverFaceHandler::enableReport(std::string prefix)
  {
    reportFailures = true; 
    reportPrefix   = prefix;
  }

  // -------------------------------------------------------------------

}

#endif

