// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_LINEAREDGECOLLAPSEOP
#define _H_LINEAREDGECOLLAPSEOP

#include "MAdOperatorBase.h"
#include "edgeCollapseOp.h"

namespace MAd {

  // -------------------------------------------------------------------
  class LinearedgeCollapseOp: public edgeCollapseOp
  {
  public:

    LinearedgeCollapseOp(pMesh, DiscreteSF *);
    LinearedgeCollapseOp(const LinearedgeCollapseOp &);
    ~LinearedgeCollapseOp() {}

    operationType type() const { return MAd_ECOLLAPSE; }

    void collapseOnBoundary(bool, double);
    virtual void setCollapseEdge(pEdge, pVertex, pVertex);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    bool evaluateShapes2D();
    void evaluateLengths() const;
  
  private:

    pEdge edgeDel;
    pVertex vDel; // the vertex to be deleted
    pVertex vTgt; // the target vertex

    bool constrainBoundary;
    double dVTol, dATol; // tolerance on the relative change of volume or area
  };

  // -------------------------------------------------------------------
  inline LinearedgeCollapseOp::LinearedgeCollapseOp(const LinearedgeCollapseOp & _ec):
    edgeCollapseOp(_ec)
  {
    edgeDel = _ec.edgeDel;
    vDel = _ec.vDel;
    vTgt = _ec.vTgt;
    constrainBoundary = _ec.constrainBoundary;
    dVTol = _ec.dVTol;
    dATol = _ec.dATol;
  }

  // -------------------------------------------------------------------
  inline LinearedgeCollapseOp::LinearedgeCollapseOp(pMesh _m, DiscreteSF * _sf):
    edgeCollapseOp(_m,_sf)
  {
    edgeDel = NULL;
    vDel = NULL;
    vTgt = NULL;
    constrainBoundary = true;
    dVTol = MAdTOL;
    dATol = MAdTOL;
  }

  // -------------------------------------------------------------------
  inline void LinearedgeCollapseOp::collapseOnBoundary(bool cob, double tolerance)
  {
    constrainBoundary = !cob;
    dVTol = tolerance;
    dATol = tolerance;
  }

  // -------------------------------------------------------------------
  inline void LinearedgeCollapseOp::setCollapseEdge(pEdge _edgeDel,
                                              pVertex _vDel,
                                              pVertex _vTgt)
  {
    edgeDel = _edgeDel;
    vDel = _vDel;
    vTgt = _vTgt;
    results->reset();
  }

  // -------------------------------------------------------------------

}

#endif

