// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_LINEAREDGESPLITOP
#define _H_LINEAREDGESPLITOP

#include "MAdOperatorBase.h"
#include "MeshSizeBase.h"
#include "edgeSplitOp.h"

namespace MAd {

  // -------------------------------------------------------------------
//   class LinearedgeSplitOp: public MAdOperatorBase
  class LinearedgeSplitOp: public edgeSplitOp
  {
  public:

    LinearedgeSplitOp(pMesh, DiscreteSF *);
    LinearedgeSplitOp(const LinearedgeSplitOp &);
    ~LinearedgeSplitOp() { if (xyzSize) delete xyzSize; }

    operationType type() const { return MAd_ESPLIT; }

    // stores the edge to be splitted, computes the location of the split 
    // and returns the (unique) adimensional square length of the resulting edges
    double setSplitEdge(pEdge);

    // stores the edge to be splitted and parametric location of the split 
    // _t is the parametric location and must be in [0,1]
    void setSplitEdge(pEdge _edge, double _t);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    bool evaluateShapes2D();
    void evaluateLengths() const;

  private:

    pEdge edge;
    double xyz[3], u; // new vertex location: euclidian and edge parameter
    pMSize xyzSize;   // size at the new vertex
  };

  // -------------------------------------------------------------------
  inline LinearedgeSplitOp::LinearedgeSplitOp(const LinearedgeSplitOp & _es):
//   inline LinearedgeSplitOp::LinearedgeSplitOp(const LinearedgeSplitOp & _es):
  edgeSplitOp(_es)
//   MAdOperatorBase(_es)
  {
    edge = _es.edge;
    xyz[0] = _es.xyz[0];
    xyz[1] = _es.xyz[1];
    xyz[2] = _es.xyz[2];
    u = _es.u;
    xyzSize = MS_copy(_es.xyzSize);
  }

  // -------------------------------------------------------------------
  inline LinearedgeSplitOp::LinearedgeSplitOp(pMesh _m, DiscreteSF * _sf):
  edgeSplitOp(_m,_sf), edge(NULL)
//   MAdOperatorBase(_m,_sf), edge(NULL)
  {
    xyz[0] = 0.;
    xyz[1] = 0.;
    xyz[2] = 0.;
    u = -1.;
    xyzSize = NULL;
  }

  // -------------------------------------------------------------------
  inline double LinearedgeSplitOp::setSplitEdge(pEdge _edge)
  {
      newVertex = NULL;
      if (xyzSize) delete xyzSize;
      edge = _edge;
      double lenReduc;
      u = sizeField->SF_E_center(_edge, xyz, &lenReduc, &xyzSize);
      return lenReduc;
  }

  // -------------------------------------------------------------------
  inline void LinearedgeSplitOp::setSplitEdge(pEdge _edge, double t)
  {
      newVertex = NULL;
      edge = _edge;
      u = t;
      E_coordP1(edge, t, xyz);

      if (xyzSize) {
          delete xyzSize;
          if (sizeField)
              xyzSize = sizeField->SF_E_interpolate(edge, u);
      }
  }

  // -------------------------------------------------------------------

}

#endif
