// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_LINEAREDGESWAPOP
#define _H_LINEAREDGESWAPOP

#include "MAdOperatorBase.h"
#include "EdgeSwapConfig.h"
#include "edgeSwapOp.h"

namespace MAd {

  // -------------------------------------------------------------------
  class LinearedgeSwapOp : public edgeSwapOp
  {
  public:

    LinearedgeSwapOp();
    LinearedgeSwapOp(pMesh, DiscreteSF *);
    LinearedgeSwapOp(const LinearedgeSwapOp &);
    ~LinearedgeSwapOp() {}

    operationType type() const { return MAd_ESWAP; }
    int getMaxNumRgns() { return 7; }

    void setSwapEdge(pEdge);
    void swapOnBoundary(bool,double);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    void evaluateLengths() const;

    bool checkGeometry2D();
    bool checkGeometry3D();
    bool evaluateShapes2D();
    bool evaluateShapes3D();

    int apply3D();
    int apply2D();

    pEdge edge;          // the edge to be swapped
    int conf;            // best swap configuration - set by geomCheck()
    pVertex vt;          // top and bottom vertices of the edge (define the polygon orientation
    pVertex vb;          // w/r to vertices)
    std::vector<pVertex> vertices; // oriented list of vertices defining the polygon

    EdgeSwapConfiguration swap_config;

    bool constrainBoundary;
    bool checkVol; // tells if it's interesting to compute the variation of 
                   // volume in the cavity (deduced from classif of edge 
                   // and constrainBoundary)
    double dVTol;

    // reporting
    void reportSwap( );
    bool   report;
    int    reportId;
    std::string reportPrefix;

  };

  // -------------------------------------------------------------------
  inline LinearedgeSwapOp::LinearedgeSwapOp(pMesh _m, DiscreteSF * _sf):
    edgeSwapOp(_m,_sf), edge(0), conf(-1), vt(0), vb(0), 
    constrainBoundary(true),  checkVol(false), dVTol(MAdTOL),
    report(false), reportId(0), reportPrefix("")
  {
    vertices.reserve(7);
  }

  // -------------------------------------------------------------------
  inline LinearedgeSwapOp::LinearedgeSwapOp(const LinearedgeSwapOp & _es):
    edgeSwapOp(_es), swap_config( _es.swap_config )
  {
    edge        = _es.edge;
    conf        = _es.conf;
    vt          = _es.vt;
    vb          = _es.vb;
    vertices    = _es.vertices;

    constrainBoundary = _es.constrainBoundary;
    checkVol          = _es.checkVol;
    dVTol             = _es.dVTol;

    report       = _es.report;
    reportId     = _es.reportId;
    reportPrefix = _es.reportPrefix;
  }

  // -------------------------------------------------------------------
  inline void LinearedgeSwapOp::setSwapEdge(pEdge e) 
  {
    edge = e;
    results->reset();
    conf = -1;
    checkVol = false;
  }
  
  // -------------------------------------------------------------------
  inline void LinearedgeSwapOp::swapOnBoundary(bool eswapob, double dV)
  {
    constrainBoundary = !eswapob;
    dVTol = dV;
  }
}
// -------------------------------------------------------------------
#endif
