// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_LINEARFACESWAPOP
#define _H_LINEARFACESWAPOP

#include "MAdOperatorBase.h"
#include "faceSwapOp.h"

namespace MAd {

  // -------------------------------------------------------------------
  class LinearfaceSwapOp: public faceSwapOp
  {
  public:

    LinearfaceSwapOp(pMesh, DiscreteSF *);
    LinearfaceSwapOp(const LinearfaceSwapOp &);
    ~LinearfaceSwapOp() {};

    operationType type() const { return MAd_FSWAP; }
  
    void setSwapFace(pFace);

    void getCavity(pPList *) const;

    void apply();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    void evaluateLengths() const;

  private:

    pFace face;           // face to be swapped
    pRegion fRegions[2];  // its regions
    pVertex fOppVerts[2]; // opposite vertices in regions
  };

  // -------------------------------------------------------------------
  inline LinearfaceSwapOp::LinearfaceSwapOp(const LinearfaceSwapOp & _fs):
  faceSwapOp(_fs), face(_fs.face)
  {
    fRegions[0]  = _fs.fRegions[0];
    fRegions[1]  = _fs.fRegions[1];
    fOppVerts[0] = _fs.fOppVerts[0];
    fOppVerts[1] = _fs.fOppVerts[1];
  }

  // -------------------------------------------------------------------
  inline LinearfaceSwapOp::LinearfaceSwapOp(pMesh _m, DiscreteSF * _sf):
  faceSwapOp(_m,_sf), face(NULL)
  {
    fRegions[0]  = NULL;
    fRegions[1]  = NULL;
    fOppVerts[0] = NULL;
    fOppVerts[1] = NULL;
  }

  // -------------------------------------------------------------------
  inline void LinearfaceSwapOp::setSwapFace(pFace _face)
  {
    face = _face;

    for(int iR=0; iR<2; iR++) {
      fRegions[iR] = F_region(face,iR);
      if (fRegions[iR]) fOppVerts[iR] = R_fcOpVt(fRegions[iR],face);
    }

    results->reset();
  }

  // -------------------------------------------------------------------

}

#endif
