// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_LINEARVERTEXMOVEOP
#define _H_LINEARVERTEXMOVEOP

#include "vertexMoveOp.h"
#include "MAdOperatorBase.h"
#include "MobileObject.h"
#include "DistanceFunction.h"

#include <set>

#define OPTILOC_MIN_IMPROVE_RATIO  1.e-3
#define OPTILOC_MIN_IMPROVE_SHAPE  1.e-3

namespace MAd {

  // -------------------------------------------------------------------
  class LinearvertexMoveOp : public vertexMoveOp
  {
  public:

    LinearvertexMoveOp(pMesh, DiscreteSF *, bool _fixBndry=true);
    LinearvertexMoveOp(const LinearvertexMoveOp &);
    ~LinearvertexMoveOp() {}

    operationType type() const { return MAd_VERTEXMOVE; }

    void setFixedBndry(bool);

    void resetDisplacements();
    void addVDisplacement(pVertex, double[3]);
    void addVDisplacement(vDisplacement&);
    void setDisplacement(pVertex, double[3]);
    void setPosition(pVertex, double[3]);
    void setPositionToOptimal(pVertex);

    bool computeOptimalLocation(pVertex, double[3]) const;

    void getAffectedEdges(pPList *) const;
    void getCavity(pPList *) const;

    void apply();
    bool move (std::set<vDisplacement,vDisplacementLess>&, double factor=1.0);
    bool move (std::multiset<vDisplacement,vDisplacementLess>&, double factor=1.0);
    bool move (pVertex, double[3]);
    bool move (vDisplacement&);
    bool operate ();

  private:

    bool checkConstraints() const;
    bool checkGeometry();
    bool evaluateShapes();
    bool evaluateShapes2D();
    void evaluateLengths() const;

  private:

    std::multiset<vDisplacement,vDisplacementLess> vDisps;
    bool fixedBndry; // indicates if the boundary vertices can move
  };

  // -------------------------------------------------------------------
  inline LinearvertexMoveOp::LinearvertexMoveOp(const LinearvertexMoveOp & _vm):
  vertexMoveOp(_vm) 
  {
    vDisps = _vm.vDisps;
    fixedBndry = _vm.fixedBndry;
  }

  // -------------------------------------------------------------------
  inline LinearvertexMoveOp::LinearvertexMoveOp(pMesh _m, DiscreteSF * _sf, 
                                    bool _fixBndry): 
   vertexMoveOp(_m,_sf), fixedBndry(_fixBndry)
  {}

  // -------------------------------------------------------------------
  inline void LinearvertexMoveOp::setFixedBndry(bool fixed)
  {
    fixedBndry = fixed;
  }

  // -------------------------------------------------------------------

}

#endif
