// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_EDGESPLITOP
#define _H_EDGESPLITOP

#include "MAdOperatorBase.h"
#include "MeshSizeBase.h"

namespace MAd {
  
class edgeSplitOp: public MAdOperatorBase
  {
  public:

    edgeSplitOp(): newVertex(NULL) {};
    edgeSplitOp(pMesh _mesh, DiscreteSF * sf);
    edgeSplitOp(const edgeSplitOp & es);
    virtual ~edgeSplitOp();

    operationType type() const { return MAd_ESPLIT; }

    // stores the edge to be splitted, computes the location of the split 
    // and returns the (unique) adimensional square length of the resulting edges
    virtual double setSplitEdge(pEdge) = 0;

    // stores the edge to be splitted and parametric location of the split 
    // _t is the parametric location and must be in [0,1]
    virtual void setSplitEdge(pEdge _edge, double _t) = 0;

    void getCavity(pPList *) const = 0;

    void apply() = 0;

    inline pVertex getNewVertex() const { return newVertex; }

  protected:

      // stores the new vertex once created
      pVertex newVertex;

  private:

    virtual bool checkConstraints() const = 0;
    virtual bool checkGeometry() = 0;
    virtual bool evaluateShapes() = 0;
    virtual bool evaluateShapes2D() = 0;
    virtual void evaluateLengths() const = 0;

  };

  inline edgeSplitOp::edgeSplitOp(pMesh _mesh, DiscreteSF * _sf) : MAdOperatorBase(_mesh,_sf), newVertex(NULL)
  {
    
  }
  
  inline edgeSplitOp::edgeSplitOp(const edgeSplitOp & _es) : MAdOperatorBase(_es), newVertex(NULL)
  {
    
  }
  inline edgeSplitOp::~edgeSplitOp()
  {
  }
}

#endif
