// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_VERTEXMOVEOP
#define _H_VERTEXMOVEOP

#include "MAdOperatorBase.h"
#include "MobileObject.h"
#include "DistanceFunction.h"

#include <set>


namespace MAd {
  
  // -------------------------------------------------------------------
  class vertexMoveOp : public MAdOperatorBase
  {
  public:

    vertexMoveOp(pMesh, DiscreteSF *, bool _fixBndry=true);
    vertexMoveOp(const vertexMoveOp &);
    virtual ~vertexMoveOp() {};

    operationType type() const { return MAd_VERTEXMOVE; }

    virtual void setFixedBndry(bool) = 0;

    virtual void resetDisplacements() = 0;
    virtual void addVDisplacement(pVertex, double[3]) = 0;
    virtual void addVDisplacement(vDisplacement&) = 0;
    virtual void setDisplacement(pVertex, double[3]) = 0;
    virtual void setPosition(pVertex, double[3]) = 0;
    virtual void setPositionToOptimal(pVertex) = 0;

    virtual bool computeOptimalLocation(pVertex, double[3]) const = 0;

    virtual void getAffectedEdges(pPList *) const = 0;
    virtual void getCavity(pPList *) const = 0;

    virtual void apply() = 0;
    virtual bool move (std::set<vDisplacement,vDisplacementLess>&, double factor=1.0) = 0;
    virtual bool move (std::multiset<vDisplacement,vDisplacementLess>&, double factor=1.0) = 0;
    virtual bool move (pVertex, double[3]) = 0;
    virtual bool move (vDisplacement&) = 0;
    virtual bool operate () = 0;

  private:

    virtual bool checkConstraints() const = 0;
    virtual bool checkGeometry() = 0;
    virtual bool evaluateShapes() = 0;
    virtual bool evaluateShapes2D() = 0;
    virtual void evaluateLengths() const = 0;

  };

  // -------------------------------------------------------------------
  inline vertexMoveOp::vertexMoveOp(const vertexMoveOp & _vm):
    MAdOperatorBase(_vm) 
  {
  }

  // -------------------------------------------------------------------
  inline vertexMoveOp::vertexMoveOp(pMesh _m, DiscreteSF * _sf, 
                                    bool _fixBndry): 
    MAdOperatorBase(_m,_sf)
  {
    
  }
  // -------------------------------------------------------------------

}

#endif
