// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include "GammaEvaluator.h"
#include "MeshSizeBase.h"
#include "MAdDefines.h"

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
using std::string;

namespace MAd {

  // -------------------------------------------------------------------
  gammaEvaluator::gammaEvaluator(const DiscreteSF * f): 
    elementEvaluatorBase(f)
  {}

  // -------------------------------------------------------------------
  int gammaEvaluator::F_shape(const pFace face, const double normal[3],
                              double * shape) const
  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,"not implemented");
    return 0;
  }
  
  // -------------------------------------------------------------------
  int gammaEvaluator::F_shapeWithDisp(const pFace face, const double normal[3],
                                      double disp[3][3], double * shape) const
  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,"not implemented");
    return 0;
  }
  
  // -------------------------------------------------------------------
  int gammaEvaluator::XYZ_F_shape(const double xyz[3][3], const pMSize pS[3], 
                                  const double nor[3], double * shape) const
  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,"not implemented");
    return 0;
  }
  
  // -------------------------------------------------------------------
  int gammaEvaluator::XYZ_F_shape(const double xyz[3][3], const pMSize pS, 
                                  const double nor[3], double * shape) const
  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,"not implemented");
    return 0;
  }

  // -------------------------------------------------------------------
  int gammaEvaluator::R_shape(const pRegion rgn, double * shape) const
  {
    double rxyz[4][3];
    pMSize pS[4];
  
    void *iter=0;
    int i=0;
    pPList verts=R_vertices(rgn);
    while( pVertex vt=(pVertex)PList_next(verts,&iter) ) {
      pS[i]=sizeField->findSize(vt);
      V_coord(vt,rxyz[i++]);
    }  
    PList_delete(verts);

    return XYZ_R_shape(rxyz,pS,shape);
  }

  // -------------------------------------------------------------------
  int gammaEvaluator::R_shapeWithDisp(const pRegion rgn, double disp[4][3], 
                                      double * shape) const
  {
    double rxyz[4][3];
    pMSize pS[4];
  
    void *iter=0;
    int i=0;
    pPList verts=R_vertices(rgn);
    while( pVertex vt=(pVertex)PList_next(verts,&iter) ) {
      pS[i]=sizeField->findSize(vt);
      V_coord(vt,rxyz[i++]);
    }  
    PList_delete(verts);

    for (int j=0; j<4; j++) for (int k=0; k<3; k++) rxyz[j][k] += disp[j][k];

    return XYZ_R_shape(rxyz,pS,shape);
  }

  // -------------------------------------------------------------------
  int gammaEvaluator::XYZ_R_shape(const double xyz[4][3], const pMSize pS[4], 
                                  double * shape) const
  {
    if( sizeField->getType() == NULLSFIELD ) {
      if( ! XYZ_R_shape(xyz,pS[0],shape) )
        { *shape=0.; return 0; }
      return 1;
    }

    // use central size
    pMSize pSZ = MS_interpolate(pS[0], pS[1], pS[2], pS[3], 0.25, 0.25, 0.25);

    // use minimal size
//     pMSize pSZ;
//     double hmin = MAdBIG;
//     for(int i=0; i<4; i++) {
//       if( pS[i] ) {
//         double s = pS[i]->minSize();
//         if( s <  hmin ) { hmin = s; pSZ = pS[i]; }
//       }
//     }
//     assert( hmin != MAdBIG );

    int res = XYZ_R_shape(xyz,pSZ,shape);
    if ( pSZ ) delete pSZ;
    return res;
  }

  // -------------------------------------------------------------------
  int gammaEvaluator::XYZ_R_shape(const double xyz[4][3], const pMSize pS, 
                                  double * shape) const
  {
    double vol = sizeField->SF_XYZ_volume(xyz,pS);
    if ( vol < 0. ) { *shape = 0.; return 0; }

    //    const double * face[3] = {xyz[0],xyz[1],xyz[2]};
    double face[3][3];
    for (int i=0; i<3; i++) { face[0][i] = xyz[0][i]; face[1][i] = xyz[1][i]; face[2][i] = xyz[2][i]; }
    double s1 = sqrt(sizeField->SF_XYZ_areaSq(face,pS,NULL));
    for (int i=0; i<3; i++) { face[0][i] = xyz[0][i]; face[1][i] = xyz[1][i]; face[2][i] = xyz[3][i]; }
    double s2 = sqrt(sizeField->SF_XYZ_areaSq(face,pS,NULL));
    for (int i=0; i<3; i++) { face[0][i] = xyz[0][i]; face[1][i] = xyz[3][i]; face[2][i] = xyz[2][i]; }
    double s3 = sqrt(sizeField->SF_XYZ_areaSq(face,pS,NULL));
    for (int i=0; i<3; i++) { face[0][i] = xyz[3][i]; face[1][i] = xyz[1][i]; face[2][i] = xyz[2][i]; }
    double s4 = sqrt(sizeField->SF_XYZ_areaSq(face,pS,NULL));

    double rhoin = 3. * fabs(vol) / (s1 + s2 + s3 + s4);
  
    double maxL = sizeField->SF_XYZ_lengthSq(xyz[1],xyz[0],pS);
    maxL = std::max(maxL, sizeField->SF_XYZ_lengthSq(xyz[2],xyz[0],pS));
    maxL = std::max(maxL, sizeField->SF_XYZ_lengthSq(xyz[3],xyz[0],pS));
    maxL = std::max(maxL, sizeField->SF_XYZ_lengthSq(xyz[3],xyz[1],pS));
    maxL = std::max(maxL, sizeField->SF_XYZ_lengthSq(xyz[3],xyz[2],pS));
    maxL = std::max(maxL, sizeField->SF_XYZ_lengthSq(xyz[1],xyz[2],pS));

    *shape = 2. * sqrt(6.) * rhoin / sqrt(maxL);

    if(*shape < MAdTOL) return 0;
    else if (*shape > (1.+MAdTOL) ) *shape = 1.;

    return 1;
  }

  // -------------------------------------------------------------------

}
