// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_GEOMATCHER
#define _H_GEOMATCHER

#include "MAdElasticityOp.h"
#include "LinearSliverFaceHandler.h"
#include "LinearSliverRegionHandler.h"
#include <list>

namespace MAd {

  class IMeshAdapter;

  // -------------------------------------------------------------------
  class geoMatcher: public MAdElasticityOp
  {

  public:

    geoMatcher(pMesh, IMeshAdapter *);
    geoMatcher(const geoMatcher&);
    ~geoMatcher();

    void setForceRelocation(bool _force) { force = _force; }
    void setStrictChecking (bool sc) { strictChecking = sc; }

    void setSliverFaceHandler(sliverFaceHandler * s) { sliverFOp = s; }
    void setSliverRegionHandler(sliverRegionHandler * s) { sliverROp = s; }

    bool snap();

    void reportFailure(double ratio) { failures.push_back(ratio); }
    void printFailures(std::ostream&) const;

  private:

    bool force; // if the nodes are snapped without any check nor repositioning inside the domain
    bool strictChecking; // true: snap() returns true only if we the full snapping is done

    std::list<double> failures;

    IMeshAdapter * adapter;

    sliverFaceHandler    *  sliverFOp;
    sliverRegionHandler  *  sliverROp;

  };

  // -------------------------------------------------------------------

}

#endif
