// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_IMOBILEOBJECT
#define _H_IMOBILEOBJECT

#include "MAdExport.h"
#include "MeshDataBaseInterface.h"

#include <vector>
#include <string>

namespace MAd {

    class IMeshAdapter;
    class ISizeField;

    // ----------------------------------------------------------------------
    enum velocityFormulation {
        NOFORMULATION,
        PARSED,
        TRANSLATION,
        FULLRANDOM,
        RANDOM_TRANSLATION,
        ISOMETRY,
        ROTATION,
        SHEAR_YZ,
        SHEAR_ZX,
        SHEAR_XY,
        BENCH2,
        CROSS_EXPANSION_WITH_ROTATION
    };

    // ----------------------------------------------------------------------
    class MAD_EXPORT IMobileObject
    {
    public:

        static IMobileObject * BuildMobileObject(pMesh mesh, const char * name = "");

        virtual ~IMobileObject() {};

        virtual void setName(const char * nm) = 0;
        virtual void setDxKinematics(std::vector<std::string> _Vstr) = 0;
        virtual void setVKinematics(velocityFormulation type, double V[3], double C[3], std::vector<std::string> _Vstr) = 0;
        virtual void addLocalSField(ISizeField* lsf) = 0;
        virtual void addGEntity(int type, int tag) = 0;
    };


    // ----------------------------------------------------------------------
    class MAD_EXPORT IMobileObjectSet
    {
    public:

        static IMobileObjectSet * BuildMobileObjectSet();

        virtual ~IMobileObjectSet() {};

        // move objects and reposition volume nodes. Needs the elastic operator. 
        virtual void setupElasticRepositioning(pMesh mesh, double t, double dt,
            double qualThr = 0., double chi = -1,
            bool meshIsCavity = true, int cavityThickness = 3) = 0;

        // Advances the relocation as far as possible
        // Returns:
        //    - 0: no relocation possible
        //    - 1: advanced but not to the final position
        //    - 2: reached the full prescribed relocation
        virtual int reposition(double * ratio) = 0;

        virtual bool moveAndReposition(pMesh mesh, double t, double dt, bool subAdaptation,
            double qualityThreshold, double chi, bool meshIsCavity,
            int cavityThickness, IMeshAdapter * ma) = 0;

        // empty or fill the set
        virtual void clear() = 0;
        virtual void insert(IMobileObject*) = 0;
        virtual int size() const = 0;
    };

    // ----------------------------------------------------------------------

}

#endif
