// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include "LaplaceSmoothingOp.h"
#include "LinearVertexMoveOp.h"
#include "MeshParametersManager.h"
#include "MathUtils.h"

#include <vector>
using std::vector;
#include <math.h>

namespace MAd {

  // -------------------------------------------------------------------
  LaplaceSmoothingOp::LaplaceSmoothingOp(pMesh m, DiscreteSF * sf): 
    nodesRepositioningOp(m,sf)
  {}

  // -------------------------------------------------------------------
  int LaplaceSmoothingOp::run(double * L2Norm)
  {
    return runOptimal(L2Norm);
  }

  // -------------------------------------------------------------------
  int LaplaceSmoothingOp::runOptimal(double * L2Norm)
  {
    *L2Norm = 0.0;

    LinearvertexMoveOp * vMove = new LinearvertexMoveOp(mesh,sizeField,true);

    VIter vit = M_vertexIter(mesh);
    while( pVertex pv = VIter_next(vit) ) {

      if ( V_whatInType(pv) != dim ) continue;

      double ori[3];
      V_coord(pv,ori);

      double opt[3];
      if (vMove->computeOptimalLocation(pv, opt)) vMove->setPosition(pv, opt);

      double worst;
      if( vMove->evaluate(&worst) ) {
        vMove->apply();
        double disp[3];
        diffVec(ori,opt,disp);
        *L2Norm += dotProd(disp,disp);
      }

      vMove->resetDisplacements();
    }

    return 1;
  }

  // -------------------------------------------------------------------
  int LaplaceSmoothingOp::runFast(double * L2Norm)
  {
    *L2Norm = 0.0;

    LinearvertexMoveOp * vMove = new LinearvertexMoveOp(mesh,sizeField,true);

    VIter vit = M_vertexIter(mesh);
    while( pVertex pv = VIter_next(vit) ) {

      if ( V_whatInType(pv) != dim ) continue;

      double ori[3];
      V_coord(pv,ori);

      double opt[3];
      V_cavityCenter(pv,opt);
      vMove->setPosition(pv, opt);

      double worst;
      if( vMove->evaluate(&worst) ) {
        vMove->apply();
        double disp[3];
        diffVec(ori,opt,disp);
        *L2Norm += dotProd(disp,disp);
      }

      vMove->resetDisplacements();
    }

    return 1;
  }

  // -------------------------------------------------------------------

}

