// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_ANALYTICALSFIELD
#define _H_ANALYTICALSFIELD

#include "MAdExport.h"
#include "ISizeField.h"
#include "SizeFieldBase.h"

#include <vector>

namespace MAd {

  class MAdStringFieldEvaluator;

  // -------------------------------------------------------------------
  class AnalyticalSField: public SizeFieldBase, public IAnalyticalSizeField
  {
  public:

    AnalyticalSField();
    AnalyticalSField(const char * size);
    AnalyticalSField(std::vector<std::string>,std::vector<std::string>,
                     std::vector<std::string>,std::vector<std::string>);
    AnalyticalSField(sizeFunction);
    ~AnalyticalSField();

  public:

    sizeFieldType getType() const { return ANALYTICALSFIELD; }
    virtual const char * getName() const { return SizeFieldBase::getName(); } // avoiding VS buggy warnings
    void describe() const;

    void scale(double);

    // set the size
    void setSize(sizeFunction);
    void setSize(const char * size);
    void setSize(std::vector<std::string>,std::vector<std::string>,
                 std::vector<std::string>,std::vector<std::string>);

    // get the size at a location (allocates memory!)
    MeshSizeBase * getSize(const pVertex) const;
    MeshSizeBase * getSizeOnEntity(const pEntity, const double[3]) const;

    // edge length (squared)
    double SF_VV_lengthSq(const pVertex, const pVertex) const;
    double SF_XYZ_lengthSq(const double[3], const double[3],
                           const pMSize, const pMSize=NULL) const;

    // face area (squared)
    double SF_F_areaSq(const pFace) const;
    double SF_XYZ_areaSq(const double[3][3], const pMSize,
                         const double[3]) const;

    // region volume
    double SF_R_volume(const pRegion) const;
    double SF_XYZ_volume(const double[4][3], const pMSize) const;

    // center and its associated size
    double SF_E_center(const pEdge, double[3], double * reducSq, pMSize *) const;
    double SF_VV_center(const pVertex, const pVertex,
                        double[3], double * reducSq, pMSize *) const;

    // redeclaration to avoid VS buggy warnings
    virtual void printPosIsotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosIsotropic(m, fileName); }
    virtual void printPosAnisotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosAnisotropic(m, fileName); }

  private:

    // description of the sizes if we use functions
    sizeFunction sFct;

    // description of the sizes if we use strings
    bool isotropic;
    std::string h0, h1, h2;
    std::vector<std::string> e0, e1, e2;
    MAdStringFieldEvaluator * evaluator;

  private:

    pMSize eval(const double[3]) const;
  };

}

// -------------------------------------------------------------------
#endif
