// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_BACKGROUNDSF
#define _H_BACKGROUNDSF

#include "MAdExport.h"
#include "SizeFieldBase.h"

namespace MAd {

  // -------------------------------------------------------------------
  class BackgroundSF: public SizeFieldBase, public IBackgroundSizeField
  {
  public:

    BackgroundSF(std::string _name="");
    ~BackgroundSF();

    void loadData(const char * fileName);

  public:

    sizeFieldType getType() const { return BACKGROUNDSFIELD; }
    virtual const char * getName() const { return SizeFieldBase::getName(); } // avoiding VS buggy warnings

    void scale(double) {throw;}

    // get the size at a location (allocate space!)
    pMSize getSize(const pVertex) const;
    pMSize getSizeOnEntity(const pEntity, const double[3]) const { throw; return 0; }

    // edge length (squared)
    double SF_VV_lengthSq(const pVertex, const pVertex) const { throw; return -1.; }
    double SF_XYZ_lengthSq(const double[3], const double[3],
                           const pMSize, const pMSize=NULL) const { throw; return -1.; }

    // face area (squared)
    double SF_F_areaSq(const pFace) const { throw; return -1.; }
    double SF_XYZ_areaSq(const double[3][3], const pMSize,
                         const double[3]) const { throw; return -1.; }

    // region volume
    double SF_R_volume(const pRegion) const { throw; return -1.; }
    double SF_XYZ_volume(const double[4][3], const pMSize) const { throw; return -1.; }

    // center and its associated size
    double SF_E_center(const pEdge, double[3], double * reducSq, pMSize *) const { throw; return -1.; }
    double SF_VV_center(const pVertex, const pVertex,
                        double[3], double * reducSq, pMSize *) const { throw; return -1.; }

    // redeclaration to avoid VS buggy warnings
    virtual void printPosIsotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosIsotropic(m, fileName); }
    virtual void printPosAnisotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosAnisotropic(m, fileName); }

  private:

    void setSize(pEntity, pMSize);
    void setSize(pEntity, double);

    void redistributeToVertices();

    pGModel bgModel;
    pMesh bgMesh;
    pMeshDataId pMSizeFieldId;
  };

}

// -------------------------------------------------------------------
#endif
