// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_BOXSIZEFIELD
#define _H_BOXSIZEFIELD

#include "SizeFieldBase.h"

namespace MAd {

  // -------------------------------------------------------------------
  class BoxSizeField : public SizeFieldBase, public IBoxSizeField
  {
  public:

    BoxSizeField();
    ~BoxSizeField();

    sizeFieldType getType() const { return BOXSFIELD; }
    virtual const char * getName() const { return SizeFieldBase::getName(); } // avoiding VS buggy warnings

    void scale(double);

    // set the size
    void setBox(double _box[6]) { std::copy(_box, _box + 6, box); }
    void setSizeIn(double size) { sizeIn = size; }
    void setSizeOut(double size) { sizeOut = size; }
    void setGradient(bool useGrad, double grad) { useGradient = useGrad;  gradient = grad; }

    // get the size at a location (allocates memory!)
    MeshSizeBase * getSize(const pVertex) const;
    MeshSizeBase * getSizeOnEntity(const pEntity, const double[3]) const;

    // edge length (squared)
    double SF_VV_lengthSq(const pVertex, const pVertex) const;
    double SF_XYZ_lengthSq(const double[3], const double[3],
                           const pMSize, const pMSize = NULL) const;

    // face area (squared)
    double SF_F_areaSq(const pFace) const;
    double SF_XYZ_areaSq(const double[3][3], const pMSize,
                         const double[3]) const;

    // region volume
    double SF_R_volume(const pRegion) const;
    double SF_XYZ_volume(const double[4][3], const pMSize) const;

    // center and its associated size
    double SF_E_center(const pEdge, double[3], double * reducSq, pMSize *) const;
    double SF_VV_center(const pVertex, const pVertex,
                        double[3], double * reducSq, pMSize *) const;

    // redeclaration to avoid VS buggy warnings
    virtual void printPosIsotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosIsotropic(m, fileName); }
    virtual void printPosAnisotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosAnisotropic(m, fileName); }
        
  private:
    pMSize eval(const double xyz[3]) const;

  private:
    double box[6];
    double sizeIn, sizeOut;
    bool useGradient;
    double gradient;
  };

}

// -------------------------------------------------------------------
#endif
