// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_DISCRETESF
#define _H_DISCRETESF

#include "MAdExport.h"
#include "SizeFieldBase.h"

#include <string>

namespace MAd {

  // -------------------------------------------------------------------
  enum DiscreteSFType {
    UNKNOWN_DSFTYPE,
    VERTEX_P1_DSFTYPE
  };

  // -------------------------------------------------------------------
  class DiscreteSF : public virtual SizeFieldBase
  {
  public:

    DiscreteSF(pMesh, std::string name="");
    virtual ~DiscreteSF();

    sizeFieldType getType() const { return DISCRETESFIELD; }
    virtual const char * getName() const { return SizeFieldBase::getName(); } // avoiding VS buggy warnings
    virtual DiscreteSFType discretization() const = 0;
    virtual pMesh getMesh() const { return mesh; }

    // delete sizes
    virtual void cleanUp() = 0;
    virtual void deleteSize(pEntity);

    // Intersect with another size field
    virtual void intersect(const ISizeField *) = 0;
    virtual void intersect(pVertex pv, const ISizeField *) = 0;

    // smooth the size field
    virtual void smooth(double) = 0;

    // set a size at all vertices 
    virtual void setCurrentSize() = 0;
    virtual void setCurvatureSize(bool aniso, 
                                  double alpha=2., // prescribe 2*PI*alpha edges around a circle
                                  double hMin=1.e-4) = 0; // minimal size
    virtual void setAllVSizes(pMSize) = 0;
    virtual void setAllVSizes(double[3][3], double[3]) = 0;
    virtual void setAllVSizes(double) = 0;
    virtual void scale(double) = 0;

    // set the size at a vertex 
    void setSize(pEntity, double[3][3], double[3]);
    void setSize(pEntity, pMSize);
    void setSize(pEntity, double);
    void setSize(int, double);

    // get the size at a location (allocates memory!)
    virtual MeshSizeBase * getSize(const pVertex) const = 0;
    virtual MeshSizeBase * getSizeOnEntity(const pEntity, const double[3]) const = 0;

    // get the size at a vertex (do not allocate space)
    virtual pMSize findSize(const pVertex) const = 0;
    virtual pMSize findSize(const pVertex) = 0;

    // edge length (squared)
    virtual double SF_VV_lengthSq(const pVertex, const pVertex) const = 0;
    virtual double SF_XYZ_lengthSq(const double[3], const double[3],
                                   const pMSize, const pMSize=NULL) const = 0;

    // face area (squared)
    virtual double SF_F_areaSq(const pFace) const = 0;
    virtual double SF_XYZ_areaSq(const double[3][3], const pMSize,
                                 const double[3]) const = 0;
    
    // region volume
    virtual double SF_R_volume(const pRegion) const = 0;
    virtual double SF_XYZ_volume(const double[4][3], const pMSize) const = 0;

    // center and its associated size
    virtual double SF_E_center(const pEdge, double[3], double *reducSq, pMSize *) const = 0;
    virtual double SF_VV_center(const pVertex, const pVertex,
                                double[3], double *reducSq, pMSize *) const = 0;

    // interpolate the size at parametric location u
    virtual pMSize SF_E_interpolate(const pEdge edge, double u) const = 0;

    // redeclaration to avoid VS buggy warnings
    virtual void printPosIsotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosIsotropic(m, fileName); }
    virtual void printPosAnisotropic(const pMesh m, const char * fileName) { SizeFieldBase::printPosAnisotropic(m, fileName); }

  protected:

    pMesh mesh;
    int dim;
    pMeshDataId pMSizeFieldId;

  };

}

// -------------------------------------------------------------------

#endif
