// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_SIZEFIELDBASE
#define _H_SIZEFIELDBASE

#include "MAdExport.h"
#include "MeshDataBaseInterface.h"
#include "ISizeField.h"

namespace MAd {

  // -------------------------------------------------------------------
  typedef class SizeFieldBase * pSField;

  // -------------------------------------------------------------------
  class SizeFieldBase : public virtual ISizeField
  {
  public:
  
    SizeFieldBase(std::string _name=""): name(_name) {};
    virtual ~SizeFieldBase() {};
  
  public:
  
    virtual sizeFieldType getType() const = 0;
    virtual const char * getName() const { return name.c_str(); }
  
    virtual void scale(double) = 0;
  
  public:

    // edge length (squared)
    virtual double SF_E_lengthSq(const pEdge) const;
    virtual double SF_VV_lengthSq(const pVertex,const pVertex) const = 0;
    virtual double SF_XYZ_lengthSq(const double[3], const double[3],
                                   const pMSize, const pMSize=NULL) const = 0;

    // face area (squared)
    virtual double SF_F_areaSq(const pFace) const = 0;
    virtual double SF_XYZ_areaSq(const double[3][3], const pMSize,
                                 const double[3]) const = 0;

    // region volume
    virtual double SF_R_volume(const pRegion) const = 0;
    virtual double SF_XYZ_volume(const double[4][3], const pMSize) const = 0;

    // center and its associated size
    virtual double SF_E_center(const pEdge, double[3], double * reducSq, pMSize *) const = 0;
    virtual double SF_VV_center(const pVertex, const pVertex,
                                double[3], double * reducSq, pMSize *) const = 0;
    virtual double SF_XYZ_center(const double[2][3], const pMSize[2],
                                 double[3], double * reducSq, pMSize *) const;

  public:

    // get the size at a location
    // --- Memory is allocated ! ---
      virtual MeshSizeBase * getSize(const pVertex) const = 0;
      virtual MeshSizeBase * getSizeOnEntity(const pEntity, const double[3]) const = 0;

  public:

    // visualisation of the isotropic size in a .pos file
    void printPosIsotropic  (const pMesh mesh, const char * fileName);
    // visualisation of the anisotropic size in a .pos file
    void printPosAnisotropic(const pMesh mesh, const char * fileName);

  private:

    std::string name;

  };

}

// -------------------------------------------------------------------

#endif
