// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_SIZEFIELDMANAGER
#define _H_SIZEFIELDMANAGER

#include "SizeFieldBase.h"
#include "PWLinearSField.h"
#include "LocalSizeField.h"
#include "AnalyticalSField.h"
#include "BackgroundSF.h"
#include "BoxSizeField.h"
#include "DistanceFunction.h"

#include <set>

namespace MAd {

  // -------------------------------------------------------------------
  // This class lists, intersects and updates all size fields
  // -------------------------------------------------------------------

  // -------------------------------------------------------------------

  class SizeFieldManager {

  public:

    SizeFieldManager(pMesh m, pSField psf=NULL);
    ~SizeFieldManager();

  public:

    void setMesh(pMesh m);
    void setSmoothing(bool enable, double maxGrad) 
    {
      smooth = enable;
      maxGradient = maxGrad;
    }

    void intersect(const pSField sf);

    void addSizeField(pSField sf);

    void update();
    void update(pVertex pv);

    DiscreteSF* getSizeField()             { return mainSF; }
    const DiscreteSF* getSizeField() const { return mainSF; }

    std::set<LocalSizeField *> getLocalSizeFields() const { return locals; }

  private:

    void addPWLinearSF(PWLSField * sf);
    void addAnalyticalSF(AnalyticalSField * sf);
    void addLocalSF(LocalSizeField * sf);
    void addBGSF(BackgroundSF * bg);
    void addBoxSizeField(BoxSizeField * bg);

  private:

    DiscreteSF* mainSF;

    std::set<PWLSField *>        linears;
    std::set<AnalyticalSField *> analyticals;
    std::set<LocalSizeField *>   locals;
    std::set<BackgroundSF *>     bgs;
    std::set<BoxSizeField *>     boxs;

    pMesh mesh;

    double localTime;

    bool smooth;
    double maxGradient;
  };

  // -------------------------------------------------------------------

}

#endif
