// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_DISTANCETOGENT
#define _H_DISTANCETOGENT

#include "ModelInterface.h"
#include "MeshDataBaseInterface.h"
#include "DistanceFunction.h"
#include <set>
#include <string>

// Computes the distance from a point to a wall given by a set of pGEntity.
// This distance is either:
//    - the distance to the closest node of the wall (toFaces=false)
//    - the distance to the closest face(3D) or edge(2D) of the wall (toFaces=true)
// The latter choice is more expensive but more accurate.

// The distance from a point xyz can be computed (using the class functions), 
// or all distances from the nodes of a mesh can be printed in a file 
// (format: 'X Y Z d\n'), using the class functions or the function here below.

namespace MAd {

  void M_distanceToWalls(std::string outName, std::string mshName, bool toFaces=false);
  void M_distanceToWalls(std::string outName, pMesh mesh, bool toFaces=false);

  class DistanceToGEntity {

  public:
  
    DistanceToGEntity(): treeIsComputed(false), distFct(NULL) {}
    ~DistanceToGEntity() { if (distFct) delete distFct; }
  
    void addGEntity(pGEntity pge) { entities.insert(pge); }
    void addGEntity(pGModel model, int dim, int tag) 
    { entities.insert( GM_entityByTag(model,dim,tag) ); }
  
    void compute(pMesh m, bool toFaces=false);
  
    double getDistance(double xyz[3]) const;

    void printDistanceToAllVertices(std::string txtName, pMesh mesh) const;
    void posDistanceToAllVertices  (std::string posName, pMesh mesh) const;

  private:
  
    bool treeIsComputed;
    std::set<pGEntity> entities;
    distanceFunction * distFct;
  };

}

#endif
