// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include <iostream>
#include <string>
#include <stdlib.h>

#include "MAdLib.h"

#ifdef PARALLEL
#include "mpi.h"
#endif

using namespace MAd;
using std::string;
using std::cout;

// ----------------------------------------------------------------------
int main(int argc, char* argv[]) {

#ifdef PARALLEL 
  MPI_Init(&argc, &argv);
#endif

  // Check input
  // ------------
  if ( argc != 3 && argc != 2  ) {
    printf("Error: usage: \'executable mshFile [geoFile]\'\n");
    exit(1);
  }
  string meshFile = argv[1];

  // Build tools
  // ------------
  printf ("Checking mesh %s...\n\n",meshFile.c_str());

  // --- Reading model ---
  pGModel model = 0;
  GM_create(&model,"theModel");
  
  if ( argc == 3 ) {
    string geoFile  = argv[2];
    GM_read(model, geoFile.c_str());
  }
  else {
    GM_readFromMSH(model, meshFile.c_str());
  }

  pMesh mesh = M_new(model);
  M_load(mesh,meshFile.c_str());

  MeshStatus status = VALID;
  int return_value;
  
  if ( !checkMesh(mesh, CHECK_ALL, 1, std::cout, &status) )
    {
      printf("\nThe mesh is invalid, status = %d\n\n",status);
      return_value = 1;
    }
  else
    {
      printf("\nThe mesh is valid, status = %d\n\n",status);
      return_value = 0;
    }


#ifdef PARALLEL 
  MPI_Finalize();
#endif
  
  return return_value;
}

// ----------------------------------------------------------------------
