// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifdef PARALLEL
#include "mpi.h"
#include "autopack.h"
#endif

#include "MAdConfig.h"
#include "MAdLib.h"
#include "MAdMessage.h"
#include "MAdResourceManager.h"
#include "MAd_InterpolantFactory.hpp"

namespace MAd {

  // ---------------------------------------------------------------------------
  
  void MD_initialize(int * argc, char** argv[])
  {
    MAdResourceManagerSgl::instance().initialize();
    MAdResourceManagerSgl::instance().setArguments(argc,argv);
    MAd_InterpolantFactorySgl::instance().initialize();
    
#ifdef PARALLEL
    int flag;
    MPI_Initialized(&flag);
    if( flag == 0 ) MPI_Init(argc,argv);
#endif
  }
  
  // ---------------------------------------------------------------------------  
  
#ifdef PARALLEL
  void MD_setCommunicator(MPI_Comm comm) {
    MAdResourceManagerSgl::instance().setCommunicator(comm);
    int* argc(NULL);
    char*** argv(NULL);
    MAdResourceManagerSgl::instance().getArguments(argc,argv);
    AP_init(argc,argv,comm);
    AP_setparam(1000,1,10,100);
  }
#endif

  // ---------------------------------------------------------------------------

  void MD_finalize()
  {

#ifdef PARALLEL
    // MPI_Barrier(MAdResourceManagerSgl::instance().getCommunicator());
    // AP_finalize();
#endif
    
    MAdResourceManagerSgl::instance().finalize();
    
  }

  // ---------------------------------------------------------------------------

  const char* MD_getVersion() {
    return MADLIB_PACKAGE_VERSION;
  }
  const char* MD_getRevision() {
    return MADLIB_SVN_REVISION;
  }

  // ---------------------------------------------------------------------------

  void setOutputWriter(WriterFunction writer, void * data)
  {
      MAdMsgSgl::instance().setOutWriter(writer, data);
  }
  void setErrorWriter(WriterFunction writer, void * data)
  {
      MAdMsgSgl::instance().setErrWriter(writer, data);
  }
  void resetWriters()
  {
      MAdMsgSgl::instance().resetWriters();
  }
  void setMessagePrefix(const char * prefix)
  {
      MAdMsgSgl::instance().setMessagePrefix(prefix);
  }

  // ---------------------------------------------------------------------------
}
