// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_MAD_MADLIB
#define _H_MAD_MADLIB

#include "MAdExport.h"

#ifdef PARALLEL
#include <mpi.h>
#endif

namespace MAd {

  //!\brief initialize all internal objects except for parallel data
  MAD_EXPORT void MD_initialize(int * argc, char** argv[]);

  //!\brief clean the MAdLib data base and finalize all messages, monitors, ...
  MAD_EXPORT void MD_finalize();

#ifdef PARALLEL
  //!\brief set the appropriate communicator and initiate autopack
  MAD_EXPORT void MD_setCommunicator(MPI_Comm c);
#endif

  //!\brief get version number of the current library
  MAD_EXPORT const char* MD_getVersion();

  //!\brief get revision number of the current library
  MAD_EXPORT const char* MD_getRevision();

  // This is a writer function that can be registered as a callback in the methods below.
  // MAdLib will use it to send messages. It is up to the user to define where the message 
  // should go and how it should be written.
  // The first argument is the message. It includes any prefix given to 'setMessagePrefix'.
  // The second is the pointer given as argument to 'setOutputWriter' or 'setErrorWriter'.
  // That pointer can be a pointer to any object, but it was added with the idea that it would
  // provide a destination for the messages to the WriterFunction. For instance, it could be a 
  // pointer to a stream or to a FILE object. A caller of 'setErrorWriter' can register
  // a WriterFunction callback and pass a FILE* pointer for the argument 'data'. In that case 
  // the WriterFunction can cast the data pointer to a FILE* and write the message to it.
  // It is up to the caller to do anything else with the data pointer.
  typedef void(*WriterFunction)(const char * message, void * data);

  //!\brief set the function to be used by MAdLib to write general information messages, excluding warnings/errors
  MAD_EXPORT void setOutputWriter(WriterFunction writer, void * data = 0);

  //!\brief set the function to be used by MAdLib to write warning/error messages
  MAD_EXPORT void setErrorWriter(WriterFunction writer, void * data = 0);

  //!\brief reset the output/error writers to std::cout/std::cerr respectively
  MAD_EXPORT void resetWriters();

  //!\brief add a prefix to all new lines of the messages and error messages
  MAD_EXPORT void setMessagePrefix(const char * prefix);
}
#endif

