// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_MESSAGE_MAD_
#define _H_MESSAGE_MAD_

#include "MAdLib.h"
#include "MAdSingleton.h"

#include <list>
#include <string>

namespace MAd {

  // -------------------------------------------------------------------
  // abort function
  typedef void (*AbortFunction)(void *);

  // -------------------------------------------------------------------
  class MAdMsg {

  public:

    MAdMsg();
    ~MAdMsg() {}
  
    void resetWriters();

    void registerAbortFct(AbortFunction, void *);

    void write(const std::string& message) const;

    void info   (int line, const char* file, const char* msg, ...) const;
    void warning(int line, const char* file, const char* msg, ...) const;
    void error  (int line, const char* file, const char* msg, ...) const;

    void setOutWriter(WriterFunction writer, void * data);
    void setErrWriter(WriterFunction writer, void * data);

    void setMessagePrefix(const char * _prefix);

  private:

      void appendLineAndFile(std::string * str, int line, const char * file) const;
      void addPrefix(std::string * message) const;

  private:

      WriterFunction outWriter;
      void * outData;
      WriterFunction errWriter;
      void * errData;

      std::string prefix;

    std::list<std::pair<AbortFunction,void*> > abortFcts; 

  };

  // -------------------------------------------------------------------

  typedef MAdSingleton<MAdMsg> MAdMsgSgl;
}

#endif
