// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef _VECTOR3D_H_
#define _VECTOR3D_H_

#include <vector>
//using std::vector;
#include <iostream>
//using std::cerr;
#include <cmath>
//using std::min;
//using std::sqrt;
#include <sstream>
//using std::ostringstream;
#include <string>
//using std::string;
#include <cstring>


//------------------------------------------------------------------------------
class MAd_Vector3D {

public:

  MAd_Vector3D(double = 0.0, double = 0.0, double = 0.0);
  MAd_Vector3D(const double [3]);
  MAd_Vector3D(const double*,size_t);
  MAd_Vector3D(const double [3],const double [3]);
  MAd_Vector3D(const MAd_Vector3D&);
  MAd_Vector3D(const MAd_Vector3D&,const MAd_Vector3D&);
  MAd_Vector3D(const std::vector<double>&);
  
  ~MAd_Vector3D();

  double& operator[](size_t);
  const double& operator[](size_t) const;

  MAd_Vector3D& operator=(double);
  MAd_Vector3D& operator=(const double [3]);
  MAd_Vector3D& operator=(const MAd_Vector3D&);
  MAd_Vector3D& operator=(const std::vector<double>&);
  MAd_Vector3D& operator+=(const MAd_Vector3D&);
  MAd_Vector3D& operator-=(const MAd_Vector3D&);
  MAd_Vector3D& operator*=(double);
  MAd_Vector3D& operator/=(double);
  bool   operator==(const MAd_Vector3D&) const;
  MAd_Vector3D& normalise();

  MAd_Vector3D operator+(const MAd_Vector3D&) const;
  MAd_Vector3D operator-(const MAd_Vector3D&) const;
  MAd_Vector3D operator^(const MAd_Vector3D&) const;
  
  double operator*(const MAd_Vector3D&) const;
  double norm() const;
  double normSq() const;

  int principalComponent() const;

  void computeOrthonormalBasis(MAd_Vector3D&,MAd_Vector3D&,MAd_Vector3D&) const;

  const double* data() const {return v;}
  double *      data()       {return v;}

  std::string print() const {
    std::ostringstream name;
    name << "(" << v[0]  << "," << v[1] << "," << v[2] << ")";
    return name.str();
  }
  
private:

  double v[3];

};

//------------------------------------------------------------------------------

inline 
MAd_Vector3D::MAd_Vector3D(double x, double y, double z) {
  v[0] = x; 
  v[1] = y; 
  v[2] = z; 
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D::MAd_Vector3D(const double x[3]) {
  std::memcpy(v,x,3*sizeof(double));
}

//------------------------------------------------------------------------------

inline
MAd_Vector3D::MAd_Vector3D(const double* c,size_t d) {
  *this = 0.;
  std::memcpy(v,c,std::min(d,(size_t) 3)*sizeof(double));
}
  
//------------------------------------------------------------------------------

inline 
MAd_Vector3D::MAd_Vector3D(const double x1[3],const double x2[3]) { 
  v[0] = x2[0]-x1[0]; 
  v[1] = x2[1]-x1[1]; 
  v[2] = x2[2]-x1[2]; 
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D::MAd_Vector3D(const MAd_Vector3D& v2) { 
  v[0] = v2[0]; 
  v[1] = v2[1]; 
  v[2] = v2[2]; 
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D::MAd_Vector3D(const MAd_Vector3D& v1,const MAd_Vector3D& v2) { 
  v[0] = v2[0]-v1[0]; 
  v[1] = v2[1]-v1[1]; 
  v[2] = v2[2]-v1[2]; 
}


//------------------------------------------------------------------------------

inline
MAd_Vector3D::MAd_Vector3D(const std::vector<double>& v2) {
  
  if (v2.size() > 3) {
    std::cerr << "Too many data (" << v2.size() << ") for vector initiation --> exiting\n";
    throw;
  }
  for (size_t i=0;i<v2.size();i++) v[i] = v2[i];
  for (size_t i=v2.size();i<3;i++) v[i] = 0;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D::~MAd_Vector3D() {}

//------------------------------------------------------------------------------

inline 
double& MAd_Vector3D::operator[](size_t i) { 
  return v[i]; 
}

//------------------------------------------------------------------------------

inline 
const double& MAd_Vector3D::operator[](size_t i) const { 
  return v[i]; 
}

//------------------------------------------------------------------------------

inline 
double MAd_Vector3D::normSq() const { 
  return (v[0]*v[0] +
          v[1]*v[1] +
          v[2]*v[2]); 
}

//------------------------------------------------------------------------------

inline 
double MAd_Vector3D::norm() const { 
  return sqrt(v[0]*v[0] +
              v[1]*v[1] +
              v[2]*v[2]); 
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator=(double v2) {
  v[0] = v2;
  v[1] = v2;
  v[2] = v2;
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator=(const double v2[3]) {
  v[0] = v2[0];
  v[1] = v2[1];
  v[2] = v2[2];
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator=(const MAd_Vector3D& v2) {
  v[0] = v2[0];
  v[1] = v2[1];
  v[2] = v2[2];
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator=(const std::vector<double>& v2) {

  if (v2.size() > 3) {
    std::cerr << "Too many data (" << v2.size() << ") for vector initiation --> exiting\n";
    throw;
  }
 
  for (size_t i=0;i<v2.size();i++) v[i] = v2[i];
  for (size_t i=v2.size();i<3;i++) v[i] = 0;
  
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator+=(const MAd_Vector3D& v2) {
  v[0] += v2[0];
  v[1] += v2[1];
  v[2] += v2[2];
  return *this;
}

//------------------------------------------------------------------------------

inline 
bool MAd_Vector3D::operator==(const MAd_Vector3D& v2) const {
  return (v[0] == v2[0] && v[1] == v2[1] && v[2] == v2[2]);
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator-=(const MAd_Vector3D& v2) {
  v[0] -= v2[0];
  v[1] -= v2[1];
  v[2] -= v2[2];
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D& MAd_Vector3D::operator*=(double cst) {
  v[0] *= cst;
  v[1] *= cst;
  v[2] *= cst;
  return *this;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D MAd_Vector3D::operator+(const MAd_Vector3D& v2) const {
  MAd_Vector3D res;
  res[0] = v[0]+v2[0];
  res[1] = v[1]+v2[1];
  res[2] = v[2]+v2[2];
  return res;
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D MAd_Vector3D::operator-(const MAd_Vector3D& v2) const {
  MAd_Vector3D res;
  res[0] = v[0]-v2[0];
  res[1] = v[1]-v2[1];
  res[2] = v[2]-v2[2];
  return res;
}

//------------------------------------------------------------------------------
// define vector cross product

inline 
MAd_Vector3D MAd_Vector3D::operator^(const MAd_Vector3D& v2) const {

  return MAd_Vector3D(v[1]*v2[2]-v[2]*v2[1],
                      v[2]*v2[0]-v[0]*v2[2], 
                      v[0]*v2[1]-v[1]*v2[0]);
}

//------------------------------------------------------------------------------

inline 
double MAd_Vector3D::operator*(const MAd_Vector3D& v2) const {
  return v[0]*v2[0] + v[1]*v2[1] + v[2]*v2[2];
}

//------------------------------------------------------------------------------

inline 
MAd_Vector3D operator*(double c, const MAd_Vector3D& v2) {
  MAd_Vector3D res;
  res[0] = c*v2[0];
  res[1] = c*v2[1];
  res[2] = c*v2[2];
  return res;
}

//------------------------------------------------------------------------------
inline 
MAd_Vector3D operator*(const MAd_Vector3D& v2,double c) {
  MAd_Vector3D res;
  res[0] = c*v2[0];
  res[1] = c*v2[1];
  res[2] = c*v2[2];
  return res;
}

//------------------------------------------------------------------------------
inline
MAd_Vector3D operator/(double c,const MAd_Vector3D& v2) {
  MAd_Vector3D res;
  double overC = 1./c;
  return v2*overC;
}

//------------------------------------------------------------------------------
inline
MAd_Vector3D operator/(const MAd_Vector3D& v1,double c) {
  MAd_Vector3D res;
  double overC = 1./c;
  return v1*overC;
}


//------------------------------------------------------------------------------
inline
MAd_Vector3D& MAd_Vector3D::operator/=(double c) {
  double overC = 1./c;
  v[0] *= overC;
  v[1] *= overC;
  v[2] *= overC;
  return *this;
}

//------------------------------------------------------------------------------
inline
MAd_Vector3D& MAd_Vector3D::normalise() {
  double overNorm = 1./norm();
  v[0] *= overNorm;
  v[1] *= overNorm;
  v[2] *= overNorm;
  return *this;
}
  
//------------------------------------------------------------------------------
inline 
int MAd_Vector3D::principalComponent() const {

  double maxVal = std::fabs(v[0]);
  int maxIdx = 0;

  for (int j=1;j<=2;j++) {
    if (fabs(v[j]) > maxVal) {maxIdx = j; maxVal = fabs(v[j]);} 
  }

  return v[maxIdx] >= 0 ? maxIdx:-maxIdx;
  
}

//------------------------------------------------------------------------------

inline
MAd_Vector3D max(const MAd_Vector3D& v1,const MAd_Vector3D& v2) {

  return MAd_Vector3D(std::max(v1[0],v2[0]),
                      std::max(v1[1],v2[1]),
                      std::max(v1[2],v2[2]));
}

//------------------------------------------------------------------------------

inline
MAd_Vector3D min(const MAd_Vector3D& v1,const MAd_Vector3D& v2) {

  return MAd_Vector3D(std::min(v1[0],v2[0]),
                      std::min(v1[1],v2[1]),
                      std::min(v1[2],v2[2]));
}

//------------------------------------------------------------------------------

inline 
void MAd_Vector3D::computeOrthonormalBasis(MAd_Vector3D& e1,
                                           MAd_Vector3D& e2,
                                           MAd_Vector3D& e3) const {
  e1 = *this;
  e1.normalise();

  int i1 = e1.principalComponent();
  int i2 = (i1+1)%3;
  int i3 = (i1+2)%3;

  e2[i1] = e1[i2];
  e2[i2] = - e1[i1];
  e2[i3] = 0;
  
  e2.normalise();

  e3 = e1^e2;
}

#endif
