// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// ------------------------------------------------------------------


#include "FunctionFamily.h"

#include <string>
//using std::string;
#include <map>
//using std::pair;
#include <vector>
//using std::vector;

/* --------------------------------------------------------------------------- */
/*! \brief Jacobi polynomials \f$P^{(\alpha,\beta)}_n\f$                       */
/*! Jacobi polynomials are mutually orthogonal on the interval [-1,1] with     
    respect to the weighting function \f$(1-x)^\alpha (1+x)^\beta\f$           */
/*! Abramowitz and Stegun, chapter 22                                          */
/* --------------------------------------------------------------------------- */

class Jacobi : public Hierarchic1DFunctionFamily{
  
public:
  
  Jacobi(double a,double b,int o);
  ~Jacobi();

public:

  std::string name() const;
  size_t      size() const;
  bool        orthogonal() const {return true;}

public:

  void evaluateValues1D     (double x,std::vector<double>& fcn) const;  
  void evaluateDerivatives1D(double x,std::vector<double>& der) const;
  double evaluateWeightFunction1D(double x) const;
  std::pair<double,double> range() const;
  
private: // definition 
 
  double alpha; //!< power of \f$ (1-x) \f$ 
  double beta;  //!< power of \f$ (1+x) \f$ 
  int n;        //!< maximal order 

private:  // auxiliary numbers 
  
  double alphaPlusBeta;
  double a2MinusB2;

};

/* ---------------------------------------------------------------------------- */
