// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#include "Legendre.h"

#include <string>
using std::string;
#include <sstream>
using std::ostringstream;
#include <vector>
using std::vector;
using std::pair;
using std::make_pair;

// -----------------------------------------------------------------------------

Legendre::Legendre(int o):n(o) {}

// -----------------------------------------------------------------------------

Legendre::~Legendre() {}

// -----------------------------------------------------------------------------

string Legendre::name() const {
  ostringstream nn;
  nn << "Legendre polynomials up to order " << n;
  return nn.str();
}

// -----------------------------------------------------------------------------

size_t Legendre::size() const { return n+1; }

// -----------------------------------------------------------------------------

void Legendre::evaluateValues1D(double x,vector<double>& fcn) const {

  fcn.resize(n+1);
  // initial values
  fcn[0] = 1;
  if (n>0) fcn[1] = x;

  // recursive definition
  for (int i=1; i<n; i++){
    
    double ii = (double) i;
    
    double a1i = ii+1.;
    double a2i = 2.*ii+1.;

    fcn[i+1] = a2i/a1i * x * fcn[i] - ii/a1i * fcn[i-1];
    
  }
  
}
// -----------------------------------------------------------------------------

void Legendre::evaluateDerivatives1D(double x,vector<double>& der) const {

  der.resize(n+1);

  vector<double> tmp(size());
  evaluateValues1D(x,tmp);

  der[0] = 0;  
  if (n>0) der[1] = 1;
  
  for (int i=1;i<n;i++) {

    double ii = (double) i;
    
    double a1i = ii+1.;
    double a2i = 2.*ii+1.;
    
    der[i+1] = a2i/a1i * ( tmp[i] + x * der[i] ) - ii/a1i * der[i-1];
    
  }
}
// -----------------------------------------------------------------------------

double Legendre::evaluateWeightFunction1D(double x) const {return 1.;}

// -----------------------------------------------------------------------------

pair<double,double> Legendre::range() const { return make_pair(-1,1);}
