// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef LEGENDREM_H
#define LEGENDREM_H

#include <string>
//using std::string;


#include "FunctionFamily.h"

// --------------------------------------------------------------------------- */
/*! \brief Legendre polynomials \f$P_n = P^{(0,0)}_n \f$                       */
/*! Legendre polynomials are Jacobi functions for \f$\alpha=\beta=0\f$ and hence 
  mutually orthogonal on the interval [-1,1]        */
/*! Abramowitz and Stegun, chapter 22                                          */
// --------------------------------------------------------------------------- */

class Legendre : public Hierarchic1DFunctionFamily {
  
public:
  
  Legendre(int o);
  ~Legendre();

public:

  std::string name()       const;
  size_t size()       const;
  bool   orthogonal() const {return true;}
  
public:

  void evaluateValues1D     (double,std::vector<double>&) const;
  void evaluateDerivatives1D(double,std::vector<double>&) const;
  double evaluateWeightFunction1D(double x) const;
  std::pair<double,double> range() const;
  
private:
 
  int n; //!< maximal order 
  
};

// -----------------------------------------------------------------------------

#endif
