// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADPROXIES__H
#define MADPROXIES__H
 
#include <cstdlib>
#include <iostream>
//using std::cerr;
//using std::endl;

/* ===========================================================================*/
/*! \brief proxy view on a vector                                             */
/* ===========================================================================*/

template <class SCALAR> 
struct mad_vector_view {

public:
  
  mad_vector_view(SCALAR* b,const SCALAR* e,size_t n,size_t l):
    size(n),stride(l),data(b),end(e) {}

  ~mad_vector_view() {}

  size_t size;
  size_t stride;
  SCALAR* const data;
  const SCALAR* const end;
};

/* ===========================================================================*/
/*! \brief proxy view on a constant vector                                    */
/* ===========================================================================*/

template <class SCALAR> 
struct mad_const_vector_view {

public:
  
  mad_const_vector_view(const SCALAR* b,const SCALAR* e,size_t n,size_t l):
    size(n),stride(l),data(b),end(e) {}

  ~mad_const_vector_view() {}

  size_t size;
  size_t stride;
  const SCALAR* const data;
  const SCALAR* const end;
};


/* ===========================================================================*/
/*! \brief View on dense matrix                                               */
/* ===========================================================================*/

template <class SCALAR> class mad_matrix_view {
  
public:
  
  ~mad_matrix_view() {}
  
  mad_matrix_view(SCALAR* b,const SCALAR* e,size_t n1,size_t n2,size_t l):
    data(b),end(e),size1(n1),size2(n2),ld(l) {}
  
  SCALAR* const data;
  const SCALAR* end;
  
  size_t size1;
  size_t size2;
  size_t ld;
};

/* ===========================================================================*/
/*! \brief View on constant dense matrix                                      */
/* ===========================================================================*/

template <class SCALAR> class mad_const_matrix_view {

public:

  mad_const_matrix_view(const SCALAR* b,const SCALAR* e,size_t n1,size_t n2,size_t l):
    data(b),end(e),size1(n1),size2(n2),ld(l) {}

  ~mad_const_matrix_view() {}

  const SCALAR* const data;
  const SCALAR* const end;
  size_t size1;
  size_t size2;
  size_t ld;
};

/* ===========================================================================*/
/*! \brief View on column-major dense matrix                                  */
/* ===========================================================================*/

template <class SCALAR> class mad_matrix_view_cm {
  
public:
  
  ~mad_matrix_view_cm() {}
  
  mad_matrix_view_cm(SCALAR* b,const SCALAR* e,size_t n1,size_t n2,size_t l):
    data(b),end(e),size1(n1),size2(n2),ld(l) {}
  
  SCALAR* const data;
  const SCALAR* end;
  
  size_t size1;
  size_t size2;
  size_t ld;
};

/* ===========================================================================*/
/*! \brief View on constant column-major dense matrix                         */
/* ===========================================================================*/

template <class SCALAR> class mad_const_matrix_view_cm {

public:

  mad_const_matrix_view_cm(const SCALAR* b,const SCALAR* e,size_t n1,size_t n2,size_t l):
    data(b),end(e),size1(n1),size2(n2),ld(l) {}

  ~mad_const_matrix_view_cm() {}

  const SCALAR* const data;
  const SCALAR* const end;
  size_t size1;
  size_t size2;
  size_t ld;
};

#endif
