// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#include "MAd_EdgeInterpolant.h"

// -----------------------------------------------------------------------------

MAd_EdgeInterpolant::MAd_EdgeInterpolant(const int &o)
: MAd_Interpolant(MSH_LIN_2, o), ndof(size(o)), Left(0), Right(o){    

  coef.resize(size(o),size(o));
  if(o<12){
    for (int i=0; i< int(ndof); i++){
      for (int j=0; j< int(ndof); j++){
        coef(i,j) = MAd_EdgeInterpolant::getCoef(o,i,j);
      }
    }
  }
  else{
    double x=0,y=0,z=0;    
    for (int i=0; i<=o; i++){  // monomial
      for (int j=0; j<=o; j++){ // nodal
        nodeNumberCoordinate(j,x,y,z);
        coef(i,j) = pow(x,i);
      }
    }
    coef = coef.inverse();
  }
}

// -----------------------------------------------------------------------------

MAd_EdgeInterpolant::~MAd_EdgeInterpolant(){}

// -----------------------------------------------------------------------------

double MAd_EdgeInterpolant::getCoef(int order,int i,int j){
  switch(order) {
  case 0: return coef0[i][j];
  case 1: return coef1[i][j];
  case 2: return coef2[i][j];
  case 3: return coef3[i][j];
  case 4: return coef4[i][j];
  case 5: return coef5[i][j];
  case 6: return coef6[i][j];
  case 7: return coef7[i][j];
  case 8: return coef8[i][j];
  case 9: return coef9[i][j];
  case 10: return coef10[i][j];
  case 11: return coef11[i][j];
  default : throw; break;
  }         
}

// -----------------------------------------------------------------------------
void MAd_EdgeInterpolant::fcts (const double &u, const double &v, const double &w, 
                                      doubleVector &f)const
{
  const int n=int(ndof);
  doubleVector m(n);
  f = 0;
  for (int i=0; i< int(ndof) ; i++) m(i) = pow(u,(double)(i));
  coef.mult(m,f,false);
  return;
}

// -----------------------------------------------------------------------------
void MAd_EdgeInterpolant::grads (const double &u, const double &v, const double &w, 
                                       doubleVector gr[3])const{
  const int n =int(ndof);
  doubleVector m(n);
  for (size_t k=0;k<3;k++) gr[k] = 0;
  m(0) = 0;
  for (int i=1; i< int(ndof) ; i++) m(i) = i*pow(u,(double)(i-1));
  coef.mult(m,gr[0],false);
  return;  
}

// -----------------------------------------------------------------------------
int MAd_EdgeInterpolant::size(int q) {
  return q+1;
}

//-------------------------------
//  Private Members
//-------------------------------


double MAd_EdgeInterpolant::coef0[1][1] = {{1}};

double MAd_EdgeInterpolant::coef1[2][2] = {{0.5, -0.5}, 
						 {0.5, 0.5}};

double MAd_EdgeInterpolant::coef2[3][3] = {{0, -0.5, 0.5}, 
						 {1, 0, -1}, 
						 {0, 0.5, 0.5}};

double MAd_EdgeInterpolant::coef3[4][4] = {{-0.0625, 0.0625, 0.5625, -0.5625}, 
						 {0.5625, -1.6875, -0.5625, 1.6875}, 
						 {0.5625, 1.6875, -0.5625, -1.6875}, 
						 {-0.0625, -0.0625, 0.5625, 0.5625}};

double MAd_EdgeInterpolant::coef4[5][5] = {{0, 0.166666666666667, -0.166666666666667, -0.666666666666667, 0.666666666666667}, 
						 {0, -1.33333333333333, 2.66666666666667, 1.33333333333333, -2.66666666666667}, 
						 {1, -5.80822592402949e-20, -5, 0, 4}, 
						 {0, 1.33333333333333, 2.66666666666667, -1.33333333333333, -2.66666666666667}, 
						 {0, -0.166666666666667, -0.166666666666667, 0.666666666666667, 0.666666666666667}};

double MAd_EdgeInterpolant::coef5[6][6] = {{0.01171875, -0.01171875, -0.325520833333333, 0.325520833333333, 0.813802083333333, -0.813802083333333}, 
						 {-0.09765625, 0.162760416666667, 2.5390625, -4.23177083333333, -2.44140625, 4.06901041666667}, 
						 {0.5859375, -2.9296875, -2.21354166666667, 11.0677083333333, 1.62760416666667, -8.13802083333333}, 
						 {0.5859375, 2.9296875, -2.21354166666667, -11.0677083333333, 1.62760416666667, 8.13802083333333}, 
						 {-0.09765625, -0.162760416666667, 2.5390625, 4.23177083333333, -2.44140625, -4.06901041666667}, 
						 {0.01171875, 0.01171875, -0.325520833333333, -0.325520833333333, 0.813802083333333, 0.813802083333333}};

double MAd_EdgeInterpolant::coef6[7][7] = {{8.28902744832256e-21, -0.05, 0.05, 0.5625, -0.5625, -1.0125, 1.0125}, 
						 {-6.94923662200002e-20, 0.45, -0.675, -4.5, 6.75, 4.05, -6.075}, 
						 {1.58168629069265e-19, -2.25, 6.75, 7.3125, -21.9375, -5.0625, 15.1875}, 
						 {1, 8.83850646028287e-19, -12.25, -3.26344853918137e-18, 31.5, -1.08781617972712e-18, -20.25}, 
						 {4.95514274143766e-20, 2.25, 6.75, -7.3125, -21.9375, 5.0625, 15.1875}, 
						 {1.52465930505774e-20, -0.45, -0.675, 4.5, 6.75, -4.05, -6.075}, 
						 {-3.81164826264435e-21, 0.05, 0.05, -0.5625, -0.5625, 1.0125, 1.0125}};

double MAd_EdgeInterpolant::coef7[8][8] = {{-0.00244140625, 0.00244140625, 0.137706163194444, -0.137706163194444, -0.911838107638889, 0.911838107638889, 1.27657335069444, -1.27657335069444}, 
						 {0.02392578125, -0.03349609375, -1.32655164930556, 1.85717230902778, 7.68549262152778, -10.7596896701389, -6.38286675347222, 8.93601345486111}, 
						 {-0.11962890625, 0.279134114583333, 6.21591796875, -14.50380859375, -17.58544921875, 41.03271484375, 11.48916015625, -26.8080403645833}, 
						 {0.59814453125, -4.18701171875, -5.02707248263889, 35.1895073784722, 10.8117947048611, -75.6825629340278, -6.38286675347222, 44.6800672743056}, 
						 {0.59814453125, 4.18701171875, -5.02707248263889, -35.1895073784722, 10.8117947048611, 75.6825629340278, -6.38286675347222, -44.6800672743056}, 
						 {-0.11962890625, -0.279134114583333, 6.21591796875, 14.50380859375, -17.58544921875, -41.03271484375, 11.48916015625, 26.8080403645833}, 
						 {0.02392578125, 0.03349609375, -1.32655164930556, -1.85717230902778, 7.68549262152778, 10.7596896701389, -6.38286675347222, -8.93601345486111}, 
						 {-0.00244140625, -0.00244140625, 0.137706163194444, 0.137706163194444, -0.911838107638889, -0.911838107638889, 1.27657335069444, 1.27657335069444}};

double MAd_EdgeInterpolant::coef8[9][9] = {{0, 0.0142857142857143, -0.0142857142857143, -0.311111111111111, 0.311111111111111, 1.42222222222222, -1.42222222222222, -1.62539682539683, 1.62539682539683}, 
						 {0, -0.152380952380952, 0.203174603174603, 3.2, -4.26666666666667, -12.8, 17.0666666666667, 9.75238095238095, -13.0031746031746}, 
						 {0, 0.8, -1.6, -15.0222222222222, 30.0444444444444, 36.9777777777778, -73.9555555555556, -22.7555555555556, 45.5111111111111}, 
						 {0, -3.2, 12.8, 21.6888888888889, -86.7555555555556, -41.2444444444444, 164.977777777778, 22.7555555555556, -91.0222222222222}, 
						 {1, -9.57301236387769e-19, -22.7777777777778, 3.03905154408816e-17, 121.333333333333, -1.55599439057314e-17, -213.333333333333, 3.88998597643284e-17, 113.777777777778}, 
						 {0, 3.2, 12.8, -21.6888888888889, -86.7555555555556, 41.2444444444444, 164.977777777778, -22.7555555555556, -91.0222222222222}, 
						 {0, -0.8, -1.6, 15.0222222222222, 30.0444444444444, -36.9777777777778, -73.9555555555556, 22.7555555555556, 45.5111111111111}, 
						 {0, 0.152380952380952, 0.203174603174603, -3.2, -4.26666666666667, 12.8, 17.0666666666667, -9.75238095238095, -13.0031746031746}, 
						 {0, -0.0142857142857143, -0.0142857142857143, 0.311111111111111, 0.311111111111111, -1.42222222222222, -1.42222222222222, 1.62539682539683, 1.62539682539683}};

double MAd_EdgeInterpolant::coef9[10][10] = {{0.0005340576171875, -0.0005340576171875, -0.0506783621651786, 0.0506783621651786, 0.627374267578125, -0.627374267578125, -2.16243896484375, 2.16243896484375, 2.08520900181362, -2.08520900181362}, 
						   {-0.0061798095703125, 0.00794546944754464, 0.58238525390625, -0.748781040736607, -6.88331909179688, 8.84998168945312, 20.9035766601563, -26.8760271344866, -14.5964630126953, 18.7668810163225}, 
						   {0.03460693359375, -0.06229248046875, -3.20643833705357, 5.77158900669643, 33.5489501953125, -60.3881103515625, -72.081298828125, 129.746337890625, 41.7041800362723, -75.0675240652902}, 
						   {-0.13458251953125, 0.40374755859375, 11.694287109375, -35.082861328125, -65.6206787109375, 196.862036132812, 112.446826171875, -337.340478515625, -58.3858520507812, 175.157556152344}, 
						   {0.605621337890625, -5.45059204101562, -9.0195556640625, 81.1760009765625, 38.3276733398438, -344.949060058594, -59.1066650390625, 531.959985351562, 29.1929260253906, -262.736334228516}, 
						   {0.605621337890625, 5.45059204101562, -9.0195556640625, -81.1760009765625, 38.3276733398437, 344.949060058594, -59.1066650390625, -531.959985351562, 29.1929260253906, 262.736334228516}, 
						   {-0.13458251953125, -0.40374755859375, 11.694287109375, 35.082861328125, -65.6206787109375, -196.862036132812, 112.446826171875, 337.340478515625, -58.3858520507813, -175.157556152344}, 
						   {0.03460693359375, 0.06229248046875, -3.20643833705357, -5.77158900669643, 33.5489501953125, 60.3881103515625, -72.081298828125, -129.746337890625, 41.7041800362723, 75.0675240652902}, 
						   {-0.0061798095703125, -0.00794546944754464, 0.58238525390625, 0.748781040736607, -6.88331909179688, -8.84998168945312, 20.9035766601563, 26.8760271344866, -14.5964630126953, -18.7668810163225}, 
						   {0.0005340576171875, 0.0005340576171875, -0.0506783621651786, -0.0506783621651786, 0.627374267578125, 0.627374267578125, -2.16243896484375, -2.16243896484375, 2.08520900181362, 2.08520900181362}};

double MAd_EdgeInterpolant::coef10[11][11] = {{0, -0.00396825396825397, 0.00396825396825397, 0.141231261022928, -0.141231261022928, -1.17549189814815, 1.17549189814815, 3.22937334656085, -3.22937334656085, -2.69114445546737, 2.69114445546737}, 
						    {0, 0.0496031746031746, -0.0620039682539683, -1.73748897707231, 2.17186122134039, 13.744212962963, -17.1802662037037, -33.5854828042328, 41.981853505291, 21.529155643739, -26.9114445546737}, 
						    {0, -0.297619047619048, 0.496031746031746, 10.063244047619, -16.7720734126984, -70.80078125, 118.001302083333, 133.696056547619, -222.826760912698, -72.660900297619, 121.101500496032}, 
						    {0, 1.19047619047619, -2.97619047619048, -36.1193783068783, 90.2984457671958, 169.270833333333, -423.177083333333, -263.516865079365, 658.792162698413, 129.174933862434, -322.937334656085}, 
						    {0, -4.16666666666667, 20.8333333333333, 48.2928240740741, -241.46412037037, -175.238715277778, 876.193576388889, 244.140625, -1220.703125, -113.02806712963, 565.140335648148}, 
						    {1, -4.03825680269823e-17, -36.5902777777778, 3.43455780593123e-16, 331.814236111111, 2.87164928191874e-16, -1110.02604166667, -7.8317707688693e-17, 1491.97048611111, 8.35388882012726e-16, -678.168402777778}, 
						    {0, 4.16666666666667, 20.8333333333333, -48.2928240740741, -241.46412037037, 175.238715277778, 876.193576388889, -244.140625, -1220.703125, 113.02806712963, 565.140335648148}, 
						    {0, -1.19047619047619, -2.97619047619048, 36.1193783068783, 90.2984457671958, -169.270833333333, -423.177083333333, 263.516865079365, 658.792162698413, -129.174933862434, -322.937334656085}, 
						    {0, 0.297619047619048, 0.496031746031746, -10.063244047619, -16.7720734126984, 70.80078125, 118.001302083333, -133.696056547619, -222.826760912698, 72.660900297619, 121.101500496032}, 
						    {0, -0.0496031746031746, -0.0620039682539682, 1.73748897707231, 2.17186122134039, -13.744212962963, -17.1802662037037, 33.5854828042328, 41.981853505291, -21.529155643739, -26.9114445546737}, 
						    {0, 0.00396825396825397, 0.00396825396825397, -0.141231261022928, -0.141231261022928, 1.17549189814815, 1.17549189814815, -3.22937334656085, -3.22937334656085, 2.69114445546737, 2.69114445546737}};

double MAd_EdgeInterpolant::coef11[12][12] = {{-0.000120162963867188, 0.000120162963867186, 0.0172130633157397, -0.0172130633157397, -0.340444737725367, 0.340444737725367, 2.09246740835684, -2.09246740835684, -4.75918339118756, 4.75918339118756, 3.49006782020421, -3.49006782020421}, 
						    {0.00161552429199219, -0.00197452969021266, -0.230622283693344, 0.281871680069642, 4.4640006050231, -5.45600073947267, -26.040706837972, 31.8275305797436, 53.2163233741881, -65.0421730128966, -31.4106103818379, 38.3907460222463}, 
						    {-0.0103855133056641, 0.016320092337472, 1.47244012620714, -2.31383448403979, -27.2758271864903, 42.862014150199, 142.918527532507, -224.586257551082, -239.257128666065, 375.975487903817, 122.152373707147, -191.953730111231}, 
						    {0.0436191558837891, -0.0959621429443356, -6.08084442501976, 13.3778577350435, 100.398494760826, -220.876688473817, -397.218394385444, 873.880467647976, 564.612211409069, -1242.14686509995, -261.755086515316, 575.861190333694}, 
						    {-0.145397186279297, 0.533123016357421, 19.0184194050138, -69.7342044850506, -177.073856147887, 649.270805875586, 563.164075215658, -2064.93494245741, -719.069345104884, 2636.58759871791, 314.106103818379, -1151.72238066739}, 
						    {0.610668182373047, -6.71735000610351, -14.1966058858236, 156.162664744059, 99.8276327062536, -1098.10395976879, -284.915968933105, 3134.07565826416, 345.257122378879, -3797.82834616767, -146.582848448577, 1612.41133293434}, 
						    {0.610668182373047, 6.71735000610351, -14.1966058858236, -156.162664744059, 99.8276327062536, 1098.10395976879, -284.915968933106, -3134.07565826416, 345.257122378879, 3797.82834616767, -146.582848448577, -1612.41133293434}, 
						    {-0.145397186279297, -0.533123016357421, 19.0184194050138, 69.7342044850506, -177.073856147887, -649.270805875586, 563.164075215658, 2064.93494245741, -719.069345104884, -2636.58759871791, 314.106103818379, 1151.72238066739}, 
						    {0.0436191558837891, 0.0959621429443358, -6.08084442501976, -13.3778577350435, 100.398494760826, 220.876688473818, -397.218394385444, -873.880467647976, 564.612211409069, 1242.14686509995, -261.755086515316, -575.861190333694}, 
						    {-0.0103855133056641, -0.0163200923374721, 1.47244012620714, 2.31383448403979, -27.2758271864903, -42.862014150199, 142.918527532507, 224.586257551082, -239.257128666065, -375.975487903817, 122.152373707147, 191.953730111231}, 
						    {0.00161552429199219, 0.00197452969021267, -0.230622283693344, -0.281871680069642, 4.4640006050231, 5.45600073947267, -26.040706837972, -31.8275305797436, 53.2163233741881, 65.0421730128966, -31.4106103818379, -38.3907460222463}, 
						    {-0.000120162963867188, -0.000120162963867187, 0.0172130633157397, 0.0172130633157397, -0.340444737725367, -0.340444737725367, 2.09246740835684, 2.09246740835684, -4.75918339118756, -4.75918339118756, 3.49006782020421, 3.49006782020421}};

