// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADEDGEINTERPOLANT_H
#define MADEDGEINTERPOLANT_H

#include "MAd_Interpolant.h"
#include "MAd_VertexInterpolant.h"
#include <sstream>
//using std::ostringstream;
// -----------------------------------------------------------------------------
/*! \brief Equidistant Lagrangian base for an edge  */

class MAd_EdgeInterpolant : public MAd_Interpolant {
 
public:
  
  virtual std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " << order() << " on edge ";
    return nameStr.str();
  }
  int size () const {return int(ndof);};
  size_t dim() const {return 1;};
  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;
  // size_t getClosureSpaceSize(int=-1) const {return 1;};
  // const int* getClosure (int iEd, int iSign, int iRotate) const;
  // size_t  getNbClosures() const            {return 2;}
  // size_t  getNbClosureOrientations() const {return 1;}
  // size_t  getNbClosureRotations(int=-1) const    {return 1;}
  using MAd_Interpolant::getCoef;
  static double getCoef(int order, int i, int j);
  static int size(int);

  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w) const { 
    u=-1+(i*2)/(1.*order()); v=0; w=0; return;
  };
  int nodeNumberTopoLevel(const size_t & i) const {
    if ((i==0)||(i==ndof-1))
      return 0;
    else return 1;
  };
  void getMonomialExponents ( int iMonomial , int &powerOfU, int &powerOfV, int &powerOfW) const
    {
      powerOfU = iMonomial;
      powerOfV = 0;
      powerOfW = 0;
    }
  // const MAd_Interpolant *getBoundaryFunctionSpace(int i) const{
  //   MAd_Interpolant* fs;
  //   fs = new MAd_VertexInterpolant(order());
  //   return fs;
  // }; 

  size_t getMaximumClosedSize() const {return (order()+1)*(order()+1);}
  
  ~MAd_EdgeInterpolant();
  MAd_EdgeInterpolant(const int&);

  MAd_Interpolant *clone(int o) const {return new MAd_EdgeInterpolant(o);}
  
  /*! map rotationally symmetric set of points to the values in the original element */
  virtual bool findOriginalCoordinates(int iRot,int iSwp,
                                       const doubleMatrix& uvwTfo,
                                       doubleMatrix& uvwOri) const {
    
    if (iRot != 0)      return false;
    if (abs(iSwp) != 1) return false;
    const doubleVector uTfo(uvwTfo.seeColumn(0));
    doubleVector uOri(uvwOri.touchColumn(0));
    uOri = uTfo;
    uOri *= iSwp;
    return true;
  }

  int nbRotations()    const {return 2;}
  int nbOrientations() const {return 1;}
 
  
protected:

  friend class MAd_TriInterpolant;
  friend class MAd_QuadInterpolant;

private:
  
  const size_t ndof;
  const int Left , Right;
  static double coef0[1][1];
  static double coef1[2][2];
  static double coef2[3][3];
  static double coef3[4][4];
  static double coef4[5][5];
  static double coef5[6][6];    
  static double coef6[7][7];
  static double coef7[8][8];
  static double coef8[9][9];
  static double coef9[10][10];
  static double coef10[11][11];
  static double coef11[12][12];
};

#endif
