// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADHEXINTERPOLANT_H
#define MADHEXINTERPOLANT_H

#include "MAd_Interpolant.h"
#include "MAd_EdgeInterpolant.h"
#include "MAd_QuadInterpolant.h"

// -----------------------------------------------------------------------------
/*! \brief Lagrangian base for a hexahedron based on equidistant points  */

class MAd_HexInterpolant : public MAd_Interpolant {
  
public:
  
  std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " << order() << " on hexahedron ";
    return nameStr.str();
  }
  
public:
  
  ~MAd_HexInterpolant();
  MAd_HexInterpolant(const int &);
  
public:
  int size () const {return int(ndof);};
  size_t dim() const {return 3;};
  
public:
  
  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;

public:
  
  const int* getClosure (int iEd, int iSign, int iRotate) const;
  /*  size_t*/ int  getClosureSpaceSize(int=-1) const;
  size_t  getNbClosures() const                 {return 6;}
  size_t  getNbClosureOrientations()      const {return 2;}
  size_t  getNbClosureRotations(int i=-1) const {return 4;}

  
  void getIJKfromIdx(int idx, int &i, int &j, int &k) const;
  int getIdxFromIJK(int i, int j, int k) const;
  double getCoef(int i, int j) const;
  void getMonomialExponents (int iMonomial , int &powerOfU, int &powerOfV,  int &powerOfW) const;
  void nodePositionHex(int order, double *u, double *v, double *w, int *idIJK) const;
  
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w)const;

  MAd_Interpolant *clone(int o) const {return new MAd_HexInterpolant(o);}
  static int size (int);
  static void createQuad(int o, int ordhex, const int pt[4][3], double *u, double *v, double *w);
  
protected:
  
  void getClosureSpace (const int &, const int &, const int & , int*)const;
  size_t getMaximumClosedSize() const {return 0;}
  
protected:
  
  virtual double monomeof(const double &u, const double &v, const double &w, int i) const;
  virtual double dmonomeof(const double &u, const double &v, const double &w, int i, int k) const;

 private: 

  MAd_QuadInterpolant* quad;
  
  static const int fToN[6][4];
  static const int fToE[6][4];
  static const int fToEdir[6][4];
  static const int NToC[8][3];

  void changeDir(int* idx, int order) const;
  void changeOrientation(int* idx, int rotate, int order) const;

  double *NodeNumberu;
  double *NodeNumberv;
  double *NodeNumberw;
  int *idFromIJK;

  const size_t ndof;
  MAd_EdgeInterpolant lefs;
  const int ndofe;
  integerMatrix OrientedFaceDof;
  inline void getOrientationFromCode(int code, int& facenumber, int &dir, int &rotate){
    int reste = code%8;
    int div = code/8;
    facenumber = div;
    if (reste<4){
      rotate = reste;
      dir = -1;
    }
    else {
      rotate =(reste-4);
      dir = 1;
    }
    return;
  }
  inline int getCodeFromOrientation(const int faceNumber, const int dir, const int rotate) const {
    int code = faceNumber*8 + 2*(dir+1) + rotate; // faceNumber*8 + 4.*(dir+1)/2. + rotate
    return code;
  }
};

#endif
