// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -----------------------------------------------------------------------------

#include "MAd_InterpolantFactory.hpp"


#include "MAd_TetInterpolant.h"
#include "MAd_TriInterpolant.h"
#include "MAd_QuadInterpolant.h"
#include "MAd_HexInterpolant.h"
#include "MAd_PrismInterpolant.h"
#include "MAd_PyramidInterpolant.h"

// -----------------------------------------------------------------------------

const int MAd_InterpolantFactory::maxOrder = 4;

// -----------------------------------------------------------------------------

MAd_InterpolantFactory::~MAd_InterpolantFactory() {

  InterpolantMap::iterator iter = interpolant.begin();
  for (;iter!=interpolant.end();++iter) delete iter->second;
}

// -----------------------------------------------------------------------------  

MAd_InterpolantFactory::MAd_InterpolantFactory():initialized(false) {
  initialize();
}

// -----------------------------------------------------------------------------

bool MAd_InterpolantFactory::initialize() {
  if (!initialized) for (int o=1;o<=maxOrder;o++) add(o);
  initialized = true;
  return true;
}

// -----------------------------------------------------------------------------

void MAd_InterpolantFactory::add(int o) {
  
  interpolant[std::make_pair(MSH_TYPE_PNT,o)] = new MAd_VertexInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_LIN,o)] = new MAd_EdgeInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_QUA,o)] = new MAd_QuadInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_HEX,o)] = new MAd_HexInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_PRI,o)] = new MAd_PrismInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_PYR,o)] = new MAd_PyramidInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_TRI,o)] = new MAd_TriInterpolant(o);
  interpolant[std::make_pair(MSH_TYPE_TET,o)] = new MAd_TetInterpolant(o);
}
  
// -----------------------------------------------------------------------------

const MAd_Interpolant* MAd_InterpolantFactory::find(int t,int o) const {
  
  InterpolantMap::const_iterator iter = interpolant.find(std::make_pair(t,o));  
  if (iter !=interpolant.end()) return iter->second;
  throw;
  return NULL;
}

// -----------------------------------------------------------------------------
