// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -----------------------------------------------------------------------------

#include "MAd_InterpolantManager.h"
#include "MAd_VertexInterpolant.h"
#include "MAd_EdgeInterpolant.h"
#include "MAd_TriInterpolant.h"
#include "MAd_QuadInterpolant.h"
#include "MAd_TetInterpolant.h"
#include "MAd_HexInterpolant.h"
#include "MAd_PrismInterpolant.h"
#include "MAd_PyramidInterpolant.h"

// -----------------------------------------------------------------------------

MAd_InterpolantManager::~MAd_InterpolantManager() {
  for (std::map<int,MAd_Interpolant*>::iterator iter = interpolants.begin();
       iter != interpolants.end();++iter) {
    delete iter->second;
  }
}

// -----------------------------------------------------------------------------

MAd_Interpolant* MAd_InterpolantManager::createFunctionSpace(int elType) const {
  
  MAd_Interpolant* fs = NULL;
  int order = getMappingOrderMSH(elType);

  if (isSerendipityMSH(elType))  {
    MAdMsgSgl::instance().error(__LINE__,__FILE__,
                                "Incomplete mappings not yet supported" 
                                " (checking for element type %i)",
                                elType);
  }
  else {
    
    switch (getElementTypeMSH(elType)) {   
    case MSH_TYPE_PNT: 
      {
        fs = new MAd_VertexInterpolant(order);
        break;
      } 
    case MSH_TYPE_LIN:
      {
        fs = new MAd_EdgeInterpolant(order);
        break;
      } 
    case MSH_TYPE_TRI:
      {
        fs = new MAd_TriInterpolant(order);
        break;
      } 
    case MSH_TYPE_QUA:
      {
        fs = new MAd_QuadInterpolant(order);
        break;
      }
    case MSH_TYPE_TET:
      { 
        fs = new MAd_TetInterpolant(order);
        break;
      }
    case MSH_TYPE_PRI:
      {
        fs = new MAd_PrismInterpolant(order);
        break;
      }
    case MSH_TYPE_PYR:
      {
        fs = new MAd_PyramidInterpolant(order);
        break;
      }
    case MSH_TYPE_HEX:
      {
        fs = new MAd_HexInterpolant(order);
        break;
      }
    default:
      {
        break;
      }
    }
  }
   
  return fs;
}


// -----------------------------------------------------------------------------

const MAd_Interpolant* MAd_InterpolantManager::findFunctionSpace(int elType)  {


  std::map<int,MAd_Interpolant*>::iterator iter = interpolants.find(elType);
  if (iter != interpolants.end()) return iter->second;

  MAd_Interpolant* interpolant = createFunctionSpace(elType);
  interpolants[elType] = interpolant;

  return interpolant;
}

// -----------------------------------------------------------------------------
