// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADPRISMINTERPOLANT_H
#define MADPRISMINTERPOLANT_H

#include "MAd_Interpolant.h"
#include "MAd_EdgeInterpolant.h"
#include "MAd_TriInterpolant.h"
#include "MAd_QuadInterpolant.h"

// -----------------------------------------------------------------------------
/*! \brief Lagrangian base for a prism based on equidistant points */

class MAd_PrismInterpolant : public MAd_Interpolant {
  
public:
  
  ~MAd_PrismInterpolant();
  MAd_PrismInterpolant(const int &);
  
public:
  
  std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " << order() << " on prism ";
    return nameStr.str();
  }
  
public:

  int size () const {return int(ndof);};
  size_t dim() const {return 3;};

public:

  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;
  
public:
  
  const int* getClosure (int iEd, int iSign, int iRotate) const;
  size_t  getClosureSpaceSize(int=-1) const;
  size_t  getNbClosures() const            {return 5;}
  size_t  getNbClosureOrientations() const {return 2;}
  size_t  getNbClosureRotations(int i=-1) const    {
    if (i<2 && i>=0) return 3;
    else if (i<6 && i>=0) return 4;
    else throw;
  }

public:
  
  void getIdTri(int* idTri) const;
  double getCoef(int i, int j) const;
  void getMonomialExponents (int iMonomial , int &powerOfU, int &powerOfV,  int &powerOfW) const;
  void nodePositionPrism(int orderTri, int orderQuad,double *u, double *v, double *w) const;
  
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w)const;

  const MAd_Interpolant *getBoundaryFunctionSpace(int i) const {
    MAd_Interpolant* fs;
    if (i<2) {
     fs = new MAd_TriInterpolant(order());
    }
    else {
     fs = new MAd_QuadInterpolant(order());
    }
    return fs;
  }; 

  MAd_Interpolant *clone(int o) const {return new MAd_PrismInterpolant(o);}
  static int size (int);
  static void createQuadPrism(int oQuad, int oTri, int OQuad, int OTri,const int pt[4][3], double *u, double *v, double *w);

  
protected:
  
  // void getClosureSpace (const int &, const int &, const int & , int*)const;
  size_t getMaximumClosedSize() const {return 0;}

protected:
  
  virtual double monomeof(const double &u, const double &v, const double &w, int i) const;
  virtual double dmonomeof(const double &u, const double &v, const double &w, int i, int k) const;

 private: 

  static int fToNTri[2][3];
  static int fToNQuad[3][4];
  static int fToEdirQuad[3][4];
  static int fToEQuad[3][4];
  static int NToCQuad[6][3];
  static int posWTri[2];
  void changeDirTri(int* idx, int order) const;
  void changeOrientationTri(int* idx, int rotate, int order) const;
  void changeDirQuad(int* idx, int order) const;
  void changeOrientationQuad(int* idx, int rotate, int order) const;
  double *NodeNumberu;
  double *NodeNumberv;
  double *NodeNumberw;
  int *idTri;

  const size_t ndof;
  MAd_EdgeInterpolant lefs;
  const size_t ndofe;
  MAd_TriInterpolant ltfs;
  const size_t ndoft;
  
};

#endif
