// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef _MADPYRAMIDINTERPOLANT_H_
#define _MADPYRAMIDINTERPOLANT_H_

// Functions

#include "Bergot.h"

// DGLib 

#include "MAd_Interpolant.h"
#include "MshTags.h"


#include <sstream>
//using std::ostringstream;

//--------------------------------------------------------------------
/*! \brief Lagrangian base for a pyramid based on equidistant control points */

class MAd_PyramidInterpolant : public MAd_Interpolant {

public:

  friend class MAd_Mapping;

public:

  MAd_PyramidInterpolant(const int &);
  ~MAd_PyramidInterpolant();  
  
public:

  virtual std::string name() const;
  
  int size() const {return size(order());}
  size_t dim() const {return 3;};

public:

  void fcts(const double&,const double&,const double&,doubleVector&) const;
  void grads(const double&,const double&,const double&,doubleVector[3]) const;

  void nodeNumberCoordinate(const int&,double&,double &,double&)const;
  double getCoef(int i, int j) const { return coef(i,j); }
  void getMonomialExponents(int,int&,int&,int&) const;
  
  const MAd_Interpolant *getBoundaryFunctionSpace(int) const; 
  virtual MAd_Interpolant *clone(int) const;
  static int size(int);

  const int nDof;
  double (*uvw) [3]; /*!< ordered parametric coordinates */
  
  size_t getMaximumClosedSize() const {return 0;}

private:
  
  void monomialFcts(const double&,const double&,const double&,doubleVector&) const;
  void monomialGrads(const double&,const double&,const double&,doubleVector[3]) const;

private:

  int computePoints(int order,double (*uvw)[3]) const;

private:
  
  Bergot* bergot; /*!< Bergots family of orthonormal base functions for the given order */ 

  int* iOrder; //!< power of u in the monomial 
  int* jOrder; //!< power of v in the monomial 
  int* kOrder; //!< power of w in the monomial 
  int* dOrder; //!< power of 1-w in the weight function and the denominator 

protected: // given a priori  
  
  static const int edgeDef[8][2]; //!< edges in terms of vertices 
  static const int quadDef[4];    //!< base in terms of vertices 
  static const int triaDef[4][3]; //!< triangular faces in terms of vertices 
  static const double vCoord[5][3]; //!< principal vertices in parametric space 

protected: // computed during construction

  int nbDofTriangle;   //!< number of points per triangle
  int nbDofQuadrangle; //!< number of points on the quadrilateral base

  int* quadDof; //!< points on the quadrilateral base, for all orientations 
  int* triaDof; //!< points on the triangular faces, for all orientations 
  
};

//--------------------------------------------------------------------
#endif
