// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADQUADINTERPOLANT_H
#define MADQUADINTERPOLANT_H

#include "MAd_Interpolant.h"
#include "MAd_EdgeInterpolant.h"

// -----------------------------------------------------------------------------
/*! \brief Lagrangian base for a quadrilateral based on equidistant points */

class MAd_QuadInterpolant : public MAd_Interpolant {

public:

  virtual std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " 
            << order() << " on quadrilateral ";
    return nameStr.str();
  }

  int size () const {return int(ndof);};
  virtual size_t dim() const {return 2;};

public:

  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;
  const int* getClosure (int iEd, int iSign, int iRotate) const; // replace double*
  size_t  getClosureSpaceSize(int=-1) const;
  void getIJfromIdx(int idx, int &i, int &j) const;
  size_t getIdxFromIJ(int i, int j) const;
  double getCoef(int i, int j) const;
  void getMonomialExponents (int iMonomial, int &powerOfU, int &powerOfV,  int &powerOfW) const;
  static void nodePositionQuad(int order, double *u, double *v, int *idIJ);
  
  size_t  getNbClosures() const            {return 4;}
  size_t  getNbClosureOrientations() const {return 2;}
  size_t  getNbClosureRotations(int=-1) const    {return 1;}
  
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w) const;
  virtual const MAd_Interpolant *getBoundaryFunctionSpace(int i) const {
   MAd_Interpolant* fs;
   fs = new MAd_EdgeInterpolant(order());
   return fs;
  };
  
  size_t getMaximumClosedSize() const {return (order()+1)*(order()+1)*(order()+1);}

  ~MAd_QuadInterpolant();
  MAd_QuadInterpolant(const int &);

  
  virtual MAd_Interpolant *clone(int o) const {return new MAd_QuadInterpolant(o);}
  static int size (int);
  
  /*! map rotationally symmetric set of points to the values in the original element */
  virtual bool findOriginalCoordinates(int,int,const doubleMatrix&,doubleMatrix&) const;
  virtual bool reorientData(doubleMatrix&,int) const;

protected:
  
  void getClosureSpace (const int &, const int &, int*) const; //3rd argument int*
  
protected:

  const size_t ndof;
  MAd_EdgeInterpolant lefs;
  const int ndofe;
  
private:
  
  friend class MAd_HexInterpolant;
  friend class MAd_PrismInterpolant;
  integerMatrix codeClosure;
  double *NodeNumberu;
  double *NodeNumberv;
  int *idFromIJ;
  static int directionEdge[4][2];
  double monomeof(const double &u, const double &v, int i) const;
  double dmonomeof(const double &u, const double &v, int i, int k) const;
  inline int getOrientationCode(int facenumber, int dir) const;
  int nodeNumberTopoLevel(const int & i) const {
    if (i<4)
      return 0;
    else if (i<4*p) return 1;
    else return 2;
  };
};

#endif
