// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// ------------------------------------------------------------------


#ifndef MADTETINTERPOLANT_H_
#define MADTETINTERPOLANT_H_

#include "MAd_Interpolant.h"
#include <sstream>

//--------------------------------------------------------------------
/*! \brief Lagrangian base for a tetrahedron based on equidistant control points */

class MAd_TetInterpolant : public MAd_Interpolant{

public:

  MAd_TetInterpolant(const int &);
  ~MAd_TetInterpolant();  
  
public:

  virtual std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " << order() << " on tetrahedron ";
    return nameStr.str();
  }

  int size () const {return ndof;};
  size_t dim() const {return 3;};

public:

  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;
  
public:
  
  void getOrientedFaceDof (int i, int* res) const; 
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w)const;
  virtual void getMonomialExponents ( int iMonomial , int &powerOfU, int &powerOfV,  int &powerOfW) const;
    
  virtual MAd_Interpolant *clone(int o) const;
  static int size(int);
  
protected:
  
  virtual double monomeof(const double &u, const double &v, const double &w, int i) const;
  virtual double dmonomeof(const double &u, const double &v, const double &w, int i, int k) const;
  
  int (*pp)[3];

  const int ndof;
  double *NodeNumberu;
  double *NodeNumberv;
  double *NodeNumberw;
  integerMatrix OrientedFaceDof;

  int nbdoftriangle(int) const;


private:  
  
  //  DGMatrix coef;
  static double coef0[1][1];
  static double coef1[4][4];
  static double coef2[10][10];
  static double coef3[20][20];
  static double coef4[35][35];
  static double coef5[56][56];    
  static double coef6[84][84];
  static double coef7[120][120];
  static double coef8[165][165];
  static double coef9[220][220];
  static double coef10[286][286];
  static int P[286][3];
  
  void nodepositionface0(int order,  double *u,  double *v,  double *w);
  void nodepositionface1(int order,  double *u,  double *v,  double *w);
  void nodepositionface2(int order,  double *u,  double *v,  double *w);
  void nodepositionface3(int order,  double *u,  double *v,  double *w);
  void nodeposition(int order,  double *u,  double *v,  double *w);

  inline void getOrientationFromCode(int code, int& facenumber, int &dir, int &rotate){
    int reste = code%6;
    int div = code /6;
    facenumber = div;
    if (reste <3){
      rotate = reste;
      dir = 1;
    }
    else{
      rotate =(reste-3);
      dir = -1;
    }
    return;
  }
  inline int getCodeFromOrientation(const int faceNumber, const int dir, const int rotate) const {
    int code = faceNumber*6 - (dir - 1)/2 * 3 + rotate;
    return code;
  }
};

//--------------------------------------------------------------------
#endif
