// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#ifndef MADTRIINTERPOLANT_H
#define MADTRIINTERPOLANT_H

#include "MAd_Interpolant.h"
#include "MAd_EdgeInterpolant.h"

// -----------------------------------------------------------------------------
/*! \brief Lagrangian base for a triangle based on equidistant points */

class MAd_TriInterpolant : public MAd_Interpolant{
    
public:

  virtual std::string name () const {
    std::ostringstream nameStr;
    nameStr << "Equidistant Lagrangian function space of order " << order() << " on triangle ";
    return nameStr.str();
  }

  int size () const {return ndof;}; 
  virtual size_t dim() const {return 2;};

public:

  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const;
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const;

public:

  const int* getClosure (int iEd, int iSign, int iRotate) const;
  size_t  getClosureSpaceSize(int=-1) const;
  
  size_t  getNbClosures() const            {return 3;}
  size_t  getNbClosureOrientations() const {return 2;}
  size_t  getNbClosureRotations(int=-1) const    {return 1;}

public:
  
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w)const;
  void getMonomialExponents (int iMonomial , int &powerOfU, int &powerOfV,  int &powerOfW) const
    {
      powerOfU = pp[iMonomial][0];
      powerOfV = pp[iMonomial][1];
      powerOfW = 0;
    } 
  virtual const MAd_Interpolant *getBoundaryFunctionSpace(int i) const {
   MAd_Interpolant* fs;
   fs = new MAd_EdgeInterpolant(order());
   return fs;
  }; 
  
  ~MAd_TriInterpolant();
  MAd_TriInterpolant(const int &);
  
  size_t getMaximumClosedSize() const {return (order()+1)*(order()+2)*(order()+1)/2;}

  virtual MAd_Interpolant *clone(int o) const {return new MAd_TriInterpolant(o);}
  
  using MAd_Interpolant::getCoef;
  static double getCoef(int order, int i, int j);
  static int size (int);

  virtual bool reorientData(doubleMatrix&,int) const;
  /*! map rotationally symmetric set of points to the values in the original element */
  virtual bool findOriginalCoordinates(int,int,const doubleMatrix&,doubleMatrix&) const;

protected:
  
  void getClosureSpace (const int &, const int &, int*) const;

protected:

  int (*pp)[2];
  const int ndof;
  
 private:
  
  friend class MAd_TetInterpolant;
  friend class MAd_PrismInterpolant;
  static double coef0[1][1];
  static double coef1[3][3];
  static double coef2[6][6];
  static double coef3[10][10];
  static double coef4[15][15];
  static double coef5[21][21];    
  static double coef6[28][28];
  static double coef7[36][36];
  static double coef8[45][45];
  static double coef9[55][55];
  static double coef10[66][66];
  static double coef11[78][78];
  static int P[78][2];
  integerMatrix  codeClosure;
  void nodePositionTriangle(int order, double *u, double *v);
  double *NodeNumberu;
  double *NodeNumberv;
  double monomeof(const double &u, const double &v, int i) const;
  double dmonomeof(const double &u, const double &v, int i, int k) const;
  inline int getOrientationCode(int facenumber, int dir) const;
  int nodeNumberTopoLevel(const int & i) const {
    if (i<3)
      return 0;
    else if (i<3*p) return 1;
    else return 2;
  };
};

#endif
