// -*- C++ -*- 
// -----------------------------------------------------------------------------
// MAdLiB - Copyright (C) 2015 Cenaero
// -----------------------------------------------------------------------------
//
// See the Copyright.txt and License.txt files for license information. 
// -----------------------------------------------------------------------------


#ifndef MADVERTEXINTERPOLANT_H
#define MADVERTEXINTERPOLANT_H

#include "MAd_Interpolant.h"


// -----------------------------------------------------------------------------
/*! Used to define the closure for edge shape functions */ 

class MAd_VertexInterpolant : public MAd_Interpolant {

public:
  MAd_VertexInterpolant(const /*size_t*/int& o):MAd_Interpolant(MSH_PNT,o), ndof(1) {}
  ~MAd_VertexInterpolant(){};
  
public:

  std::string name() const {return "Vertex function space of infinite order";}
  int size () const {return 1;};
  size_t dim() const {return 0;};

public:

  void fcts  (const double &u, const double &v, const double &w, doubleVector  &f) const {f(0) = 1;};
  void grads (const double &u, const double &v, const double &w, doubleVector gr[3]) const{
    gr[0](0) = 0; gr [1](0) = 0 ; gr[2](0) = 0;
  }

public:
  
  virtual void nodeNumberCoordinate(const int & i, double &u, double &v, double &w) const { u=0; v=0; w=0; return;};
  
  size_t getMaximumClosedSize() const {return order()+1;}

public:
  
  const MAd_Interpolant *getBoundaryFunctionSpace(int i) const{
    MAd_Interpolant* fs;
    fs = new MAd_VertexInterpolant(order());
    return fs;
  }; 
public:
  
  int nodeNumberTopoLevel(const size_t & i) const {
    return 0;
  };
  
public:

  // const MAd_Interpolant *getBoundaryFunctionSpace(int i) const {
  //   throw;
  //   return 0;
  // };  

  MAd_Interpolant *clone(int o) const {throw;}
  
  void getMonomialExponents ( int iMonomial , int &powerOfU, int &powerOfV, int &powerOfW) const {
    powerOfU = 0;
    powerOfV = 0;
    powerOfW = 0;
  }

 private:
  friend class MAd_EdgeInterpolant;
  const int ndof;
}; 

#endif
