// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include "MAdMapping.h"
#include "MappingInterface.h"

using std::vector;
namespace MAd {
  // ----------------------------------------------------------------------
  pMapping Map_create(pEntity ent, double s)
  {
    pMapping mapping = new MAdMapping(ent,s);
   return mapping;
  }
  // ----------------------------------------------------------------------
  void Map_delete(pMapping mapping)
  {
   if(mapping) {delete mapping; /*mapping=NULL;*/}
  }
  // ----------------------------------------------------------------------
  int Map_dim(pMapping mapping)
  {
   return mapping->dim();
  } 
  // ----------------------------------------------------------------------
  int Map_getNumberOfKnots(pMapping mapping)
  {
    return mapping->getNumberOfKnots();
  }
  // ----------------------------------------------------------------------
  bool Map_getVertexUVW(pMapping mapping, int k, double* uvw)
  {
   return mapping->getVertexUVW(k, uvw[0], uvw[1], uvw[2]);
  } 
  // ----------------------------------------------------------------------
  void Map_COG(pMapping mapping, double* uvw)
  {
   mapping->COG(uvw[0], uvw[1], uvw[2]);
  }
  // ----------------------------------------------------------------------
  int Map_getEdgeNumber(pMapping mapping, pEntity entity)
  {
   return mapping->getEdgeNumber(entity);
  }
  // ----------------------------------------------------------------------
  int Map_getFaceNumber(pMapping mapping, pEntity entity)
  {
   return mapping->getFaceNumber(entity);
  }
  // ----------------------------------------------------------------------
  void Map_boundBox(pMapping mapping, double bbox[3][2])
  {
    
    mapping->boundingBox(bbox);
  }
  // ----------------------------------------------------------------------
  double Map_getMaxRadius(pMapping mapping)
  {
   return mapping->getMaxRadius();
  }  
  // ----------------------------------------------------------------------
  void Map_getEdgePtsOnFace(pMapping mapping, 
                            int k, int dir, double quadPt, double* uv)
  {
   mapping->getEdgePointsOnFace(k, dir, quadPt, uv[0], uv[1]);
  }
  // ----------------------------------------------------------------------
  void Map_getFacePtsOnElt(pMapping mapping, int k, int dir, int rot, 
                           double ksi, double eta, double* uvw)
  {
   mapping->getFacePointsOnElement(k,dir,rot,ksi,eta,uvw[0],uvw[1],uvw[2]);
  } 
  // ----------------------------------------------------------------------
  void Map_eval(pMapping mapping, const double* uvw, double* xyz)
  {
   mapping->eval(uvw[0], uvw[1], uvw[2], xyz[0], xyz[1], xyz[2]);
  }
  // ----------------------------------------------------------------------
  void Map_deval(pMapping mapping, const double* uvw, double* dXdU)
  {
   mapping->deval(uvw[0] , uvw[1] , uvw[2], 
                  dXdU[0], dXdU[1], dXdU[2], 
                  dXdU[3], dXdU[4], dXdU[5], 
                  dXdU[6], dXdU[7], dXdU[8]);
  }
  // ----------------------------------------------------------------------
  double Map_jacInv(pMapping mapping, const double* xyz, double* InvJac)
  {
   double* invjac = new double[9];
   for(int i=0; i<9; i++) invjac[i]=InvJac[i];
   doubleMatrix* MatrixInvJac = new doubleMatrix(invjac, 3, 3);
   double jacinv= mapping->invertedJacobian(xyz[0], xyz[1], xyz[2], 
                                            *MatrixInvJac);
   for(int i=0; i<9; i++) InvJac[i]=invjac[i];
   delete MatrixInvJac;
   return jacinv;
  }

  // ----------------------------------------------------------------------
  
  double Map_determinant(pMapping mapping,
                         const double* uvw) {
    return mapping->determinant(uvw);
  }
  
  // ----------------------------------------------------------------------
  double Map_invertedJacobian(pMapping mapping,
                              const double* uvw,
                              double* invJac) {

    return mapping->invertedJacobian(uvw,invJac);
  }

  // ----------------------------------------------------------------------
  void Map_metricTensor(pMapping mapping,const double* uvw,double* metric) {
    
    return mapping->metricTensor(uvw,metric);
  }
  
  // ----------------------------------------------------------------------
  double Map_paraProject(pMapping mapping, double* uvw,double eps)
  {
    return mapping->parametricProjection(uvw[0], uvw[1], uvw[2],eps);
  }
  // ----------------------------------------------------------------------
  bool Map_projAlongDir(pMapping mapping, 
                        const double* xyz, 
                        const double* dir, 
                        double* uvw, 
                        double* prj, 
                        double& distance, double tol)
  {
    return mapping->projectAlongDirection(xyz,dir,uvw,prj,distance,tol);
  }
  // ----------------------------------------------------------------------
  bool Map_project(pMapping mapping,const double* xyz,double* uvw,double& distance)
  {
   return mapping->project(xyz[0],xyz[1],xyz[2],uvw[0],uvw[1],uvw[2],distance);
  }
  // ----------------------------------------------------------------------
  bool Map_invert(pMapping mapping,const double* xyz,double* uvw,
                  double eps, int maxiter, int np)
  {
    return mapping->invert(xyz[0], xyz[1], xyz[2], 
                           uvw[0], uvw[1], uvw[2], 
                           eps, maxiter,np);
  }
  
  // -----------------------------------------------------------------------
  
  void Map_normal(pMapping mapping,
                  pEntity border,
                  int direction,
                  int rotation,
                  const double* uvw,
                  double * normal) 
  {  
    mapping->normalVector(border,direction,rotation,uvw,normal); 
  }

  // ----------------------------------------------------------------------
  bool Map_planeCut(pMapping mapping, 
                    const double* point, 
                    const double* normal, 
                    int nbPoints, 
                    double* coord, 
                    double* param) 
  {
    double* coordT = new double[3*nbPoints];
    for(int i=0; i<3*nbPoints; i++) coordT[i]=coord[i];
    double* paramT = new double[3*nbPoints];
    for(int i=0; i<3*nbPoints; i++) paramT[i]=param[i];
    doubleMatrix* coords = new doubleMatrix(coordT, nbPoints, 3); 
    doubleMatrix* params = new doubleMatrix(paramT, nbPoints, 3); 
    bool logical = mapping->computeCut(point,normal,nbPoints,*coords, *params);
    for(int i=0; i<3*nbPoints; i++) {
      param[i]=paramT[i];
      coord[i]=coordT[i];
    }
    delete coords;
    delete params;
    return logical;
  }
  // ----------------------------------------------------------------------
  bool Map_inRefElt(pMapping mapping, const double* uvw, double eps)
  {
   return mapping->inReferenceElement(uvw[0], uvw[1], uvw[2], eps);
  } 
  // ----------------------------------------------------------------------
  
}
