// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _CHECKMESH_H_
#define _CHECKMESH_H_

#include "MAdExport.h"
#include "MeshDataBaseInterface.h"

namespace MAd {

  // -------------------------------------------------------------------
  typedef enum MeshStatus { 
    VALID                   = 0,
    NEGATIVE_VOLUME         = 1,
    GEOM_INCOMPATIBILITY    = 2,
    WRONG_EDGE_TO_RGN_CONN  = 3,
    WRONG_FACE_TO_RGN_CONN  = 4,
    WRONG_FACE_TO_VTX_CONN  = 5,
    WRONG_EDGE_CONN         = 6,
    WRONG_ENTITY_POINTERS   = 7,
    WRONG_ITERATORS         = 8,
    WRONG_PARAMETERS        = 9
  } MeshStatus;

  // -------------------------------------------------------------------
  typedef enum checkType { 
    CHECK_ALL,
    CHECK_VOLUME,
    CHECK_GEOM_COMPATIBILITY,
    CHECK_EDGE_TO_RGN_CONN,
    CHECK_FACE_TO_RGN_CONN,
    CHECK_FACE_TO_VTX_CONN,
    CHECK_EDGE_CONN,
    CHECK_ENTITY_POINTERS,
    CHECK_ITERATORS,
    CHECK_PARAMETERS
  } checkType;

  // -------------------------------------------------------------------
  // return 1 if the mesh passes the test successfully, 0 otherwise
  MAD_EXPORT bool checkMesh(MDB_Mesh * mesh, checkType type=CHECK_ALL, int verbose=1, MeshStatus * status=NULL);

  // -------------------------------------------------------------------

  MAD_EXPORT void M_info(const pMesh);

  MAD_EXPORT void R_info(const pRegion, const char * name = "");
  MAD_EXPORT void F_info(const pFace, const char * name = "");
  MAD_EXPORT void E_info(const pEdge, const char * name = "");
  //! \brief write informations about vertex pv to the output (with optional name) 
  //! \ingroup api_mesh_vertex
  MAD_EXPORT void V_info(const pVertex, const char * name = "");

}

#endif
