// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_MDBEXTRUSION
#define _H_MDBEXTRUSION

#include "MeshDataBaseInterface.h"
#include "MeshDataBaseParallelInterface.h"
#include "MeshDataBaseComm.h"

#include <limits>
#include <map>
#include <vector>
#include <cstdlib>
#include <string>

/*! \defgroup extrusion Extrusion data \ingroup parallel */

namespace MAd {  
  
  // ---------------------------------------------------------------------------
  /*! \brief internal data \ingroup internal */
  // ---------------------------------------------------------------------------
  
  struct ExtrusionData {
    pEntity seedElmt;
    int     seedProc;
    int     layerIdx;
  };
  
  // ---------------------------------------------------------------------------
  /*! \brief stores the data for an extrusion face \ingroup extrusion */
  /*! 
    <ul> 
    <li> seedFace holds the starting face of the extrusion </li>
    <li> seedPart holds the partition containing the starting face </li>
    <li> myRegion contains the region on top of this face </li>
    <li> index gives the ordering number in the extrusion </li>
    </ul>
  */
  // ---------------------------------------------------------------------------

  struct MAd_extrusionData {
    
  public:
    
    pEntity seedElmt;
    int     seedProc;
    pEntity neighbour;
    int     layerIdx;

    MAd_extrusionData() {
      seedElmt    = NULL;
      seedProc    = -1;
      neighbour   = NULL;
      layerIdx    = -1;
    }
    
    MAd_extrusionData(const ExtrusionData& ed,const pEntity pr) {
      seedElmt    = ed.seedElmt;
      seedProc    = ed.seedProc;
      layerIdx    = ed.layerIdx;
      neighbour   = pr;
    }
  };

  bool isExtruded(pRegion pr, pFace pf); 

   
};

#endif
