// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _LOADGMSHMESH_H_
#define _LOADGMSHMESH_H_

#include "MAdExport.h"
#include "MeshDataBaseInterface.h"
#include "MeshDataBaseParallelInterface.h"
#include "TransformationInterface.h"

namespace MAd {

  // -------------------------------------------------------------------
  
  /*! \brief Gmsh reader supporting all versions */
  /*! if the partition is not specified, we read the partition corresponding
      to the current rank, else, we create a standalone mesh containing the
      elements classified on the specified partition 
  */

  MAD_EXPORT void LoadGmshMesh (MAd::pMesh   m, //!< mesh (contains communicator)
                     const char * f, //!< file name 
                     int  part=-1,   //!< partition 
                     bool swap=false //!< switch roles of elementary and physical tags
                     );

  /*! \brief Gmsh mesh writer supporting versions 1 and 2 */
  /*! 
    - single file export 
    - If a partitioning table is submitted, write the right partition numbers in the file
  */
  MAD_EXPORT void SaveGmshMesh (const MAd::pMesh,
                     const char *, int version=2, 
                     bool saveAll=true, 
                     const int * partitionTable=NULL);

  /*! \brief Gmsh mesh format writer for old-style periodic meshes */
  /*! \warning obsolescent */

  MAD_EXPORT void SaveGmshMeshPer (const MAd::pMesh,
                        const char *,  
                        MDB_DataExchangerPeriodic &deperiodic,
                        int version=1);

  
  /*! \brief Gmsh mesh format writer for high order meshes */

  MAD_EXPORT void SaveGmshMeshHighOrder (const MAd::pMesh,
                              const char *,
                              bool newNum=true);

  /*! \brief Gmsh mesh format writer for periodicities */
  MAD_EXPORT void SaveGmshPeriodicities(const MAd::pMesh mesh,FILE* fp,bool distributed);
  
  /*! \brief Gmsh mesh format writer for bounding boxes */
  MAD_EXPORT void SaveGmshBoundingBox(const MAd::pMesh mesh,FILE* fp,bool distributed);

  /*! \brief Gmsh mesh format writer for periodicities */
  MAD_EXPORT bool ReadGmshPeriodicities(FILE* fp,TransformationSet&);
  
  /*! \brief Gmsh mesh format writer for bounding boxes */
  MAD_EXPORT bool ReadGmshBoundingBox(FILE* fp,double bb[3][2],int rank);

  // -------------------------------------------------------------------

}

#endif
