// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef H_MESHDATABASEITERATORS
#define H_MESHDATABASEITERATORS

#include <cstdio>

namespace MAd {

  template <class CONTAINER, class DATA, class CLASSIF>
  class MDB_Iter
  {
  public:
    CONTAINER *l;
    CLASSIF   g;
    std::set<CLASSIF> allGs;

    int closure;
  
    typename CONTAINER::iterator current;
    typename CONTAINER::iterator end;
  
    MDB_Iter (CONTAINER *_l):l(_l),g(NULL),closure(0)
    {
      reset();
    }
    MDB_Iter (CONTAINER *_l,CLASSIF _g):l(_l),g((CLASSIF) _g),closure(0) {
      reset();
    }
  
    MDB_Iter (CONTAINER *_l,CLASSIF _g,int _c):l(_l),g((CLASSIF) _g),closure(_c) {
    
      // if (_c == 1) {
      //   printf("Closure iterator not yet implemented \n");
      //   exit(1);
      // }
      
      reset();
    }
    
    virtual ~MDB_Iter() {}

    bool addClosure(CLASSIF gg,std::set<CLASSIF>& theSet) const {

      if (GEN_type(gg) == 0) return false;
      pPGList closureList = GEN_closure(gg);
      void * temp = NULL;
      while (pGEntity pGE = PGList_next(closureList, &temp)) {
        std::set<CLASSIF> subList;
        addClosure(pGE, subList);
        theSet.insert(subList.begin(), subList.end());
        theSet.insert(pGE);
      }
      PGList_delete(closureList);
      return true;
    }
    
    inline void reset() {

      if (g) {
        allGs.insert(g);
        if (closure) addClosure(g,allGs);
      }
      
      end = l->end();
      current = l->begin();

      if (!g) {
        while(current != end && (*current)->deleted) {
          typename CONTAINER::iterator currentb = current;
          current++;
          delete *currentb;
          l->erase(currentb);       
        }
      }
      else {
        while(current != end && ((*current)->deleted || 
                                 allGs.find((*current)->g) == allGs.end())) {
          typename CONTAINER::iterator currentb = current;
          current++;
          if ((*currentb)->deleted) {
            delete *currentb;
            l->erase(currentb);
          }
        }
      }
    }
    inline void cleanup()
    {
      current = l->begin();
      while(next()){}  
    }
  
    inline virtual DATA * next ()
    {
      if (current == end) return 0;
    
      DATA *r = *current;
      ++ current;

      if (!g) {
        while(current != end && (*current)->deleted)
          {
            typename CONTAINER::iterator currentb = current;
            current++;
            delete *currentb;
            l->erase(currentb);       
          }
      }
      else {  
        while(current != end && ((*current)->deleted || 
                                 allGs.find((*current)->g) == allGs.end()))
          {
            typename CONTAINER::iterator currentb = current;
            current++;
            if ((*currentb)->deleted) {
              delete *currentb;
              l->erase(currentb);
            }
          }
      }

      // if the result data was marked for deletion
      // after previous call to next() look for the next one
      if (r->deleted) {

        if (current == end) return 0;

        r = *current;
        ++ current;
      
        if (!g) {
          while(current != end && (*current)->deleted)
            {
              typename CONTAINER::iterator currentb = current;
              current++;
              delete *currentb;
              l->erase(currentb);       
            }
        }
        else {  
          while(current != end && ((*current)->deleted || 
                                   allGs.find((*current)->g) == allGs.end()))
            {
              typename CONTAINER::iterator currentb = current;
              current++;
              if ((*currentb)->deleted) {
                delete *currentb;
                l->erase(currentb);
              }
            }
        }

      }

      return r;
    }
  
  };

  template <class CONTAINER, class DATA, class CLASSIF>
  int countClassifiedElements (CONTAINER* c,CLASSIF g) {
  
    MDB_Iter<CONTAINER,DATA,CLASSIF>* iter = new MDB_Iter<CONTAINER,DATA,CLASSIF>(c,g);
  
    DATA* el = NULL;
    int count = 0;
    while ( (el = iter->next()) ) count++;
    delete iter;

    return count;
  
  }
}

#endif
