// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_NULLENTITIES
#define _H_NULLENTITIES

#ifndef _HAVE_GMSH_

#include "Physical.h"
#include <stdlib.h>

namespace MAd {

  class NullModel;

  // -------------------------------------------------------------------
  class NullGEntity {

  public:
    NullGEntity(int t, NullModel *m, Physical *_p=NULL): _tag(t),phys(_p),model(m) {}
    NullGEntity(const NullGEntity& ge): _tag(ge._tag),phys(ge.phys),model(ge.model) {}
    virtual ~NullGEntity() {}

    void setPhysical(int d, int t);

    virtual int dim() const=0;
    virtual int tag() const { return _tag; }
    int pDim() const {return phys ? phys->dim():-1;} 
    int pTag() const {return phys ? phys->tag():0;}

    virtual bool addClosure(NullGEntity*) = 0;
    
    bool addInClosure(NullGEntity* ge);

    std::set<NullGEntity*> closure;
    std::set<NullGEntity*> inClosure;
    
  private:
    int _tag;
    Physical * phys;
    NullModel * model;
  };

  // -------------------------------------------------------------------
  class NullGEntityLessThan {
  public:
    bool operator()(const NullGEntity *ent1, const NullGEntity *ent2) const
    { return ent1->tag() < ent2->tag(); }
  };
  
  // -------------------------------------------------------------------
  class NullGVertex : public NullGEntity {

  public:
    NullGVertex(int t, NullModel *m, Physical *_p=NULL) : NullGEntity(t, m, _p) {}
    NullGVertex(const NullGVertex& gv) : NullGEntity(gv) {}
    virtual ~NullGVertex() {}
    int dim() const { return 0; }
    bool addClosure(NullGEntity* c) { return false; }
  };

  // -------------------------------------------------------------------
  class NullGEdge : public NullGEntity {

  public:
    NullGEdge(int t, NullModel *m, Physical *_p=NULL) : NullGEntity(t, m, _p) {}
    NullGEdge(const NullGEdge& ge) : NullGEntity(ge) {}
    virtual ~NullGEdge() {}
    int dim() const { return 1; }
    bool addClosure(NullGEntity* c);
  };

  // -------------------------------------------------------------------
  class NullGFace : public NullGEntity {

  public:
    NullGFace(int t, NullModel *m, Physical *_p=NULL) : NullGEntity(t, m, _p) {}
    NullGFace(const NullGFace& gf) : NullGEntity(gf) {}
    virtual ~NullGFace() {}
    int dim() const { return 2; }
    bool addClosure(NullGEntity* c);
  };
  
  // -------------------------------------------------------------------
  class NullGRegion : public NullGEntity {

  public:
    NullGRegion(int t, NullModel *m, Physical *_p=NULL) : NullGEntity(t, m, _p) {}
    NullGRegion(const NullGRegion& gr) : NullGEntity(gr) {}
    virtual ~NullGRegion() {}
    int dim() const { return 3; }
    bool addClosure(NullGEntity* c);
  };

  typedef class NullGEntity MAdGEntity;
  typedef class NullGEntityLessThan MAdGEntityLessThan;
  typedef class NullGRegion MAdGRegion;
  typedef class NullGFace   MAdGFace;
  typedef class NullGEdge   MAdGEdge;
  typedef class NullGVertex MAdGVertex;
  // -------------------------------------------------------------------
}

#endif

#endif

