// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#ifndef _H_NULLMODEL
#define _H_NULLMODEL

#ifndef _HAVE_GMSH_

#include "MAdModel.h"
#include "NullEntities.h"
#include <set>

#ifdef PARALLEL
#include <mpi.h>
#endif

namespace MAd {

  // -------------------------------------------------------------------
  class NullModel : public MAdModel {

  public:

    NullModel(std::string name=""): MAdModel(name) {}
    ~NullModel();

    ModelType type() const { return NULLMODEL; }

    // Gmsh mesh file format
    int readMSH(const std::string &filename);
  
    // Gmsh native CAD format
    int readGEO(const std::string &filename);
  
    // OCC formats
    int readSTEP(const std::string &filename);
    int readBREP(const std::string &filename);
    int readIGES(const std::string &filename);

    // get the number of entities in this model
    int getNumRegions()  const { return (int)(regions.size());  }
    int getNumFaces()    const { return (int)(faces.size());    }
    int getNumEdges()    const { return (int)(edges.size());    }
    int getNumVertices() const { return (int)(vertices.size()); }

    // find the entity with the given tag and create it if not found
    MAdGEntity * getEntityByTag(int dim, int tag);
    MAdGRegion * getRegionByTag(int);
    MAdGFace   * getFaceByTag  (int);
    MAdGEdge   * getEdgeByTag  (int);
    MAdGVertex * getVertexByTag(int);

    // get an iterator initialized to the first/last entity in this model.
    riter firstRegion() { return regions.begin(); }
    fiter firstFace()   { return faces.begin(); }
    eiter firstEdge()   { return edges.begin(); }
    viter firstVertex() { return vertices.begin(); }
    riter lastRegion()  { return regions.end(); }
    fiter lastFace()    { return faces.end(); }
    eiter lastEdge()    { return edges.end(); }
    viter lastVertex()  { return vertices.end(); }
    
#ifdef PARALLEL
    bool exchange(MPI_Comm);
#endif
      
  private:

    // delete all geometric entities
    void clean();

    std::set<MAdGRegion *,MAdGEntityLessThan> regions;
    std::set<MAdGFace   *,MAdGEntityLessThan> faces;
    std::set<MAdGEdge   *,MAdGEntityLessThan> edges;
    std::set<MAdGVertex *,MAdGEntityLessThan> vertices;

#ifdef PARALLEL
    template<class EntityType,class EntityTypeLessThan>
    bool exchangeTemplate(std::set<EntityType*,EntityTypeLessThan>&,MPI_Comm,int);
#endif
    
  };
  
  void SwapBytes(char *array, int size, int n);
  
}

#include "NullModel.ipp"


// -------------------------------------------------------------------

#endif

#endif
