// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

namespace MAd {

#ifdef PARALLEL  
  template<class EntityType,class EntityTypeLess>
  bool NullModel::exchangeTemplate(std::set<EntityType*,EntityTypeLess>& entities,
                                   MPI_Comm comm,int dim) {
    
    std::vector<int> tags;
    tags.push_back(entities.size());

    typename std::set<EntityType*,EntityTypeLess>::iterator eIter = entities.begin();
    for (;eIter!=entities.end();++eIter) {
      
      EntityType* e = *eIter;
      tags.push_back(e->tag());
      tags.push_back(e->pDim());
      tags.push_back(e->pTag());

      tags.push_back(e->closure.size());

      std::set<NullGEntity*>::iterator cIter = e->closure.begin();
      for (;cIter!=e->closure.end();++cIter) {
        tags.push_back((*cIter)->tag());
      }
    }

    int nbProcs;
    MPI_Comm_size(comm,&nbProcs);
    int rank;
    MPI_Comm_rank(comm,&rank);
    
    int* size = new int[nbProcs];
    int tagSize = tags.size();

    MPI_Allgather(&tagSize,1,MPI_INT,size,1,MPI_INT,comm);
    
    int* offset = new int[nbProcs+1];
    offset[0] = 0;
    for (int i=0;i<nbProcs;i++) offset[i+1] = offset[i] + size[i];

    int* allTags = new int[offset[nbProcs]];

    MPI_Allgatherv(tags.data(),tagSize,MPI_INT,
                   allTags,size,offset,MPI_INT,comm);

    for (int iProc=0;iProc<nbProcs;iProc++) {
      if (iProc!=rank) {
        int* pTags = allTags + offset[iProc];
        int  nbEntities = *(pTags++);
        for (int iEnt=0;iEnt<nbEntities;iEnt++) {
          int entTag = *pTags++;
          int physDim = *pTags++;
          int physTag = *pTags++;

          NullGEntity* gEnt = getEntityByTag(dim,entTag);
          gEnt->setPhysical(physDim,physTag);

          int nbClos = *pTags++;
          for (int iClos=0;iClos<nbClos;iClos++) {
            int closTag = *pTags++;
            NullGEntity* gClos = getEntityByTag(dim-1,closTag);
            gEnt->addClosure(gClos);
          }
        }
      }
    }
    
    delete [] size;
    delete [] offset;
    delete [] allTags;

    return true;
  }
#endif 
}
