//-*- C++ -*-
// Meshing algorithm
Mesh.Algorithm=5;
Mesh.Algorithm3D = 1;
Mesh.Optimize = 1;
Mesh.OptimizeNetgen = 1;

// Geom parameters
Geometry.OCCFixDegenerated = 0;
Geometry.OCCFixSmallEdges = 0;
Geometry.OCCFixSmallFaces = 0;
Geometry.OCCSewFaces = 0;
Geometry.OCCConnectFaces = 0;
Geometry.OCCScaling = 0.001;

// Import the step file
Merge "T106C_3D.step";
// Physical Surface("inlet")  = {1};
// Physical Surface("top")    = {2};
// Physical Surface("outlet") = {3};
// Physical Surface("bottom") = {4};
// Physical Surface("blade")  = {5};
// Physical Surface("back")   = {6};
// Physical Surface("front")  = {7};
// Physical Volume ("fluid")  = {1};

Periodic Surface {6} = {7} Translate {0.,0.,-0.019781090821925};
Periodic Surface {4} = {2} Translate {0.,-0.1879203628082, 0.};

// Geometry parameters
x_le =  0.0005;
y_le =  0.0005;
x_te =  .169;
y_te =  -0.099;

x_wake = x_te + 1.*(x_te - x_le);

alpha_le = 37.;
alpha_te = -65.;

ca_le    =  0.798635510047;
sa_le    =  0.601815023152;
ca_te    =  0.422618261741;
sa_te    =  -0.90630778703;

a = 0.;
b = 0.75355405;
c = -6.62720491;
d = -7.68023659;

// Meshing parameters
fac     = 4.;
h_far   =  fac * 1.5e-3;
h_wake  =  fac * 1e-3;
h_blade =  fac * .5e-3;
h_lete  =  fac * 1.25e-4;

Mesh.CharacteristicLengthMin = h_lete;
Mesh.CharacteristicLengthMax = h_far;
// Mesh.CharacteristicLengthFromCurvature = 1;

Characteristic Length {9,10} = h_blade;
Point(100) = {x_le,y_le,-.05};
Point(101) = {x_le,y_le,.05};
Line(100)  = {100,101};

Point(102) = {x_te,y_te,-.05};
Point(103) = {x_te,y_te,.05};
Line(101)  = {102,103};

Field[1] = Attractor;
Field[1].EdgesList = {100};
Field[1].NNodesByEdge=100;

Field[2] = Threshold;
Field[2].IField = 1;
Field[2].LcMin = h_lete;
Field[2].LcMax = h_far;
Field[2].DistMin = .005;
Field[2].DistMax = .05;

Field[3] = Attractor;
Field[3].EdgesList = {101};
Field[3].NNodesByEdge=100;

Field[4] = Threshold;
Field[4].IField = 3;
Field[4].LcMin = h_lete;
Field[4].LcMax = h_far;
Field[4].DistMin = .005;
Field[4].DistMax = .05;

// wake
nbBoxes = 20;
dx      = .02;
dy      = .02;

For i In {0:(nbBoxes-1)}
l  = i/(nbBoxes-1);
x  = x_te*l + x_wake * (1-l);
y  = y_te + sa_te/ca_te * (x - x_te);
Field[100+i]= Box;
Field[100+i].VIn  =  h_wake;
Field[100+i].VOut =  h_far;
Field[100+i].XMin =  x-dx;
Field[100+i].XMax =  x+dx;
Field[100+i].YMin =  y-dy;
Field[100+i].YMax =  y+dy;
Field[100+i].ZMin =  -.05;
Field[100+i].ZMax =  .05;
EndFor

For i In {0:(nbBoxes-1)}
l  = i/(nbBoxes-1);
x  = x_te*l + x_wake * (1-l);
y  = y_te + sa_te/ca_te * (x - x_te) + .158;
Field[100+nbBoxes+i]= Box;
Field[100+nbBoxes+i].VIn  =  h_wake;
Field[100+nbBoxes+i].VOut =  h_far;
Field[100+nbBoxes+i].XMin =  x-dx;
Field[100+nbBoxes+i].XMax =  x+dx;
Field[100+nbBoxes+i].YMin =  y-dy;
Field[100+nbBoxes+i].YMax =  y+dy;
Field[100+nbBoxes+i].ZMin =  -.05;
Field[100+nbBoxes+i].ZMax =  .05;
EndFor

For i In {0:(nbBoxes-1)}
l  = i/(nbBoxes-1);
x  = x_te*l + x_wake * (1-l);
y  = y_te + sa_te/ca_te * (x - x_te) + 2*.158;
Field[100+2*nbBoxes+i]= Box;
Field[100+2*nbBoxes+i].VIn  =  h_wake;
Field[100+2*nbBoxes+i].VOut =  h_far;
Field[100+2*nbBoxes+i].XMin =  x-dx;
Field[100+2*nbBoxes+i].XMax =  x+dx;
Field[100+2*nbBoxes+i].YMin =  y-dy;
Field[100+2*nbBoxes+i].YMax =  y+dy;
Field[100+2*nbBoxes+i].ZMin =  -.05;
Field[100+2*nbBoxes+i].ZMax =  .05;
EndFor

Field[5000] = Min;
Field[5000].FieldsList = {2,4,100:100+3*nbBoxes};
Background Field = 5000;

