#include "deform.h"

using namespace std;
using namespace MAd;

double invert33(double (&m)[3][3], double (&minv)[3][3]) {
  double det = m[0][0] * (m[1][1] * m[2][2] - m[2][1] * m[1][2]) -
    m[0][1] * (m[1][0] * m[2][2] - m[1][2] * m[2][0]) +
    m[0][2] * (m[1][0] * m[2][1] - m[1][1] * m[2][0]);

  double invdet = 1 / det;

  minv[0][0] = (m[1][1] * m[2][2] - m[2][1] * m[1][2]) * invdet;
  minv[0][1] = (m[0][2] * m[2][1] - m[0][1] * m[2][2]) * invdet;
  minv[0][2] = (m[0][1] * m[1][2] - m[0][2] * m[1][1]) * invdet;
  minv[1][0] = (m[1][2] * m[2][0] - m[1][0] * m[2][2]) * invdet;
  minv[1][1] = (m[0][0] * m[2][2] - m[0][2] * m[2][0]) * invdet;
  minv[1][2] = (m[1][0] * m[0][2] - m[0][0] * m[1][2]) * invdet;
  minv[2][0] = (m[1][0] * m[2][1] - m[2][0] * m[1][1]) * invdet;
  minv[2][1] = (m[2][0] * m[0][1] - m[0][0] * m[2][1]) * invdet;
  minv[2][2] = (m[0][0] * m[1][1] - m[1][0] * m[0][1]) * invdet;

  return det;
}

int getRHS(const pMesh mesh, const vector<int> &tags, vector<int> &isBc, Eigen::VectorXd &rhs){

  pMeshDataId tagId       = MD_lookupMeshDataId("ID");
  pMeshDataId nx = MD_lookupMeshDataId("nx");
  pMeshDataId ny = MD_lookupMeshDataId("ny");
  pMeshDataId nz = MD_lookupMeshDataId("nz");

  const int dim    = M_dim(mesh);
  double coords[3], xx,yy,zz;
  isBc.assign(rhs.size(),0);
  rhs.setZero();

  if (dim == 2){
    // loop over the all the edges
    pEdge e;
    EIter eIt= M_edgeIter(mesh);
    while ( ( e = EIter_next(eIt) ) ) {
      // Get the element tag
      pGEntity pGE = (pGEntity) E_whatIn(e);
      bool inBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
      if (inBc) { // the edge belongs to the boundaries where BL will be added
	// Vertices 
	for (int i = 0; i < E_numVertices(e); i++){
	  const pVertex v = E_vertex(e,i);
	  const int iDof = getID(tagId,(pEntity) v);
	  for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 1;
	  EN_getDataDbl(v , nx, &xx);
	  EN_getDataDbl(v , ny, &yy);
	  EN_getDataDbl(v , nz, &zz);
	  rhs [dim*iDof+0] = -xx;
	  rhs [dim*iDof+1] = -yy;
	}
	// HO points
	for(int i = 0; i < E_numPoints(e); i++) {
	  const pPoint p = E_point(e,i);
	  const int iDof =  getID(tagId,(pEntity) p);
	  for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 1;
	  EN_getDataDbl(p , nx, &xx);
	  EN_getDataDbl(p , ny, &yy);
	  EN_getDataDbl(p , nz, &zz);
	  rhs [dim*iDof+0] = -xx;
	  rhs [dim*iDof+1] = -yy;
	}	
      } else if (E_numFaces(e) == 1) { // the edge belongs to another boundary
	// Vertices 
	for (int i = 0; i < E_numVertices(e); i++){
	  const pVertex v = E_vertex(e,i);
	  const int iDof = getID(tagId,(pEntity) v);
	  if (isBc[dim*iDof] == 0)
	    for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 2;
	}
	// HO points
	for(int i = 0; i < E_numPoints(e); i++) {
	  const pPoint p = E_point(e,i);
	  const int iDof = getID(tagId,(pEntity) p);
	  if (isBc[dim*iDof] == 0)
	    for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 2;
	}	
      }
    }
    EIter_delete(eIt);
  } else if (dim == 3){
    // loop over the all the edges
    pFace f;
    FIter fIt= M_faceIter(mesh);
    while ( ( f = FIter_next(fIt) ) ) {
      // Get the element tag
      pGEntity pGE = (pGEntity) F_whatIn(f);
      bool inBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
      if (inBc) { // the edge belongs to the boundaries where BL will be added
	// Vertices 
	for (int i = 0; i < F_numVertices(f); i++){
	  const pVertex v = F_vertex(f,i);
	  const int iDof = getID(tagId,(pEntity) v);
	  for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 1;
	  EN_getDataDbl(v , nx, &xx);
	  EN_getDataDbl(v , ny, &yy);
	  EN_getDataDbl(v , nz, &zz);
	  rhs [dim*iDof+0] = -xx;
	  rhs [dim*iDof+1] = -yy;
	  rhs [dim*iDof+2] = -zz;
	}
	// HO points
	for(int j = 0; j < F_numEdges(f); j++){
	  pEdge e = F_edge(f,j);
	  for(int i = 0; i < E_numPoints(e); i++) {
	    const pPoint p = E_point(e,i);
	    const int iDof =  getID(tagId,(pEntity) p);
	    for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 1;
	    EN_getDataDbl(p , nx, &xx);
	    EN_getDataDbl(p , ny, &yy);
	    EN_getDataDbl(p , nz, &zz);
	    rhs [dim*iDof+0] = -xx;
	    rhs [dim*iDof+1] = -yy;
	    rhs [dim*iDof+2] = -zz;
	  }	
	}
      } else if (F_numRegions(f) == 1) { // the edge belongs to another boundary
	// Vertices 
	for (int i = 0; i < F_numVertices(f); i++){
	  const pVertex v = F_vertex(f,i);
	  const int iDof = getID(tagId,(pEntity) v);
	  if (isBc[dim*iDof] == 0)
	    for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 2;
	}
	// HO points
	for(int j = 0; j < F_numEdges(f); j++){
	  pEdge e = F_edge(f,j);
	  for(int i = 0; i < E_numPoints(e); i++) {
	    const pPoint p = E_point(e,i);
	    const int iDof = getID(tagId,(pEntity) p);
	    if (isBc[dim*iDof] == 0)
	      for(int k = 0; k < dim; k++) isBc[dim*iDof+k] = 2;
	  }
	}	
      }
    }
    FIter_delete(fIt);
  }

}

int getMat(const pMesh mesh, int order, const vector<int> &isBc, SparseMatrix &mat, Eigen::VectorXd &rhs){

  const int dim     = M_dim(mesh);
  
  pMeshDataId tagId = MD_lookupMeshDataId("ID");

  // Preallocation
  {
    Eigen::VectorXi nnz = Eigen::VectorXi::Constant(mat.outerSize(),0);
    if (dim == 2) {
      pVertex v;
      VIter vIt= M_vertexIter(mesh);
      while ( ( v = VIter_next(vIt) ) ) {
	const int nEdges = V_numEdges(v);
	const int nFaces = V_numFaces(v);
	const int np_per_E = (order == 1) ? 0: 1;
	const int iDof = getID(tagId,(pEntity) v);
	for(int k = 0; k < dim; k++) nnz[dim*iDof+k] = dim*(1 // itself
							    +nEdges*(1+np_per_E) // adjacent edges
							    +nFaces*np_per_E // opposite edge
							    );
      }
      VIter_delete(vIt);
      pEdge e;
      EIter eIt= M_edgeIter(mesh);
      while ( ( e = EIter_next(eIt) ) ) {
      	for (int i = 0; i < E_numPoints(e); i++){
      	  const pPoint p = E_point(e,i);
      	  const int nFaces = E_numFaces(e);
      	  const int np_per_E = (order == 1) ? 0: 1;
      	  const int iDof =  getID(tagId,(pEntity) p);
      	  for(int k = 0; k < dim; k++) nnz[dim*iDof+k] = dim*(2+np_per_E // itself
      							      +nFaces*(1 //opposite node
      								       + 2*np_per_E //other edges
      								       )
      							      );
      	}
      }
      EIter_delete(eIt);
    } else if (dim == 3) {
       pVertex v;
      VIter vIt= M_vertexIter(mesh);
      while ( ( v = VIter_next(vIt) ) ) {
	const int nEdges   = V_numEdges(v);
	const int nFaces   = V_numFaces(v);
	// const int nRegions = V_numRegions(v);
	const int np_per_E = (order == 1) ? 0: 1;
	const int iDof = getID(tagId,(pEntity) v);
	for(int k = 0; k < dim; k++) nnz[dim*iDof+k] = dim*(1 // itself
							    +nEdges*(1+np_per_E) // adjacent edges
							    +nFaces*np_per_E // opposite edges
							    );
      }
      VIter_delete(vIt);
      pEdge e;
      EIter eIt= M_edgeIter(mesh);
      while ( ( e = EIter_next(eIt) ) ) {
      	for (int i = 0; i < E_numPoints(e); i++){
      	  const pPoint p = E_point(e,i);
	  const int nFaces   = E_numFaces(e);
      	  const int nRegions = E_numRegions(e);
      	  const int np_per_E = (order == 1) ? 0: 1;
      	  const int iDof =  getID(tagId,(pEntity) p);
      	  for(int k = 0; k < dim; k++) nnz[dim*iDof+k] = dim*(2+np_per_E // itself
      							      +nFaces*(1 //opposite node
								       + 2*np_per_E //other edges
								       )
							      +nRegions*np_per_E
      							      );
      	}
      }
      EIter_delete(eIt);
    }
    mat.reserve(nnz);
    mat.setZero();

    if (dim == 3) {

      int *iptr    = mat.outerIndexPtr();
      int *idxptr  = mat.innerIndexPtr();
      int *nnzptr  = mat.innerNonZeroPtr();
      double *vptr = mat.valuePtr();

      iptr[0] = 0;
      for(int i = 1; i < mat.rows()+1; i++){
	iptr[i] = iptr[i-1]+nnz[i-1];
	nnzptr[i-1] = nnz[i-1];
      }

      pVertex v;
      VIter vIt= M_vertexIter(mesh);
      vector<int> dofs;
      while ( ( v = VIter_next(vIt) ) ) {
	const int nEdges   = V_numEdges(v);
	const int nFaces   = V_numFaces(v);
	// const int nRegions = V_numRegions(v);
	const int np_per_E = (order == 1) ? 0: 1;
	const int iDof = getID(tagId,(pEntity) v);
	dofs.clear();
	// itself
	for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof+k);
	// adjacent edges
	int iDof2;
	for(int i = 0; i < nEdges; i++) {
	  const pEdge e = V_edge(v,i);
	  const pVertex v2 = E_otherVertex(e,v);
	  iDof2 = getID(tagId,(pEntity) v2);
	  for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  if (E_numPoints(e) == 1) {
	    iDof2 = getID(tagId,(pEntity) E_point(e,0));
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  }
	}
	pPList vFaces = V_faces(v);
	void * temp = NULL;
	while ( pFace f = (pFace) PList_next(vFaces,&temp) ) {
	  pEdge e;
	  if (v == F_vertex(f,0))
	    e = F_edge(f,1);
	  else if (v == F_vertex(f,1))
	    e = F_edge(f,2);
	  else if (v == F_vertex(f,2))
	    e = F_edge(f,0);
	  else
	    throw;
	  if (E_numPoints(e) == 1) {
	    iDof2 = getID(tagId,(pEntity) E_point(e,0));
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  }
	}
	PList_delete(vFaces);
	sort(dofs.begin(),dofs.end());
	for(int k = 0;  k < dim; k++) {
	  const int j0 = iptr[dim*iDof+k];
	  const int j1 = iptr[dim*iDof+k+1];
	  for(int j = j0; j < j1; j++){
	    idxptr[j] = dofs[j-j0];
	    vptr[j]   = 0.;
	  }
	}
	  // for (size_t i = 0; i < dofs.size(); i++)
	  //   mat.insert(dim*iDof+k,dofs[i]) = 0.;
	// for(int k = 0;  k < dim; k++)
	//   for (size_t i = 0; i < dofs.size(); i++)
	//     mat.insert(dim*iDof+k,dofs[i]) = 0.;
	  
      }
      VIter_delete(vIt);
      pEdge e;
      EIter eIt= M_edgeIter(mesh);
      while ( ( e = EIter_next(eIt) ) ) {
      	for (int i = 0; i < E_numPoints(e); i++){
      	  const pPoint p = E_point(e,i);
	  const int nFaces   = E_numFaces(e);
      	  const int nRegions = E_numRegions(e);
      	  const int np_per_E = (order == 1) ? 0: 1;
      	  const int iDof =  getID(tagId,(pEntity) p);
	  dofs.clear();
	  // itself
	  for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof+k);
	  int iDof2;
	  iDof2 = getID(tagId,(pEntity) E_vertex(e,0));
	  for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  iDof2 = getID(tagId,(pEntity) E_vertex(e,1));
	  for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);

	  for(int i = 0; i < nFaces; i++) {
	    const pFace f = E_face(e,i);
	    pEdge e1,e2;
	    pVertex v1;
	    if (e == F_edge(f,0)) {
	      e1 = F_edge(f,1);
	      e2 = F_edge(f,2);
	      v1 = F_vertex(f,2);
	    } else if (e == F_edge(f,1)) {
	      e1 = F_edge(f,0);
	      e2 = F_edge(f,2);
	      v1 = F_vertex(f,0);
	    } else if (e == F_edge(f,2)) {
	      e1 = F_edge(f,0);
	      e2 = F_edge(f,1);
	      v1 = F_vertex(f,1);
	    } else
	      throw;

	    iDof2 = getID(tagId,(pEntity) E_point(e1,0));
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	    iDof2 = getID(tagId,(pEntity) E_point(e2,0));
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	    iDof2 = getID(tagId,(pEntity) v1);
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  }
	  
	  pPList eRegions = E_regions(e);
	  void * temp = NULL;
	  while ( pRegion r = (pRegion) PList_next(eRegions,&temp) ) {
	    // for(int i = 0; i < nRegions; i++) {
	    // pRegion r = E_region(e,i);
	    pEdge e1;
	    if (e == R_edge(r,0))
	      e1 = R_edge(r,5);
	    else if (e == R_edge(r,1))
	      e1 = R_edge(r,3);
	    else if (e == R_edge(r,2))
	      e1 = R_edge(r,4);
	    else if (e == R_edge(r,3))
	      e1 = R_edge(r,1);
	    else if (e == R_edge(r,4))
	      e1 = R_edge(r,2);
	    else if (e == R_edge(r,5))
	      e1 = R_edge(r,0);
	    else
	      throw;
	    iDof2 = getID(tagId,(pEntity) E_point(e1,0));
	    for(int k = 0; k < dim; k++) dofs.push_back(dim*iDof2+k);
	  }
	  PList_delete(eRegions);
	  sort(dofs.begin(),dofs.end());
	  for(int k = 0;  k < dim; k++) {
	    const int j0 = iptr[dim*iDof+k];
	    const int j1 = iptr[dim*iDof+k+1];
	    for(int j = j0; j < j1; j++){
	      idxptr[j] = dofs[j-j0];
	      vptr[j]   = 0.;
	    }
	  }
	  // for(int k = 0;  k < dim; k++)
	  //   for (size_t i = 0; i < dofs.size(); i++)
	  //     mat.insert(dim*iDof+k,dofs[i]) = 0.;
      	}
      }
      EIter_delete(eIt);
    }
  }

  // Matrix assembly
  if (dim == 2) {
    TriQuadrature triQuadrature(order == 1? 2: 6);
    const int nqp = triQuadrature.nqp();
    TriFEM triFEM(order);
    const int m = triFEM.n();
    vector<double> phi (nqp*m);
    vector<double> dphi(nqp*m*dim);
    vector<double> dphi_el(m*dim);
    vector<double> qp  (nqp*3);
    vector<int> ids(m);
    for (int iqp = 0; iqp < nqp; iqp++){
      triQuadrature.qp(iqp,&qp[3*iqp]);
      for(int i = 0; i < m; i++) {
	phi[iqp*m+i] = triFEM.eval(i,qp[3*iqp],qp[3*iqp+1]);
	for(int k = 0; k < dim; k++)
	  dphi[iqp*m*dim+i*dim+k] = triFEM.deval(i,qp[3*iqp],qp[3*iqp+1],k);
      }
    }
      
    double tmpMat[9];
    Eigen::Matrix2d invJac;
    Eigen::Vector2d v0,v1;
    pFace f;
    FIter fIt= M_faceIter(mesh);
    while ( ( f = FIter_next(fIt) ) ) {
      // Mapping
      pMapping map = Map_create((pEntity) f, 1.0);

      // Get the vertices & point IDs;
      for(int i = 0; i < 3; i++)
	ids[i] = getID(tagId,(pEntity) F_vertex(f,i));
      if (order == 2)
	for(int i = 0; i < 3; i++)
	  ids[3+i] = getID(tagId,(pEntity) E_point(F_edge(f,i),0));

      Eigen::MatrixXd block(dim*m,dim*m);
      block.setZero();

      for(int iqp = 0; iqp < nqp; iqp++){
	// Get the jacobian
	Map_jacInv(map,&qp[3*iqp],&tmpMat[0]);
	for(int k = 0; k < dim; k++)
	  for(int l = 0; l < dim; l++)
	    invJac(k,l) = tmpMat[3*k+l];
	const double detJac = 1./invJac.determinant();
	const double Jxw = triQuadrature.wp(iqp) * detJac;
	// Gradient in th element
	for(int i = 0; i < m; i++){
	  for(int k = 0; k < dim; k++)
	    v0[k] = dphi[iqp*m*dim+i*dim+k];
	  v1 =  invJac.transpose() * v0 ;
	  for(int k = 0; k < dim; k++)
	    dphi_el[i*dim+k] = v1[k];
	}

	// Material properties
	const double E = 1./fabs(detJac);
	const double nu = 0.3;
	const double lambda = E*nu / (1.+nu) / (1 - 2.*nu);
	const double mu = E/2./(1.+nu);

	// Build the block
	for(int i = 0; i < m; i++){
	  for(int j = 0; j < m; j++){
	    for(int k = 0; k < dim; k++){
	      const int iblock = i*dim + k;
	      // const int imat = ids[i]*dim + k;
	      for(int l = 0; l < dim; l++){
		const int jblock = j*dim + l;
		// const int jmat = ids[j]*dim + l;
		double val = 0.;
		val  += lambda * dphi_el[i*dim+k]* dphi_el[j*dim+l];
		if (k == l)
		  for(int kk = 0; kk < dim; kk++)
		    val += mu * dphi_el[i*dim+kk]* dphi_el[j*dim+kk];
		val += mu * dphi_el[i*dim+l]* dphi_el[j*dim+k];
		block(iblock,jblock) += Jxw * val;
		// mat.coeffRef(imat,jmat) += Jxw * val;
	      }
	    }
	  }
	}
      }
      Map_delete(map);
      for(int i = 0; i < m; i++){
      	for(int j = 0; j < m; j++){
      	  for(int k = 0; k < dim; k++){
      	    const int iblock = i*dim + k;
      	    const int imat = ids[i]*dim + k;
      	    for(int l = 0; l < dim; l++){
      	      const int jblock = j*dim + l;
      	      const int jmat = ids[j]*dim + l;
	      mat.coeffRef(imat,jmat) += block(iblock,jblock);
      	    }
      	  }
      	}
      }
    }
    FIter_delete(fIt);

  } else if (dim == 3) {

    int *iptr    = mat.outerIndexPtr();
    int *idxptr  = mat.innerIndexPtr();
    double *vptr = mat.valuePtr();
    
    TetQuadrature tetQuadrature(order == 1? 2: 6);
    const int nqp = tetQuadrature.nqp();
    TetFEM tetFEM(order);
    const int m = tetFEM.n();
    vector<double> phi (nqp*m);
    vector<double> dphi(nqp*m*dim);
    vector<double> dphi_el(m*dim);
    vector<double> qp  (nqp*3);
    vector<int> ids(m);
    for (int iqp = 0; iqp < nqp; iqp++){
      tetQuadrature.qp(iqp,&qp[3*iqp]);
      for(int i = 0; i < m; i++) {
    	phi[iqp*m+i] = tetFEM.eval(i,qp[3*iqp],qp[3*iqp+1],qp[3*iqp+2]);
    	for(int k = 0; k < dim; k++){
    	  dphi[iqp*m*dim+i*dim+k] = tetFEM.deval(i,qp[3*iqp],qp[3*iqp+1],qp[3*iqp+2],k);
    	}
      }
    }
    Eigen::MatrixXd D(m,dim),G(m,dim);

    double V = 0.;

    double tmpMat[9];
    Eigen::Matrix3d jac, invJac;
    Eigen::Vector3d v0,v1;
    pRegion r;
    RIter rIt= M_regionIter(mesh);
    while ( ( r = RIter_next(rIt) ) ) {
      // Mapping
      pMapping map = Map_create((pEntity) r, 1.0);

      // Get the vertices & point IDs;
      for(int i = 0; i < 4; i++)
	ids[i] = getID(tagId,(pEntity) R_vertex(r,i));
      if (order == 2)
	for(int i = 0; i < 6; i++)
	  ids[4+i] = getID(tagId,(pEntity) E_point(R_edge(r,i),0));//MSH_TET_EDGE[i]
      
      Eigen::MatrixXd block(dim*m,dim*m);
      block.setZero();

      for(int iqp = 0; iqp < nqp; iqp++){
	// Get the jacobian
	Map_jacInv(map,&qp[3*iqp],&tmpMat[0]);
      
	for(int k = 0; k < dim; k++)
	  for(int l = 0; l < dim; l++)
	    invJac(k,l) = tmpMat[3*k+l];

	const double detJac = 1./invJac.determinant();
	if (detJac <= 0.){
	  cout << detJac << endl;
	  throw runtime_error("negative jacobian");
	}
      
	for(int i = 0; i < m; i++)
	  for(int k = 0; k < dim; k++)
	    D(i,k) = dphi[iqp*m*dim+i*dim+k];
	G = D * invJac;
	
	const double Jxw = tetQuadrature.wp(iqp) * detJac;

	// Material properties
	const double E = 1.;// /fabs(detJac);
	const double nu = 0.3;
	const double lambda = E*nu / (1.+nu) / (1 - 2.*nu);
	const double mu = E/2./(1.+nu);

	// Build the block
	for(int i = 0; i < m; i++){
	  for(int j = 0; j < m; j++){
	    for(int k = 0; k < dim; k++){
	      const int iblock = i*dim + k;
	      // const int imat = ids[i]*dim + k;
	      for(int l = 0; l < dim; l++){
		const int jblock = j*dim + l;
		// const int jmat = ids[j]*dim + l;
		double val = 0.;
		val  += lambda * G(i,k) * G(j,l);
		if (k == l)
		  for(int kk = 0; kk < dim; kk++)
		    val += mu * G(i,kk) * G(j,kk);
		val += mu * G(i,l) * G(j,k);
		block(iblock,jblock) += Jxw * val;
		// mat.coeffRef(imat,jmat) += Jxw * val;
	      }
	    }
	  }
	}
      }

      Map_delete(map);

      vector<int> cols(m*dim),idx(m*dim);
      vector<double> vals(m*dim);

      for(int i = 0; i < m; i++){
	for(int k = 0; k < dim; k++){
	  const int iblock = i*dim + k;
	  const int imat = ids[i]*dim + k;
	  for(int j = 0; j < m; j++){
      	    for(int l = 0; l < dim; l++){
      	      const int jblock = j*dim + l;
      	      const int jmat = ids[j]*dim + l;
	      cols[jblock] = jmat;
	      vals[jblock] = block(iblock,jblock);
	      // mat.coeffRef(imat,jmat) += block(iblock,jblock);
      	    }
      	  }
	  for(int l = 0; l < m*dim; l++) idx[l] = l;
	  sort(idx.begin(), idx.end(), IdxCompare(cols));
	  
	  const int j0 = iptr[imat];
	  const int j1 = iptr[imat+1];
	  int l = 0;
	  for(int j = j0; j < j1; j++){
	    //cout << idxptr[j] << " " ;
	    if (idxptr[j] == cols[idx[l]]) {
	      vptr[j] += vals[idx[l]];
	      l++;
	      if (l == m*dim) break;
	    }
	  }
	  //cout << endl;
	  if (l != m*dim) {
	    cout << i  << " " << cols[idx[l]] << endl;
	    throw runtime_error("error in insertion");
	  }


	  // int l = 0;
	  // for (SparseMatrix::InnerIterator it(mat,imat); it; ++it) {
	  //   if (it.col() == cols[idx[l]]) {
	  //     it.valueRef() += vals[idx[l]];
	  //     l++;
	  //     if (l == m*dim) break;
	  //   }
	  // }
	  // if (l != m*dim) {
	  //   throw runtime_error("error in insertion");
	  // }
      	}
      }
    }
    RIter_delete(rIt);
  }

  // Impose the boundary conditions
  int icol;
  for(int irow = 0; irow < mat.outerSize(); irow++)
    if (isBc[irow] == 1)
      for (SparseMatrix::InnerIterator it(mat,irow); it; ++it) {
	icol = it.col();
	if (irow == icol)
	  mat.coeffRef(irow,irow) = 1.;
	else{
	  // make the matrix symmetric
	  if (isBc[icol] != 1)
	    rhs(icol) -= it.value() * rhs(irow);
	  mat.coeffRef(irow,icol) = 0.;
	  mat.coeffRef(icol,irow) = 0.;
	}
      }

  mat.makeCompressed();
  cout << mat.nonZeros() << endl;
}

bool isRotation(const pTransform transfo){
  double M[16];
  T_copyAffineTransformation(transfo,&(M[0]));
  for(int i = 0; i < 3; i++){
    for(int j = 0; j < 3; j++){
      if ((i == j) && fabs(M[4*i+j] - 1.) > 1e-10)
	return true;
      else if ((i!=j) && fabs(M[4*i+j]) > 1e-10)
	return true;
    }
  }
  return false;
}

inline int npos(vector<int> &arr){
  int m = 0;
  for(size_t i = 0; i < arr.size(); i++)
    if (arr[i] >= 0) m++;
  return m;
}

void getPeriodicityMat(const pMesh mesh, map<pTransform, vector<pair<int,int> > > &pairs, SparseMatrix &P1){

  const int dim    = M_dim(mesh);

  const int nDofs = M_numVertices(mesh) + M_numPoints(mesh);
  vector<int> tmp(nDofs);
  for(int i = 0; i < nDofs; i++) tmp[i] = i;

  const pGModel model = M_model(mesh);
  int nPerDofs = nDofs, nnz = 0;
  // build the new IDs
  {
    pTransform t;
    pGEntity pGEfrom, pGEto;
    GTIter tIt=GM_transformationIter(model);
    while ( ( t = GTIter_next(tIt,pGEfrom, pGEto) ) ){
      bool rot = isRotation(t);
      for(size_t i = 0; i < pairs[t].size(); i++){
	if ((tmp[pairs[t][i].first] < 0) && (tmp[pairs[t][i].second] >= 0)) {
	  tmp[pairs[t][i].second] =tmp[pairs[t][i].first];
	  nPerDofs -= 1;
	  nnz += rot? dim*dim : dim;
	} else if ((tmp[pairs[t][i].first] >= 0) && (tmp[pairs[t][i].second] < 0)) {
	  tmp[pairs[t][i].first] = tmp[pairs[t][i].second];
	  nPerDofs -= 1;
	  nnz += rot? dim*dim : dim;
	} else if ((tmp[pairs[t][i].first] >= 0) && (tmp[pairs[t][i].second] >= 0)) {
	  tmp[pairs[t][i].second] = -1-pairs[t][i].first;
	  nPerDofs -= 1;
	  nnz += rot? dim*dim : dim;
	}
      }
    }
    GTIter_delete(tIt);
  }
  nnz += dim * nPerDofs;
  
  vector<Eigen::Triplet<double> > triplets(nnz);

  int newId = 0, idx = 0;
  for(int i = 0; i < nDofs; i++)
    if (tmp[i] >= 0 ){
      tmp[i] = newId++;
      for(int j = 0; j < dim; j++)
	triplets[idx++] = Eigen::Triplet<double>(dim*i+j,dim*tmp[i]+j,1.);
    }
  
  cout << dim*newId << " " << idx << " " << dim * nPerDofs  << " " << dim * nDofs<< endl;

  double M[16];
  pTransform t;
  pGEntity pGEfrom, pGEto;
  GTIter tIt=GM_transformationIter(model);
  while ( ( t = GTIter_next(tIt,pGEfrom, pGEto) ) ){
    bool rot = isRotation(t);
    if (rot)
      T_copyAffineTransformation(t,&(M[0]));
    for(size_t i = 0; i < pairs[t].size(); i++){
      const int i0 = pairs[t][i].first;
      const int i1 = pairs[t][i].second;
      int i00;
      if (tmp[i0] >= 0)
	i00 = tmp[i0];
      else
	i00 = tmp[-1-tmp[i0]];
      if ((i00 < 0) || (i00 >= nPerDofs)) {
	cout << i00 << endl;
	throw;
      }
      if ((i1 < 0) || (i1 >= nDofs)) {
	cout << i1 << endl;
	throw;
      }
      if (rot){
	for(int k = 0; k < dim; k++)
	  for(int j = 0; j < dim; j++)
	    triplets[idx++] = Eigen::Triplet<double>(dim*i1+j  ,dim*i00+k,M[4*j+k]);
      } else {
	for(int j = 0; j < dim; j++)
	  triplets[idx++] = Eigen::Triplet<double>(dim*i1+j,dim*i00+j,1.);
      }
    }
  }
  GTIter_delete(tIt);

  cout << idx << " " << nnz << endl;
  P1.resize(nDofs*dim,nPerDofs*dim);
  P1.setFromTriplets(triplets.begin(), triplets.end());

}

void deformMesh(pMesh mesh, Eigen::VectorXd &sol){

  const int dim     = M_dim(mesh);
  pMeshDataId tagId = MD_lookupMeshDataId("ID");

  double xyz[3];
  pVertex v;
  VIter vIt= M_vertexIter(mesh);
  while ( ( v = VIter_next(vIt) ) ) {
    const int id = getID(tagId, (pEntity) v);
    V_coord(v,xyz);
    for(int i = 0; i < dim; i++){
      xyz[i] += sol[id*dim+i];
    }
    V_setCoord(v,xyz);
  }
  VIter_delete(vIt);

  // edges
  pEdge e;
  EIter eIt= M_edgeIter(mesh);
  while ( ( e = EIter_next(eIt) ) ){
    for(int i = 0; i < E_numPoints(e); i++) {
      const pPoint p = E_point(e,i);
      const int id = getID(tagId, (pEntity) p);
      const double x = P_x(p)+sol[id*dim], y = P_y(p)+sol[id*dim+1], z = (dim == 3) ? P_z(p)+sol[id*dim+2]: 0.;
      P_setPos(p,x,y,z);
    }
  }
  EIter_delete(eIt);

}

    // while ( ( r = RIter_next(rIt) ) ) {
    //   // Mapping
    //   pMapping map = Map_create((pEntity) r, 1.0);

    //   Eigen::MatrixXd X(m,dim);
    //   double coords[3];
    //   // Get the vertices & point IDs;
    //   for(int i = 0; i < 4; i++) {
    // 	ids[i] = getID(tagId,(pEntity) R_vertex(r,i));
    // 	V_coord(R_vertex(r,i),coords);
    // 	for(int k = 0; k < dim; k++)
    // 	  X(i,k) = coords[k];
    //   }
    //   if (order == 2)
    // 	for(int i = 0; i < 6; i++){
    // 	  ids[4+i] = getID(tagId,(pEntity) E_point(R_edge(r,i),0));//MSH_TET_EDGE[i]
    // 	  V_coord(E_point(R_edge(r,i),0),coords);
    // 	  for(int k = 0; k < dim; k++)
    // 	    X(4+i,k) = coords[k];
    // 	  // ids[4+i] = getID(tagId,(pEntity) E_point(R_edge(r,MSH_TET_EDGE[i]),0));//
    // 	}

    //   Eigen::MatrixXd block(dim*m,dim*m);
    //   block.setZero();
    //   Eigen::MatrixXd block2(dim*m,dim*m);
    //   block2.setZero();
    //   Eigen::MatrixXd D(m,dim),G(m,dim);
    //   for(int iqp = 0; iqp < nqp; iqp++){
    // 	for(int i = 0; i < m; i++)
    // 	  for(int k = 0; k < dim; k++)
    // 	    D(i,k) = dphi[iqp*m*dim+i*dim+k];
	
    // 	// double a[3];
    // 	// tetQuadrature.qp(iqp,a);
    // 	// double a0 = a[0], a1 = a[1], a2 = a[2], a3 = 1- a0 - a1 - a2;
    // 	// Eigen::MatrixXd D2(m,dim);

    // 	// D2 << 
    // 	//   -4*a3+1,-4*a3+1,-4*a3+1,
    // 	//   4*a0-1,0,0,
    // 	//   0,4*a1-1,0,
    // 	//   0,0,4*a2-1,
    // 	//   4*(a3-a0),-4*a0,-4*a0,
    // 	//   4*a1,4*a0,0,
    // 	//   -4*a1,4*(a3-a1),-4*a1,
    // 	//   -4*a2,-4*a2,4*(a3 - a2),	  
    // 	//   4*a2,0,4*a0,
    // 	//   0,4*a2,4*a1;
    // 	// cout << D  - D2 << endl << endl;
    // 	// cout << D2 << endl;
    // 	// cout << (X.transpose() * D2).determinant() << endl;
    // 	// throw;
	
	  
	  
    // 	// // Get the jacobian
    // 	// Map_jacInv(map,&qp[3*iqp],&tmpMat[0]);

    // 	// for(int k = 0; k < dim; k++)
    // 	//   for(int l = 0; l < dim; l++)
    // 	//     invJac(k,l) = tmpMat[3*k+l];
    // 	// cout << invJac.inverse() << endl;
    // 	// cout << X.transpose() * D << endl;
    // 	// throw;
    // 	// invJac << 1.,0.,0.,0.,1.,0.,0.,0.,1.;
    // 	// X << 
    // 	//   0.,0.,0.,
    // 	//   1.,0.,0.,
    // 	//   0.,1.,0.,
    // 	//   0.,0.,1.,
    // 	//   .5,0.,0.,
    // 	//   .5,.5,0.,
    // 	//   0.,.5,0.,
    // 	//   0.,0.,.5,
    // 	//   .5,0.,.5,
    // 	//   0.,.5,.5;
    // 	// for(int i = 0; i < m; i++){
    // 	//   double a0 = X(i,0), a1 = X(i,1), a2 = X(i,2), a3 = 1- a0 - a1 - a2;
    // 	//   cout << a3*(2*a3-1) << " " ;
    // 	//   cout << a0*(2*a0-1) << " " ;
    // 	//   cout << a1*(2*a1-1) << " " ;
    // 	//   cout << a2*(2*a2-1) << " " ;
    // 	//   cout << 4*a3*a0 << " " ;
    // 	//   cout << 4*a0*a1 << " " ;
    // 	//   cout << 4*a3*a1 << " " ;
    // 	//   cout << 4*a3*a2 << " " ;
    // 	//   cout << 4*a0*a2 << " " ;
    // 	//   cout << 4*a1*a2 << endl;
    // 	// }
    // 	// throw;

    // 	jac = X.transpose() * D;
    // 	invJac = jac.inverse();

    // 	// cout << jac << endl;
    // 	// throw;
    // 	const double detJac = 1./invJac.determinant();
    // 	if (detJac <= 0.){
    // 	  cout << detJac << endl;
    // 	  throw;
    // 	}

    // 	G = D * invJac;
	
    // 	const double Jxw = tetQuadrature.wp(iqp) * detJac;
    // 	// // Gradient in th element
    // 	// for(int i = 0; i < m; i++){
    // 	//   for(int k = 0; k < dim; k++)
    // 	//     v0[k] = dphi[iqp*m*dim+i*dim+k];
    // 	//   v1 = invJac.transpose() * v0 ;
    // 	//   // v1 = invJac * v0 ;
    // 	//   for(int k = 0; k < dim; k++)
    // 	//     dphi_el[i*dim+k] = v1[k];
    // 	// }
    // 	// Material properties
    // 	const double E = 1.;// /fabs(detJac);
    // 	const double nu = 0.3;
    // 	const double lambda = E*nu / (1.+nu) / (1 - 2.*nu);
    // 	const double mu = E/2./(1.+nu);

    // 	// // Build the block
    // 	// for(int i = 0; i < m; i++){
    // 	//   for(int j = 0; j < m; j++){
    // 	//     for(int k = 0; k < dim; k++){
    // 	//       const int iblock = i*dim + k;
    // 	//       // const int imat = ids[i]*dim + k;
    // 	//       for(int l = 0; l < dim; l++){
    // 	//     	const int jblock = j*dim + l;
    // 	//     	// const int jmat = ids[j]*dim + l;
    // 	//     	double val = 0.;
    // 	//     	val  += lambda * G(i,k) * G(j,l);
    // 	//     	if (k == l)
    // 	//     	  for(int kk = 0; kk < dim; kk++)
    // 	//     	    val += mu * G(i,kk) * G(j,kk);
    // 	//     	val += mu * G(i,l) * G(j,k);
    // 	//     	block2(iblock,jblock) += Jxw * val;
    // 	//     	// mat.coeffRef(imat,jmat) += Jxw * val;
    // 	//       }
    // 	//     }
    // 	//   }
    // 	// }
	
    // 	// Method 2
    // 	//Eigen::MatrixXd A,D,G,B;
    // 	Eigen::MatrixXd A(6,6);
    // 	A << 
    // 	  1-nu, nu, nu, 0,0,0,
    // 	  nu, 1-nu, nu, 0,0,0,
    // 	  nu,nu, 1-nu , 0,0,0,
    // 	  0 , 0, 0 , (1-2*nu)/2,0,0,
    // 	  0 , 0, 0 , 0,(1-2*nu)/2,0,
    // 	  0 , 0, 0 , 0,0,(1-2*nu)/2;
    // 	A  *= E/((1+nu)*(1-2*nu));

	
    // 	// Eigen::MatrixXd G = D * invJac;
    // 	Eigen::MatrixXd B(6,m*dim);
    // 	B << 
    // 	  G(0,0),0,0,
    // 	  G(1,0),0,0,
    // 	  G(2,0),0,0,
    // 	  G(3,0),0,0,
    // 	  G(4,0),0,0,
    // 	  G(5,0),0,0,
    // 	  G(6,0),0,0,
    // 	  G(7,0),0,0,
    // 	  G(8,0),0,0,
    // 	  G(9,0),0,0,
    // 	  0,G(0,1),0,
    // 	  0,G(1,1),0,
    // 	  0,G(2,1),0,
    // 	  0,G(3,1),0,
    // 	  0,G(4,1),0,
    // 	  0,G(5,1),0,
    // 	  0,G(6,1),0,
    // 	  0,G(7,1),0,
    // 	  0,G(8,1),0,
    // 	  0,G(9,1),0,
    // 	  0,0,G(0,2),
    // 	  0,0,G(1,2),
    // 	  0,0,G(2,2),
    // 	  0,0,G(3,2),
    // 	  0,0,G(4,2),
    // 	  0,0,G(5,2),
    // 	  0,0,G(6,2),
    // 	  0,0,G(7,2),
    // 	  0,0,G(8,2),
    // 	  0,0,G(9,2),
    // 	  G(0,1),G(0,0),0,
    // 	  G(1,1),G(1,0),0,
    // 	  G(2,1),G(2,0),0,
    // 	  G(3,1),G(3,0),0,
    // 	  G(4,1),G(4,0),0,
    // 	  G(5,1),G(5,0),0,
    // 	  G(6,1),G(6,0),0,
    // 	  G(7,1),G(7,0),0,
    // 	  G(8,1),G(8,0),0,
    // 	  G(9,1),G(9,0),0,
    // 	  G(0,2),0,G(0,0),
    // 	  G(1,2),0,G(1,0),
    // 	  G(2,2),0,G(2,0),
    // 	  G(3,2),0,G(3,0),
    // 	  G(4,2),0,G(4,0),
    // 	  G(5,2),0,G(5,0),
    // 	  G(6,2),0,G(6,0),
    // 	  G(7,2),0,G(7,0),
    // 	  G(8,2),0,G(8,0),
    // 	  G(9,2),0,G(9,0),
    // 	  0,G(0,2),G(0,1),
    // 	  0,G(1,2),G(1,1),
    // 	  0,G(2,2),G(2,1),
    // 	  0,G(3,2),G(3,1),
    // 	  0,G(4,2),G(4,1),
    // 	  0,G(5,2),G(5,1),
    // 	  0,G(6,2),G(6,1),
    // 	  0,G(7,2),G(7,1),
    // 	  0,G(8,2),G(8,1),
    // 	  0,G(9,2),G(9,1);
    // 	block += B.transpose()*A*B*Jxw;
    //   }
    //   // cout << block.norm() << " " << (block - block2).norm() << endl;
    //   // // cout << block2 << endl;
    //   // throw;
