#include "entityIds.h"

using namespace std;
using namespace MAd;


void setIDs(pMesh mesh){

  int vId = 0;
  pMeshDataId id = MD_lookupMeshDataId("ID");
  int maxPtPerE = 0,
    maxPtPerF = 0,
    maxPtPerR = 0;

  // vertices
  pVertex v;
  VIter vIt= M_vertexIter(mesh);
  while ( ( v = VIter_next(vIt) ) ){
    EN_attachDataInt((pEntity) v, id, vId++);
  }
  VIter_delete(vIt);

  // edges
  pEdge e;
  EIter eIt= M_edgeIter(mesh);
  while ( ( e = EIter_next(eIt) ) ){
    maxPtPerE = max(maxPtPerE, E_numPoints(e));
    for(int i = 0; i < E_numPoints(e); i++) {
      const pPoint p = E_point(e,i);
      EN_attachDataInt((pEntity) p, id, vId++);
    }
  }
  EIter_delete(eIt);

  // faces
  pFace f;
  FIter fIt= M_faceIter(mesh);
  while ( ( f = FIter_next(fIt) ) ){
    maxPtPerF = max(maxPtPerF, F_numPoints(f));
   for(int i = 0; i < F_numPoints(f); i++) {
      const pPoint p = F_point(f,i);
      EN_attachDataInt((pEntity) p, id, vId++);
   }
  }
  FIter_delete(fIt);

  // regions
  pRegion r;
  RIter rIt= M_regionIter(mesh);
  while ( ( r = RIter_next(rIt) ) ){
    maxPtPerR = max(maxPtPerR, R_numPoints(r));
    for(int i = 0; i < R_numPoints(r); i++) {
      const pPoint p = R_point(r,i);
      EN_attachDataInt((pEntity) p, id, vId++);
    }
  }
  RIter_delete(rIt);

  cout << "Max # of HO points per edge  : " << maxPtPerE << endl;
  cout << "Max # of HO points per face  : " << maxPtPerF << endl;
  cout << "Max # of HO points per region: " << maxPtPerR << endl;

}

void getPeriodicIDPairs(const pMesh mesh, map<pTransform,vector<pair<int,int> > >&pairs) {

  const pMeshDataId tagId = MD_lookupMeshDataId("ID");

  pairs.clear();

  const pGModel model = M_model(mesh);
  pTransform t;
  pGEntity pGEfrom, pGEto;
  GTIter tIt=GM_transformationIter(model);
  while ( ( t = GTIter_next(tIt,pGEfrom, pGEto) ) ){
    const int d = GEN_type(pGEfrom);
    const int tagFrom = GEN_tag(pGEfrom);
    const int tagTo   = GEN_tag(pGEto);
    if (tagFrom < tagTo) {
      pVertex v;
      VIter   vIt= M_classifiedVertexIter(mesh, pGEfrom, 0);
      while ( ( v = VIter_next(vIt) ) ){
	int iFrom = getID(tagId,(pEntity) v);
	pVertex v2 = getPeriodicVertex(mesh, t, v);
	if (!v2)
	  throw runtime_error("unable to find the corresponding node");
	int iTo = getID(tagId,(pEntity) v2);
	pairs[t].push_back(make_pair(iFrom,iTo));
      }
      VIter_delete(vIt);

      pEdge e;
      EIter eIt= M_classifiedEdgeIter(mesh, pGEfrom, 0);
      while ( ( e = EIter_next(eIt) ) ){
	pVertex v00 = E_vertex(e,0);
	pVertex v10 = E_vertex(e,1);
	pVertex v01 = getPeriodicVertex(mesh, t, v00);
	pVertex v11 = getPeriodicVertex(mesh, t, v10);
	pEdge   e1  = M_edge(mesh,V_id(v01),V_id(v11));
	if (!e1) throw runtime_error("unable to find the matching edge");
	for(int i = 0 ; i < E_numPoints(e); i++) {
	  if (i > 0) throw runtime_error("more than 1 point");
	  pPoint p0   = E_point(e,i);
	  pPoint p1   = E_point(e1,i);
	  int iFrom   = getID(tagId,(pEntity) p0);
	  int iTo     = getID(tagId,(pEntity) p1);
	  pairs[t].push_back(make_pair(iFrom,iTo));
	}
      }
      EIter_delete(eIt);

    }
  }
  GTIter_delete(tIt);
  
}


