#include "gmshIO.h"

using namespace MAd;

void SaveGmshMeshHighOrderPerso(const pMesh mesh,const char *filename) {
    
  std::ofstream gmsh(filename);
  gmsh.precision(16);
      
  // --- count classified elements
      
  long int nbElmts = 0;
      
  VIter vIter = M_vertexIter(mesh);
  while (pVertex pv=VIter_next(vIter)) if (GEN_type(V_whatIn(pv))==0) nbElmts++;

  EIter eIter = M_edgeIter(mesh);
  while (pEdge pe=EIter_next(eIter))   if (GEN_type(E_whatIn(pe))==1) nbElmts++;

  FIter fIter = M_faceIter(mesh);
  while (pFace pf=FIter_next(fIter))   if (GEN_type(F_whatIn(pf))==2) nbElmts++;
      
  RIter rIter = M_regionIter(mesh);
  while (pRegion pr=RIter_next(rIter)) if (GEN_type((pGEntity) R_whatIn(pr))==3) nbElmts++;
    
  gmsh << "$MeshFormat\n2.1 0 8\n$EndMeshFormat\n";
    
  // --- Writing the points -------------------------------------------------
      
  int mxVx = -1;
  gmsh <<  "$Nodes\n";
  gmsh << M_numVertices(mesh) + M_numPoints(mesh) << "\n";
  VIter_reset(vIter);
  while (pVertex pv = VIter_next(vIter)) {
    gmsh << EN_id((pEntity) pv) << " " 
	 << P_x(pv) << " " 
	 << P_y(pv) << " " 
	 << P_z(pv) << "\n";
    mxVx = std::max(EN_id((pEntity) pv),mxVx);
  }
  VIter pIter = M_pointIter(mesh);
  while (pVertex pv = VIter_next(pIter)) {
    gmsh << EN_id((pEntity) pv) << " " 
	 << P_x(pv) << " " 
	 << P_y(pv) << " " 
	 << P_z(pv) << "\n";
  } 
  gmsh << "$EndNodes\n";      
      
  // --- writing the elements ----------------------------------------------

  gmsh << "$Elements\n" << nbElmts << "\n";

  int elmtId = 1;

  // --- write nodes
  VIter_reset(vIter);
  while (pVertex pv = VIter_next(vIter)) {
    pGEntity pGE = V_whatIn(pv);
    if (GEN_type(pGE) == 0) {
        
      int type = EN_mshTag((pEntity) pv);
      int elmtTag = GEN_tag(pGE);
      int physTag = GEN_physTag(pGE);
        
      gmsh << elmtId++ << " " << type << " " << 2 << " "
	   << (physTag ? physTag:elmtTag) << " " << elmtTag << " " 
	   << EN_id((pEntity) pv) << "\n";
    }
  }
    
  // --- write edges 
      
  EIter_reset(eIter);
  while (pEdge pe = EIter_next(eIter)) {
    pGEntity pGE = E_whatIn(pe);
    if (GEN_type(pGE) == 1) {
        
      int type = EN_mshTag((pEntity) pe);
      int elmtTag = GEN_tag(pGE);
      int physTag = GEN_physTag(pGE);
        
      gmsh << elmtId++ << " " << type << " " << 2 << " "
	   << (physTag ? physTag:elmtTag) << " " << elmtTag << " "
	   << EN_id((pEntity) E_vertex(pe,0)) << " " 
	   << EN_id((pEntity) E_vertex(pe,1));
      for (int i=0;i<E_numPoints(pe);i++) {
	gmsh << " " << EN_id((pEntity) E_point(pe,i));
      }
      gmsh << "\n";
    }
  }
      
  // --- write faces 
      
  FIter_reset(fIter);
  while (pFace pf = FIter_next(fIter)) {
    pGEntity pGE = F_whatIn(pf);
    if (GEN_type(pGE) == 2) {
      
      int type = EN_mshTag((pEntity) pf);
      int elmtTag = GEN_tag(pGE);
      int physTag = GEN_physTag(pGE);
      gmsh << elmtId++ << " " << type << " " << 2 << " "
	   << (physTag ? physTag:elmtTag) << " " << elmtTag;
          
      std::vector<pVertex> pts;

      // --- main vertices

      for (int i=0;i<F_numVertices(pf);i++) pts.push_back(F_vertex(pf,i));
        
      // --- edge nodes 
      int i0,i1;
      for (int i=0;i<F_numEdges(pf);i++) {
	if ((type == MSH_TRI_3) || (type == MSH_TRI_6) ) {
	  i0 = MSH_TRI_EDGE_NODES[i][0];
	  i1 = MSH_TRI_EDGE_NODES[i][1];
	} else if ((type == MSH_QUA_4) || (type == MSH_QUA_8) ) {
	  i0 = MSH_QUA_EDGE_NODES[i][0];
	  i1 = MSH_QUA_EDGE_NODES[i][1];
	} else
	  throw std::runtime_error("unknown element type");
	// pEdge pe = R_edge(pr,iMadEdge);
	int j0 = V_id(F_vertex(pf,i0));
	int j1 = V_id(F_vertex(pf,i1));
	pEdge pe = M_edge(mesh,j0,j1);
	// pEdge pe = F_edge(pf,i);
	if (E_numPoints(pe) == 1)
	  pts.push_back(E_point(pe,0));
	else if (E_numPoints(pe) > 1)
	  throw std::runtime_error("cannot write this element type");
      }
        
      // // --- face nodes q
        
      // for (int i=0;i<F_numPoints(pf);i++) {
      // 	if (F_numPoints(pf) == 1)
      // 	  pts.push_back(F_point(pf,0));
      // 	else if (F_numPoints(pf) > 0)
      // 	  throw std::runtime_error("cannot write this element type");
      // }
        
      // --- 
        
      std::vector<pVertex>::iterator pIter = pts.begin();
      for (;pIter != pts.end();pIter++) gmsh << " " << EN_id((pEntity) (*pIter));

      // ---
        
      gmsh << "\n";
    }
  }
    
  // --- write regions        

  RIter_reset(rIter);
  while (pRegion pr = RIter_next(rIter)) {
    pGEntity pGE = (pGEntity) R_whatIn(pr);

    int type = EN_mshTag((pEntity) pr);
    int elmtTag = GEN_tag(pGE);
    int physTag = GEN_physTag(pGE);
    gmsh << elmtId++ << " " << type << " " << 2 << " "
  	 << (physTag ? physTag:elmtTag) << " " << elmtTag;
      
    std::vector<pVertex> pts;


    // --- principal vertices
      
    for (int i=0;i<R_numVertices(pr);i++) pts.push_back(R_vertex(pr,i));
    
    // --- edge nodes
    int iMadEdge;
    int i0,i1;
    for (int iGmshEdge=0;iGmshEdge<R_numEdges(pr);iGmshEdge++) {
      if ((type == MSH_TET_4) || (type == MSH_TET_10) ) {
	// iMadEdge = MSH_TET_EDGE[iGmshEdge];
	i0 = MSH_TET_EDGE_NODES[iGmshEdge][0];
	i1 = MSH_TET_EDGE_NODES[iGmshEdge][1];
      } else if ((type == MSH_PRI_6) || (type == MSH_PRI_15) ) {
	// iMadEdge = MSH_PRI_EDGE[iGmshEdge];
	i0 = MSH_PRI_EDGE_NODES[iGmshEdge][0];
	i1 = MSH_PRI_EDGE_NODES[iGmshEdge][1];
      } else
	throw std::runtime_error("unknown element type");
      // pEdge pe = R_edge(pr,iMadEdge);
      int j0 = V_id(R_vertex(pr,i0));
      int j1 = V_id(R_vertex(pr,i1));
      pEdge pe = M_edge(mesh,j0,j1);
      if (E_numPoints(pe) == 1)
	pts.push_back(E_point(pe,0));
      else if (E_numPoints(pe) > 1)
	throw std::runtime_error("cannot write this element type");
    }
    
    // // --- face nodes

    // for (int iFace=0;iFace<R_numFaces(pr);iFace++) {
    //   pFace pf = R_face(pr,iFace);
    //   if (F_numPoints(pf) == 1)
    // 	pts.push_back(F_point(pf,0));
    //   else if (F_numPoints(pf) > 1)
    // 	throw std::runtime_error("cannot write this element type");
    // }
      
    // // --- internal nodes

    // for (int i=0;i<R_numPoints(pr);i++) {
    //   if (R_numPoints(pr) == 1)
    // 	pts.push_back(R_point(pr,0));
    //   else if (R_numPoints(pr) > 1)
    // 	throw std::runtime_error("cannot write this element type");
    // }
      
    // --- now write
      
    std::vector<pVertex>::iterator pIter = pts.begin();
    for (;pIter!=pts.end();++pIter) gmsh << " " << EN_id((pEntity)(*pIter));
      
    gmsh << "\n";

  }
      
  // --- close element region 
      
  gmsh <<  "$EndElements\n"; 
      
  // --- write periodic node section ----------------------------------------
    

  pGModel model = M_model(mesh);
  pMeshDataId periodicTag = MD_lookupMeshDataId("PeriodicVertexID");
  int nbTransformations = GM_numTransformations(model);    
    
  if (nbTransformations) {  

    gmsh << "$Periodic\n";
    gmsh << nbTransformations << "\n";
      
    GTIter gtIter = GM_transformationIter(model);

    pGEntity ge1;
    pGEntity ge2;
      
    double xyz0[3],xyz1[3];
    const double tol = 1e-10;

    while (pTransform gt = GTIter_next(gtIter,ge2,ge1)) {
        
      double mat[16];
      T_copyAffineTransformation(gt,mat);

      gmsh << GEN_type(ge1) << " " 
	   << GEN_tag(ge1)  << " " 
	   << GEN_tag(ge2)  << "\n";
        
      gmsh << "Affine";
      for (int i=0;i<16;i++) gmsh << " " << mat[i];
      gmsh << "\n";

      // Get all the vertices belonging to the GE + closure
      int entDim = GEN_type(ge1);
      std::vector<int> tmpMask(mxVx+1,0);
      std::vector<std::pair<int,int> > tmpMap;
      if (entDim == 0) {
	VIter viter = M_classifiedVertexIter(mesh,ge1,false);
	while (pVertex pv = VIter_next(viter)) {
	  void *temp_ptr; 
	  EN_getDataPtr((pEntity) pv , periodicTag, &temp_ptr);
	  std::map<int,std::vector<int> >* conn = (std::map<int,std::vector<int> >*) temp_ptr;
	  for (std::map<int,std::vector<int> >::iterator cIter = conn->begin();cIter!=conn->end();++cIter){
	    int vId2 = cIter->first;
	    pVertex v2 = M_vertex(mesh,vId2);
	    pGEntity pGEtmp = V_whatIn(v2);
	    const int id1 = EN_id((pEntity) pv), id2 = EN_id((pEntity) v2);
	    if ((pGEtmp == ge2) && (tmpMask[id1] == 0)){
	      tmpMap.push_back(std::make_pair(id1,id2));
	      tmpMask[id1] = 1;
	    }
	  }
	}
	VIter_delete(viter);
      } else if (entDim == 1) {
	EIter eiter = M_classifiedEdgeIter(mesh,ge1,false);
	while (pEdge pe = EIter_next(eiter)) {
	  for(int i = 0; i < E_numVertices(pe); i++){
	    pVertex pv = E_vertex(pe,i);
	    V_coord(pv,xyz0);
	    void *temp_ptr; 
	    EN_getDataPtr((pEntity) pv , periodicTag, &temp_ptr);
	    std::map<int,std::vector<int> >* conn = (std::map<int,std::vector<int> >*) temp_ptr;
	    for (std::map<int,std::vector<int> >::iterator cIter = conn->begin();cIter!=conn->end();++cIter){
	      int vId2 = cIter->first;
	      pVertex v2 = M_vertex(mesh,vId2);
	      V_coord(v2,xyz1);
	      T_transformCoordinates(gt,xyz1);
	      double err = 0.;
	      for(int l = 0; l < 3; l++)
	      	err += (xyz0[l] - xyz1[l])*(xyz0[l] - xyz1[l]);
		      
	      const int id1 = EN_id((pEntity) pv), id2 = EN_id((pEntity) v2);

	      if ((err < tol*tol) && (tmpMask[id1] == 0)){
		tmpMap.push_back(std::make_pair(id1,id2));
		tmpMask[id1] = 1;
	      }
	    }
	  }
	}
	EIter_delete(eiter);
      } else if (entDim == 2) {
	FIter fiter = M_classifiedFaceIter(mesh,ge1,false);
	while (pFace pf = FIter_next(fiter)) {
	  for(int i = 0; i < F_numVertices(pf); i++){
	    pVertex pv = F_vertex(pf,i);
	    V_coord(pv,xyz0);
	    void *temp_ptr; 
	    EN_getDataPtr((pEntity) pv , periodicTag, &temp_ptr);
	    std::map<int,std::vector<int> >* conn = (std::map<int,std::vector<int> >*) temp_ptr;
	    for (std::map<int,std::vector<int> >::iterator cIter = conn->begin();cIter!=conn->end();++cIter){
	      int vId2 = cIter->first;
	      pVertex v2 = M_vertex(mesh,vId2);
	      V_coord(v2,xyz1);
	      T_transformCoordinates(gt,xyz1);
	      double err = 0.;
	      for(int l = 0; l < 3; l++)
		err += (xyz0[l] - xyz1[l])*(xyz0[l] - xyz1[l]);
		      
	      const int id1 = EN_id((pEntity) pv), id2 = EN_id((pEntity) v2);
	      if ((err < tol*tol) && (tmpMask[id1] == 0)){
		tmpMap.push_back(std::make_pair(id1,id2));
		tmpMask[id1] = 1;
	      }
	    }
	  }
	}
	FIter_delete(fiter);
      }
      gmsh << tmpMap.size() << std::endl;
      for(size_t i = 0; i < tmpMap.size(); i++) {
	gmsh << tmpMap[i].first << " " << tmpMap[i].second << std::endl;
      }

      // VIter pvIter = M_classifiedVertexIter(mesh,ge1,false);
        
      // std::map<int,int> vtxPairs;
        
      // while (pVertex pv1 = VIter_next(pvIter)) {
      // 	void* tmp_periodic;
      // 	if (EN_getDataPtr((pEntity) pv1,periodicTag,&tmp_periodic)) {
      // 	  std::map<int,std::vector<int> > *  conn = 
      // 	    (std::map<int,std::vector<int> >*) (tmp_periodic);
      // 	  std::map<int,std::vector<int> >::iterator cIter = conn->begin();
      // 	  for (;cIter!=conn->end();cIter++) {
      // 	    int id2 = cIter->first;
      // 	    pVertex pv2 = M_vertex(mesh,id2);
      // 	    pGEntity pg2 = EN_whatIn((pEntity) pv2);
      // 	    if (pg2 == ge2) {
      // 	      vtxPairs[EN_id((pEntity) (pEntity) pv1)] = EN_id((pEntity) (pEntity) pv2);
      // 	      break;
      // 	    }
      // 	  }
      // 	}
      // }
        
      // gmsh << vtxPairs.size() << std::endl;
        
      // for (std::map<int,int>::iterator vtpIter=vtxPairs.begin();
      // 	   vtpIter != vtxPairs.end();++vtpIter) {
      // 	gmsh << vtpIter->first << " " << vtpIter->second << "\n";
      // }
    }
    GTIter_delete(gtIter);
    gmsh << "$EndPeriodic\n";
  }
  // else {

  //   VIter_reset(vIter);
  //   std::map<int,std::set<int> > corr;
  //   std::set<int> treated;

  //   while (pVertex pv = VIter_next(vIter)) {
  //     void* tmp(NULL);
  //     if (EN_getDataPtr((pEntity) pv,periodicTag,&tmp)) {

  //       int id = EN_id((pEntity) (pEntity) pv);
  //       if (treated.find(id) == treated.end()) {
            
  //         std::map<int,std::vector<int> > *  conn = 
  //           (std::map<int,std::vector<int> >*) (tmp);
            
  //         std::map<int,std::vector<int> >::iterator connIter = conn->begin();
            
  //         for (;connIter!=conn->end();connIter++) {
  //           int idc = connIter->first;
  //           corr[id].insert(idc);
  //           treated.insert(idc);
  //         }
  //       }
  //     }
  //   }

  //   gmsh << "$PeriodicNodes\n";
  //   gmsh << corr.size() << "\n";
  //   std::map<int,std::set<int> >::iterator corrIter = corr.begin();
  //   for (;corrIter!=corr.end();++corrIter) {

  //     std::set<int>& copies = corrIter->second;
        
  //     gmsh << copies.size() + 1 << " " << corrIter->first;
  //     for (std::set<int>::iterator copyIter = copies.begin();
  //          copyIter!=copies.end();++copyIter) gmsh << " " << *copyIter;
  //     gmsh << " 0\n";
  //   }
  //   gmsh << "$EndPeriodicNodes\n";
  // }
    
    

  // --- delete the iterators 
    
  VIter_delete(vIter);
  EIter_delete(eIter);
  FIter_delete(fIter);
  RIter_delete(rIter);

  // --- close the file 
    
  gmsh.close();
}
