#include "layers.h"

using namespace std;
using namespace MAd;

void addLayerElements(pMesh mesh, vector<int> &tags, vector<int> &isBc, Eigen::VectorXd &displacement, int N) {

  const int dim    = M_dim(mesh);
  const pMeshDataId tagId = MD_lookupMeshDataId("ID");
  const pGModel model = M_model(mesh);
  const pGEntity pGEin = GM_entityByTag(model, dim, 1);
  const int ndofs = displacement.size() / dim;

  int maxVertId = 0;
  {
    pVertex v;
    VIter vIt= M_vertexIter(mesh);
    while ( ( v = VIter_next(vIt) ) ) {
      maxVertId = max(V_id(v)+1,maxVertId);
    }
    VIter_delete(vIt);
  }
  int maxPointId = 0;
  {
    pEdge e;
    EIter eIt= M_edgeIter(mesh);
    while ( ( e = EIter_next(eIt) ) ){
      for(int i = 0; i < E_numPoints(e); i++) {
	const pPoint p = E_point(e,i);
	maxPointId = max(P_id(p)+1,maxPointId);
      }
    }
    EIter_delete(eIt);
  }

  int order = 1;

  map<pVertex,vector<pVertex> > newNodes;
  {
    double xyz[3], xyz_new[3], dxyz[3];
    // vertices
    pVertex v;
    VIter vIt= M_vertexIter(mesh);
    while ( ( v = VIter_next(vIt) ) ) {
      const int id = getID(tagId,(pEntity) v);
      if ((V_id(v) < maxVertId) && (isBc[dim*id] == 1)) {
  	// coordinates
  	V_coord(v,xyz);
  	// displacements
  	for(int k = 0; k < dim; k++)
  	  dxyz[k] = displacement[dim*id+k];
	if (dim == 2)
	  dxyz[2] = 0.;
  	// new nodes
	vector<pVertex> tmp(2*N);
	for(int i = 0; i < 2*N; i++) {
	  for(int k = 0; k < 3; k++)
	    xyz_new[k] = xyz[k]  - (i+1) / ((double) 2*N) * dxyz[k];	    
	  tmp[i] =  M_createV2(mesh,xyz_new,-1,pGEin);
	  EN_attachDataInt((pEntity) tmp[i], tagId, -1);
	}
	V_forceWhatIn(tmp[2*N-1],V_whatIn(v));
	V_forceWhatIn(v,pGEin);
	newNodes[ v] =  tmp;
      }      
    }
    VIter_delete(vIt);
    // edges
    pEdge e;
    EIter eIt= M_edgeIter(mesh);
    while ( ( e = EIter_next(eIt) ) ){
      for(int i = 0; i < E_numPoints(e); i++) {
	order = 2;
  	const pPoint p = E_point(e,i);
  	const int id = getID(tagId, (pEntity) p);
  	if ((P_id(p) <maxPointId) && (isBc[dim*id] > 0)) {
  	  // coordinates
	  V_coord(p,xyz);
  	  // displacements
  	  for(int k = 0; k < dim; k++)
  	    dxyz[k] = displacement[dim*id+k];
	  if (dim == 2)
	    dxyz[2] = 0.;
  	  // new nodes
	  vector<pVertex> tmp(N);
  	  for(int i = 0; i < N; i++) {
  	    for(int k = 0; k < 3; k++)
  	      xyz_new[k] = xyz[k] - (i+1) / ((double) N) * dxyz[k];
  	    tmp[i] =  M_createV2(mesh,xyz_new,-1,pGEin);
	    EN_attachDataInt((pEntity) tmp[i], tagId, -1);
	  }
	  newNodes[p] =  tmp;
  	}
      }
    }
    EIter_delete(eIt);
  }

  if (dim == 2){
    pEdge e;
    EIter eIt= M_edgeIter(mesh);
    while ( ( e = EIter_next(eIt) ) ) {
      pGEntity pGE = (pGEntity) E_whatIn(e);
      bool addElem = (getID(tagId,(pEntity) e) != -1) && (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
      if (addElem) {
	const pFace    f   = E_face(e,0); // neighbor face
  	const pGEntity pGEf = (pGEntity) F_whatIn(f);
  	const pGEntity pGEe = (pGEntity) E_whatIn(e);

  	// Side faces: test if the edges belong to faces with 2 different 2d classifications
  	pGEntity sideEdgeEnt[2] = {pGEf,pGEf};
  	for(int i = 0; i < E_numVertices(e); i++){
  	  const pVertex v = E_vertex(e,i);
  	  for(int j = 0; j < V_numEdges(v); j++){
  	    const pEdge e2 = V_edge(v,j);
  	    const pGEntity pGEe2 = (pGEntity) E_whatIn(e2);
  	    bool inSide = (GEN_type(pGEe2) == dim-1) && (!isIn(tags,GEN_tag(pGEe2)));
  	    if (inSide)
  	      sideEdgeEnt[i] = pGEe2;
  	  }
  	}
	if (order == 1) {
	  pVertex v0 = E_vertex(e,0);
	  pVertex v1 = newNodes[ E_vertex(e,0)][1];
	  pVertex v2 = newNodes[ E_vertex(e,1)][1];
	  pVertex v3 = E_vertex(e,1);
	
	  pEdge e0 = e;
	  pEdge e1 = M_createE(mesh,
			       v0,v1,
			       sideEdgeEnt[0]);
	  pEdge e2 = M_createE(mesh,
			       v1,v2,
			       pGEf);
	  pEdge e3 = M_createE(mesh,
			       v2,v3,
			       sideEdgeEnt[1]);


	  pEdge es[4] = {e0,e1,e2,e3};
	  pFace newf = M_createF(mesh,4,es,pGEf);

	  E_forceWhatIn(e2,E_whatIn(e0));
	  EN_attachDataInt((pEntity) e2, tagId, -1);
	  E_forceWhatIn(e0,pGEf);
	  E_forceWhatIn(e1,sideEdgeEnt[0]);
	  E_forceWhatIn(e3,sideEdgeEnt[1]);

	  for(int i = 1; i < N; i++) {
	    v0 = v1;
	    v3 = v2;
	    v1 = newNodes[E_vertex(e,0)][2*i+1];
	    v2 = newNodes[E_vertex(e,1)][2*i+1];
      
	    e0 = e2;
	  
	    e1 = M_createE(mesh,
			   v0,v1,
			   sideEdgeEnt[0]);
	    e2 = M_createE(mesh,
			   v1,v2,
			   pGEf);
	    e3 = M_createE(mesh,
			   v2,v3,
			   sideEdgeEnt[1]);

	    pEdge es[4] = {e0,e1,e2,e3};
	    pFace newf = M_createF(mesh,4,es,pGEf);

	    E_forceWhatIn(e2,E_whatIn(e0));
	    EN_attachDataInt((pEntity) e2, tagId, -1);
	    E_forceWhatIn(e0,pGEf);
	    E_forceWhatIn(e1,sideEdgeEnt[0]);
	    E_forceWhatIn(e3,sideEdgeEnt[1]);
	  }

	} else if (order == 2) {
	  pVertex v0 = E_vertex(e,0);
	  pVertex v1 = newNodes[ E_vertex(e,0)][1];
	  pVertex v2 = newNodes[ E_vertex(e,1)][1];
	  pVertex v3 = E_vertex(e,1);
	
	  pVertex v4 = newNodes[E_vertex(e,0)][0];
	  pVertex v5 = newNodes[E_point(e,0)][0];
	  pVertex v6 = newNodes[E_vertex(e,1)][0];

	  pEdge e0 = e;
	  pEdge e1 = M_createE(mesh,
			       v0,v1,
			       2,&v4,
			       sideEdgeEnt[0]);
	  pEdge e2 = M_createE(mesh,
			       v1,v2,
			       2,&v5,
			       pGEf);
	  pEdge e3 = M_createE(mesh,
			       v2,v3,
			       2,&v6,
			       sideEdgeEnt[1]);


	  pEdge es[4] = {e0,e1,e2,e3};
	  pFace newf = M_createF(mesh,4,es,pGEf);

	  E_forceWhatIn(e2,E_whatIn(e0));
	  EN_attachDataInt((pEntity) e2, tagId, -1);
	  E_forceWhatIn(e0,pGEf);
	  E_forceWhatIn(e1,sideEdgeEnt[0]);
	  E_forceWhatIn(e3,sideEdgeEnt[1]);

	  for(int i = 1; i < N; i++) {
	    v0 = v1;
	    v3 = v2;
	    v1 = newNodes[E_vertex(e,0)][2*i+1];
	    v2 = newNodes[E_vertex(e,1)][2*i+1];
      
	    v4 = newNodes[E_vertex(e,0)][2*i];
	    v5 = newNodes[E_point(e,0)] [i];
	    v6 = newNodes[E_vertex(e,1)][2*i];

	    e0 = e2;
	  
	    e1 = M_createE(mesh,
			   v0,v1,
			   2,&v4,
			   sideEdgeEnt[0]);
	    e2 = M_createE(mesh,
			   v1,v2,
			   2,&v5,
			   pGEf);
	    e3 = M_createE(mesh,
			   v2,v3,
			   2,&v6,
			   sideEdgeEnt[1]);

	    pEdge es[4] = {e0,e1,e2,e3};
	    pFace newf = M_createF(mesh,4,es,pGEf);

	    E_forceWhatIn(e2,E_whatIn(e0));
	    EN_attachDataInt((pEntity) e2, tagId, -1);
	    E_forceWhatIn(e0,pGEf);
	    E_forceWhatIn(e1,sideEdgeEnt[0]);
	    E_forceWhatIn(e3,sideEdgeEnt[1]);
	  }
	}
      }
    }
    EIter_delete(eIt);
  } else if (dim == 3){
    pFace f;
    FIter fIt= M_faceIter(mesh);
    while ( ( f = FIter_next(fIt) ) ) {
      pGEntity pGE = (pGEntity) F_whatIn(f);
      bool addElem = (getID(tagId,(pEntity) f) != -1) && (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
      if (addElem) {
	const pRegion    r   = F_region(f,0); // neighbor cell
  	const pGEntity pGEr = (pGEntity) R_whatIn(r);
	const pGEntity pGEf = (pGEntity) F_whatIn(f);

	double normal[3],xyz[3],center[3];
	R_center(r,center);
	F_normal(f,normal);
	V_coord(F_vertex(f,0),xyz);
	
	// Find if the normal in outgoing
	double sgn = 0.;
	for (int i = 0; i < 3; i++)
	  sgn += (center[i] - xyz[i])*normal[i];
	
  	// Side faces: test if the edges belong to faces with 2 different 2d classifications
  	pGEntity sideFaceEnt[3] = {0,0,0};
  	for(int i = 0; i < F_numEdges(f); i++){
	  pEdge e = F_edge(f,i);
	  for(int j = 0; j < E_numFaces(e); j++){
	    pFace f2 = E_face(e,j);
	    pGEntity pGEf2 = (pGEntity) F_whatIn(f2);
	    bool inSide = (GEN_type(pGEf2) == dim-1) && (!isIn(tags,GEN_tag(pGEf2)));
	    if (inSide)
	      sideFaceEnt[i] = pGEf2;
	  }
  	}

	pGEntity sideEdgeEnt[3]  = {0,0,0};
	for(int i = 0; i < F_numVertices(f); i++){
	  pVertex v = F_vertex(f,i);
	  for(int j = 0; j < V_numEdges(v); j++){
	    bool corner = true;
	    pEdge e = V_edge(v,j);
	    pGEntity pGEe = (pGEntity) E_whatIn(e);
	    if (GEN_type(pGEe) != 1)
	      corner = false;
	    for(int k = 0; k < E_numFaces(e); k++){
	      pFace f2 = E_face(e,k);
	      pGEntity pGEf2 = (pGEntity) F_whatIn(f2);
	      bool inBc  = (GEN_type(pGEf2) == dim-1) && isIn(tags,GEN_tag(pGEf2));
	      if (inBc)
		corner = false;
	    }
	    if (corner) {
	      sideEdgeEnt[i] = pGEe;
	    }
	  }
	}

	pVertex v00 = F_vertex(f,0);
	pVertex v10 = F_vertex(f,1);
	pVertex v20 = F_vertex(f,2);

	pVertex p00 = (order == 2) ? E_point(F_edge(f,0),0) : NULL;
	pVertex p10 = (order == 2) ? E_point(F_edge(f,1),0) : NULL;
	pVertex p20 = (order == 2) ? E_point(F_edge(f,2),0) : NULL;

	pVertex v0,v1,v2,v3,v4,v5;
	pVertex p0,p1,p2,p3,p4,p5,p6,p7,p8;
	  
	vector<pRegion> rs(N);
	for(int i = 0; i < N; i++) {

	  if (i == 0) {
	    v0 = v00;
	    v1 = v10;
	    v2 = v20;
	  } else {
	    v0 = v3;
	    v1 = v4;
	    v2 = v5;
	  }
	    
	  v3 = newNodes[v00][2*i+1];
	  v4 = newNodes[v10][2*i+1];
	  v5 = newNodes[v20][2*i+1];

	  if (order == 2) {
	    if (i == 0) {
	      p0 = p00;
	      p1 = p10;
	      p2 = p20;
	    } else {
	      p0 = p3;
	      p1 = p4;
	      p2 = p5;
	    }
	  
	    p3 = newNodes[p00][i];
	    p4 = newNodes[p10][i];
	    p5 = newNodes[p20][i];

	    p6 = newNodes[v00][2*i];
	    p7 = newNodes[v10][2*i];
	    p8 = newNodes[v20][2*i];

	    M_createE(mesh,v0,v1,2,&p0);
	    M_createE(mesh,v1,v2,2,&p1);
	    M_createE(mesh,v2,v0,2,&p2);
	    M_createE(mesh,v3,v4,2,&p3);
	    M_createE(mesh,v4,v5,2,&p4);
	    M_createE(mesh,v5,v3,2,&p5);
	    M_createE(mesh,v0,v3,2,&p6);
	    M_createE(mesh,v1,v4,2,&p7);
	    M_createE(mesh,v2,v5,2,&p8);

	  }

	  if (sgn < 0){
	    pVertex vs[6] = {v0,v1,v2,v3,v4,v5};
	    rs[i] = M_createR(mesh,6,vs,pGEr);
	  } else {
	    pVertex vs[6] = {v3,v4,v5,v0,v1,v2};
	    rs[i] = M_createR(mesh,6,vs,pGEr);
	  }
	}
	// Set region classification
	for (int i = 0; i < N; i++) {
	  R_forceWhatIn(rs[i],pGEr);
	}

	// Set edge classification
	for(int i = 0; i < F_numEdges(f); i++){
	  pEdge e0 = F_edge(f,i);
	  pGEntity pGEe0 = (pGEntity) E_whatIn(e0);
	  if (GEN_type(pGEe0) == 1) { // Edge has a specific classification
	    const pVertex vv0 = E_vertex(e0,0);
	    const pVertex vv1 = E_vertex(e0,1);
	    const int j0 = EN_id((pEntity)  newNodes[vv0][2*N-1]);
	    const int j1 = EN_id((pEntity)  newNodes[vv1][2*N-1]);
	    pEdge eN = M_edge(mesh,j0,j1);
	    if (!eN)
	      throw runtime_error("edge not found");
	    E_forceWhatIn(eN,pGEe0);
	  }
	  E_forceWhatIn(e0,pGEr);
	}

	// Set side edge classification with a face
	for (int i = 0; i < N; i++) {
	  for(int j = 6; j < 9; j++) {// Side edges
	    pEdge edge = R_edge(rs[i],j);
	    for(int k = 0; k < E_numFaces(edge); k++){
	      if (GEN_type(E_whatIn(edge)) >= GEN_type(F_whatIn(E_face(edge,k))))
		E_forceWhatIn(edge,F_whatIn(E_face(edge,k)));
	    }
	  }
	}

	//  Corner edge classification
	for (int i = 0; i < N; i++) {
	  for(int j = 0; j < 3; j++) {// Side edges
	    if (sideEdgeEnt[j]){
	      pEdge edge = R_edge(rs[i],j+6); // Side edges
	      E_forceWhatIn(edge,sideEdgeEnt[j]);
	    }
	  }
	}

	// Set face classification
	pFace fN = R_face(rs[N-1],(sgn < 0.) ? 1:0);
	F_forceWhatIn(fN,pGEf);
	F_forceWhatIn(f ,pGEr);
	EN_attachDataInt(fN, tagId, -1);

	// Set side face classification
	for (int i = 0; i < N; i++) {
	  for(int j = 0; j < 3; j++)
	    if (sideFaceEnt[j]) {
	      F_forceWhatIn(R_face(rs[i],2+j),sideFaceEnt[j]);
	    }
	}
      }
    }
    FIter_delete(fIt);
  }

  // periodicity
  const pMeshDataId tagPeriodic = MD_lookupMeshDataId("PeriodicVertexID");

  for(map<pVertex,vector<pVertex> >::iterator it = newNodes.begin(); it != newNodes.end(); it++){
    void* tmpptr = NULL;
    int isPeriodic = EN_getDataPtr(it->first,tagPeriodic,&tmpptr);
    if (isPeriodic && (getID(tagId,it->first) != -1)){
      map<int,vector<int> > *conn = (map<int,vector<int> > *) tmpptr;
      for(size_t j = 0; j < it->second.size(); j++){
	map<int,vector<int> > *conn1 = new map<int,vector<int> >;
	for (map<int,vector<int> >::iterator citer = conn->begin(); citer != conn->end(); ++citer) {
	  pVertex v2 = M_vertex(mesh,citer->first);
	  conn1->insert(std::make_pair(V_id(newNodes[v2][j]),vector<int>()));
	}
	EN_attachDataPtr((pEntity) it->second[j],tagPeriodic,conn1);
      }
    }
  }

}
