#include "normals.h"

using namespace std;
using namespace MAd;

void computeNodeNormals(pMesh mesh, vector<int> &tags){

  const int dim    = M_dim(mesh);

  pMeshDataId nx = MD_lookupMeshDataId("nx");
  pMeshDataId ny = MD_lookupMeshDataId("ny");
  pMeshDataId nz = MD_lookupMeshDataId("nz");

  double loc_normal[3], avg_normal[3], uu,vv,ww,avg_size;

  pVertex v;
  VIter vIt= M_vertexIter(mesh);
  while ( ( v = VIter_next(vIt) ) ){
    const int nEdges = V_numEdges(v);
    int  n = 0;
    for(int i = 0; i < 3; i++) avg_normal[i] = 0.;
    avg_size = 0.;
    if (dim == 2) {
      pPList vEdges = V_edges(v);
      void * temp = NULL;
      while ( pEdge e = (pEdge) PList_next(vEdges,&temp) ) {
	pGEntity pGE = (pGEntity) E_whatIn(e);
	bool isBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
	if (isBc) {
	  n++;
	  pFace f = E_face(e,0);
	  pMapping map = Map_create((pEntity) f, 1.0);
	  if (v == F_vertex(f,0)) {
	    uu = 0.;
	    vv = 0.;
	    ww = 0.;
	  } else if (v == F_vertex(f,1)) {
	    uu = 1.;
	    vv = 0.;
	    ww = 0.;
	  } else if (v == F_vertex(f,2)) {
	    uu = 0.;
	    vv = 1.;
	    ww = 0.;
	  }
	  map->normalVector(e,uu,vv,ww,loc_normal);
	  for(int i = 0; i < 3; i++) avg_normal[i] += loc_normal[i];
	  avg_size += E_length(e);
	}
      }
      PList_delete(vEdges);
    } else if (dim == 3) {
      pPList vFaces = V_faces(v);
      void * temp = NULL;
      while ( pFace f = (pFace) PList_next(vFaces,&temp) ) {
	pGEntity pGE = (pGEntity) F_whatIn(f);
	bool isBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
	if (isBc) {
	  n++;
	  pRegion r = F_region(f,0);
	  pMapping map = Map_create((pEntity) r, 1.0);
	  if (v == R_vertex(r,0)) {
	    uu = 0.;
	    vv = 0.;
	    ww = 0.;
	  } else if (v == R_vertex(r,1)) {
	    uu = 1.;
	    vv = 0.;
	    ww = 0.;
	  } else if (v == R_vertex(r,2)) {
	    uu = 0.;
	    vv = 1.;
	    ww = 0.;
	  } else if (v == R_vertex(r,3)) {
	    uu = 0.;
	    vv = 0.;
	    ww = 1.;
	  }
	  map->normalVector(f,uu,vv,ww,loc_normal);
	  for(int i = 0; i < 3; i++) avg_normal[i] += loc_normal[i];
	  avg_size += 1./3.*(E_length(F_edge(f,0)) + 
			     E_length(F_edge(f,1)) + 
			     E_length(F_edge(f,2)) );
	}
      }
      PList_delete(vFaces);
    }

    if (n !=0){
      avg_size /= n;
      // cout << alength << endl;
      double nrm = 0.;
      for(int i = 0; i < 3; i++) nrm += avg_normal[i]*avg_normal[i];
      nrm = sqrt(nrm);
      for(int i = 0; i < 3; i++) avg_normal[i] *= avg_size/nrm;
      EN_attachDataDbl((pEntity) v, nx, avg_normal[0]);
      EN_attachDataDbl((pEntity) v, ny, avg_normal[1]);
      EN_attachDataDbl((pEntity) v, nz, avg_normal[2]);
    }
  }
  VIter_delete(vIt);

  // edges
  pEdge e;
  EIter eIt= M_edgeIter(mesh);
  while ( ( e = EIter_next(eIt) ) ){
    for(int i = 0; i < E_numPoints(e); i++) {
      if (i > 0) throw runtime_error("i > 0");
      const pPoint p = E_point(e,i);
      int n = 0;
      avg_size = 0.;
      if (dim == 2) {
	pGEntity pGE = (pGEntity) E_whatIn(e);
	bool isBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
	if (isBc) {
	  pFace f = E_face(e,0);
	  pMapping map = Map_create((pEntity) f, 1.0);
	  if (e == F_edge(f,0)) {
	    uu = .5;
	    vv = 0.;
	    ww = 0.;
	  } else if (e == F_edge(f,1)) {
	    uu = .5;
	    vv = .5;
	    ww = 0.;
	  }  else if (e == F_edge(f,2)) {
	    uu = 0.;
	    vv = .5;
	    ww = 0.;
	  }
	  map->normalVector(e,uu,vv,ww,loc_normal);
	  double nrm = 0.;
	  for(int i = 0; i < 3; i++) nrm += loc_normal[i]*loc_normal[i];
	  nrm = sqrt(nrm);
	  for(int i = 0; i < 3; i++) loc_normal[i] *= E_length(e)/nrm;
	  EN_attachDataDbl((pEntity) p, nx, loc_normal[0]);
	  EN_attachDataDbl((pEntity) p, ny, loc_normal[1]);
	  EN_attachDataDbl((pEntity) p, nz, loc_normal[2]);
	}
      } else if (dim == 3) {
	pPList eFaces = E_faces(e);
	void * temp = NULL;
	while ( pFace f = (pFace) PList_next(eFaces,&temp) ) {
	  pGEntity pGE = (pGEntity) F_whatIn(f);
	  bool isBc = (GEN_type(pGE) == dim-1) && isIn(tags,GEN_tag(pGE));
	  if (isBc) {
	    n++;
	    pRegion r = F_region(f,0);
	    pMapping map = Map_create((pEntity) r, 1.0);
	    if (e == R_edge(r,0)) {
	      uu = 0.5;
	      vv = 0.;
	      ww = 0.;
	    } else if (e == R_edge(r,1)) {
	      uu = .5;
	      vv = .5;
	      ww = 0.;
	    } else if (e == R_edge(r,2)) {
	      uu = 0.;
	      vv = .5;
	      ww = 0.;
	    } else if (e == R_edge(r,3)) {
	      uu = 0.;
	      vv = 0.;
	      ww = .5;
	    } else if (e == R_edge(r,4)) {
	      uu = .5;
	      vv = 0.;
	      ww = .5;
	    } else if (e == R_edge(r,5)) {
	      uu = 0.;
	      vv = .5;
	      ww = .5;
	    }
	    map->normalVector(f,uu,vv,ww,loc_normal);
	    for(int i = 0; i < 3; i++) avg_normal[i] += loc_normal[i];
	    avg_size += 1./3.*(E_length(F_edge(f,0)) + 
			       E_length(F_edge(f,1)) + 
			       E_length(F_edge(f,2)) );
	  }
	}
	PList_delete(eFaces);
	if (n !=0){
	  avg_size /= n;
	  // cout << alength << endl;
	  double nrm = 0.;
	  for(int i = 0; i < 3; i++) nrm += avg_normal[i]*avg_normal[i];
	  nrm = sqrt(nrm);
	  for(int i = 0; i < 3; i++) avg_normal[i] *= avg_size/nrm;
	  EN_attachDataDbl((pEntity) p, nx, avg_normal[0]);
	  EN_attachDataDbl((pEntity) p, ny, avg_normal[1]);
	  EN_attachDataDbl((pEntity) p, nz, avg_normal[2]);
	}
      }
    }
  }
  EIter_delete(eIt);

}
