// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include "ModelInterface.h"
#include "MeshDataBaseInterface.h"
#include "MeshDataBaseUntangling.h"
#include "MeshDataBaseExtrusion.h"
#include "MappingInterface.h"
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif

using namespace MAd;
using namespace std;

int main(int argc, char* argv[])
{

#ifdef _HAVE_MKL_
  atexit(mkl_free_buffers);
#endif   
  
  // --- setup mesh and faces 

  set<int> faceTags;
  faceTags.insert(2);
  faceTags.insert(3);
  faceTags.insert(4);
  faceTags.insert(5);

  std::string mshName( argv[1] );
  mshName += "/tangled_cylinder_3D.msh";

  // --- read and untangle mesh

  pGModel null_model = 0; 
  GM_create(&null_model);
  pMesh mesh = M_new(null_model);

  int nbLayers = 10;
  M_load(mesh,mshName.c_str());
  M_untangleBoundaryLayers(mesh,faceTags,nbLayers);

  // --- evaluate mapping jacobian 
  // and check for inverted elements

  int nbNegativePoints = 0;

  RIter rIter = M_regionIter(mesh);
  while (pEntity pr = (pEntity) RIter_next(rIter)) {

    pMapping mapping = Map_create((pEntity) pr,1.0);   

    int nbKnots = Map_getNumberOfKnots(mapping);
    
    for(int i=0; i<nbKnots; i++){

      double uvw[3]={0,0,0};
      Map_getVertexUVW(mapping,i,uvw);  
     
      if(Map_determinant(mapping,uvw)<0) nbNegativePoints++;      
    }
    Map_delete(mapping);        
  }
  RIter_delete(rIter);

  // --- delete the mesh

  M_delete(mesh);
  GM_delete(null_model); 

  if (nbNegativePoints)
      std::cout << "Warning " << __FILE__ <<":"<< __LINE__ << ": Found " << nbNegativePoints << " negative points" << std::endl;
  return nbNegativePoints;
}
