// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------
// Testcase to check mapping for 1st order triangles element
// Load a mesh (square discretised through first order triangle)
// Build an entity through the given node label
// Compute the jacobian determinant of the mapping at each node
// Compare the computed values to the reference values in the txt file.
// for more details on the mapping features 
// please refer to the ELCI report L.2.4.2 (farid.benyoucef@cenaero.fr)

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include "ModelInterface.h"
#include "IMeshAdapter.h"
#include "ISizeField.h"
#include "MappingInterface.h"
#include "CheckMesh.h"
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif
//const double lengthscale=1.0;

using namespace MAd;
using namespace std;

int main(int argc, char* argv[])
{
  
#ifdef _HAVE_MKL_
  atexit(mkl_free_buffers);
#endif 
  
    pGModel null_model = 0;
    GM_create(&null_model);
	pMesh p_mesh = M_new(null_model);

	std::string mshName( argv[1] );
//   mshName += "/cube_unit.msh";
	mshName += "/sphere.msh";
	
	M_load(p_mesh, mshName.c_str());
	M_writeMsh(p_mesh, "./initialMesh_tetra.msh", 2);
	int initOrder = M_maxOrder(p_mesh);
	
/* Start of code lines to check mapping functionalities   */

	IPiecewiseLinearSizeField * sizeField=CreatePiecewiseLinearSizeField(p_mesh);
	sizeField->setCurrentSize();
    IMeshAdapter* adapter = IMeshAdapter::BuildMeshAdapter(p_mesh, sizeField);
//   adapter->setEdgeLenSqBounds(2.,3.5);
	adapter->setEdgeLenSqBounds(1.0/9.0,1.0);
	adapter->run();

	std::string pref = "./";
	adapter->setOutputDirectoryPath(pref.c_str());
	std::string fn="adapted_mesh_tetra.msh";
  adapter->writeMsh(fn.c_str());
  
  /* The resulting mesh is conform ? */
  if ( ! checkMesh(p_mesh) ) {
    delete adapter;
    delete sizeField;
    M_delete( p_mesh );
    GM_delete( null_model ); 
    return 1;
  }
  
	
// memory cleaning
  delete adapter;
  delete sizeField;
	M_delete( p_mesh );
	
/* Start of code lines to check final order  */	
	p_mesh = M_new(null_model);
	M_load(p_mesh, (pref+fn).c_str());
	int finOrder = M_maxOrder(p_mesh);
	
	cout << "Initial Mesh Order : " << initOrder << endl;
	cout << "Final   Mesh Order : " << finOrder << endl;
	
	M_delete( p_mesh );
	GM_delete( null_model ); 
	
	if (initOrder != finOrder)
		return 1;
	else
		return 0;
	
}
