// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------
// Testcase to check mapping for 1st order hex element
// Load a mesh (cube discretised through first order tets)
// Build an entity through the given node label
// Compute the jacobian determinant of the mapping at each node
// Compare the computed values to the reference values in the txt file.
// for more details on the mapping features 
// please refer to the ELCI report L.2.4.2 (farid.benyoucef@cenaero.fr)

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <cmath>
#include "MappingInterface.h"
#include "ModelInterface.h"
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif
const double lengthscale=1.0;

using namespace MAd;

int main(int argc, char* argv[])
{
  
#ifdef _HAVE_MKL_
  atexit(mkl_free_buffers);
#endif   

  pGModel null_model = 0; 
  GM_create(&null_model);
  pMesh p_mesh = M_new(null_model);

  std::string mshName( argv[1] );
  mshName += "/cube_4.msh";
  
  M_load(p_mesh, mshName.c_str());

/* Start of code lines to check mapping functionalities   */
  pVertex verts[8];
  verts[0]=M_vertex(p_mesh,42);
  verts[1]=M_vertex(p_mesh,92);
  verts[2]=M_vertex(p_mesh,101);
  verts[3]=M_vertex(p_mesh,87);
  verts[4]=M_vertex(p_mesh,8);
  verts[5]=M_vertex(p_mesh,41);
  verts[6]=M_vertex(p_mesh,80);
  verts[7]=M_vertex(p_mesh,32);

//
  pRegion hexa=M_createR(p_mesh, 8, verts);
  //
  pMapping p_maps = Map_create((pEntity) hexa, lengthscale); 

  double uvw[3];

  double detJac[8];
  int idx = 0;
  for (int i=0;i<2;i++) {
    for (int j=0;j<2;j++) {
      for (int k=0;k<2;k++) {
        uvw[0] = -0.5 + i;
        uvw[1] = -0.5 + j;
        uvw[2] = -0.5 + k;
        
        detJac[idx++] = Map_determinant(p_maps,uvw);
        
      }
    }
  }
  
  Map_delete(p_maps); 
  M_removeRegion(p_mesh, hexa);
  M_delete( p_mesh );
  GM_delete( null_model );

  // compare computed data to reference value computed analytically 
  // written in the qa_values_check.txt file
  // non-regression tescase database   
  
  std::string refName( argv[1] );
  refName += "/qa_values_check.txt";
  std::ifstream ref_file( refName.c_str() );
  
  int nbr;
  ref_file >> nbr;
  bool works_fine=true;  
  double eps=1.0e-6;
  for(int i=0; i<std::min(nbr,8); i++){
    double ref;
    ref_file >> ref;
    if (std::fabs(ref-detJac[i]) > eps) {
      std::cout << "Error in linear hexahedron mapping on point " << i 
                << " : " << detJac[i] << " vs " << ref << std::endl;
      works_fine = false;
    }
  }
  
  return works_fine ? 0 : 1 ;
}
