// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include "MappingInterface.h"
#include "ModelInterface.h"
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif

using  std::cout;

const double lengthscale=1.0;

using namespace MAd;

int main(int argc, char* argv[])
{
  
#ifdef _HAVE_MKL_
  atexit(mkl_free_buffers);
#endif   

  pGModel null_model = 0; 
  GM_create(&null_model);
  pMesh p_mesh = M_new(null_model);

  std::string mshName( argv[1] );
  mshName += "/cube_tet_pyra.msh";

  M_load(p_mesh, mshName.c_str());
  int nbrVertices = M_numVertices(p_mesh);

  int numC=M_numRegions(p_mesh);
  std::cout << "number of cells is:" << numC << std::endl;
  int numT=M_numTets(p_mesh);
  std::cout << "number of tets is:" << numT << std::endl;
  int numP = numC-numT;
  std::cout << "number of Pyramids is:" << numP << std::endl;
  
  if(numP==0) { 
    /* Start of code lines to check mapping functionalities   */
    pVertex verts[5];
    verts[0]=M_vertex(p_mesh,1);
    verts[1]=M_vertex(p_mesh,40);
    verts[2]=M_vertex(p_mesh,73);
    verts[3]=M_vertex(p_mesh,9);
    verts[4]=M_vertex(p_mesh,307);
    //
    pRegion pyr=M_createR(p_mesh, 5, verts);
    //  
    pMapping p_maps = Map_create((pEntity) pyr, lengthscale);
    double* jac=NULL;
    double* EdgeQPts=NULL;
    // Map_jacobian(p_maps, jac, EdgeQPts,5); 
    Map_delete(p_maps); 
    M_removeRegion(p_mesh, pyr);
    delete jac;
    delete EdgeQPts;
    /* End of code lines to check mapping functionalities */
    
  }
  else{
   std::cout << "Pyramid elements are not supported by MAdLib interface yet"<<std::endl;
  }
  M_delete( p_mesh );
  GM_delete( null_model );

  std::string refName( argv[1] );
  refName += "/qa_values_check.txt";
  std::ifstream ref_file( refName.c_str() );
  
  int nbr_values, ref_nbrVertices;
  ref_file >> nbr_values >> ref_nbrVertices;

  if(nbrVertices == ref_nbrVertices) 
    return 0;
  else
    return 1;  
}
