// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------
// Testcase to check mapping for 2nd order quad element
// Load a mesh (square discretised through second order quad)
// Build an entity through the given node label
// Compute the jacobian determinant of the mapping at each node
// Compare the computed values to the reference values in the txt file.
// for more details on the mapping features 
// please refer to the ELCI report L.2.4.2 (farid.benyoucef@cenaero.fr)

#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <cmath>
#include "MappingInterface.h"
#include "ModelInterface.h"
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif

const double lengthscale=1.0;

using namespace MAd;

int main(int argc, char* argv[])
{
#ifdef _HAVE_MKL_
  atexit(mkl_free_buffers);
#endif    
  pGModel null_model = 0; 
  GM_create(&null_model);
  pMesh p_mesh = M_new(null_model);

  std::string mshName( argv[1] );
  mshName += "/square2O.msh";
  
  M_load(p_mesh, mshName.c_str());


/* Start of code lines to check mapping functionalities   */
//
  int verts_id[9];
  verts_id[0]=62;
  verts_id[1]=128;
  verts_id[2]=5;
  verts_id[3]=4;
  verts_id[4]=183;
  verts_id[5]=184;
  verts_id[6]=14;
  verts_id[7]=71;
  verts_id[8]=185;
  //
  pFace quad2O=M_createQuad(p_mesh, 2, verts_id);
  //
  pMapping p_maps = Map_create((pEntity) quad2O, lengthscale); 
  
  // define the coordinates of the parametric element: all x then all y then all z

  double detJac[9];
  double uvw[3];
  int idx = 0;
  for (int i=0;i<3;i++) {
    for (int j=0;j<3;j++) {
      uvw[0] = (i-1);
      uvw[1] = (j-1);
      uvw[2] = 0;
      detJac[idx++] = Map_determinant(p_maps,uvw);    
    }
  }
  
  Map_delete(p_maps);
  M_removeFace(p_mesh,quad2O);

  M_delete( p_mesh );
  GM_delete( null_model );

  // compare computed data to reference value computed analytically 
  // written in the qa_values_check.txt file
  // non-regression tescase database  
  
  std::string refName( argv[1] );
  refName += "/qa_values_check.txt";
  std::ifstream ref_file( refName.c_str() );
  
  int nbr_ref_values;
  ref_file>>nbr_ref_values;
  
  bool works_fine=true;
  double eps=1.0e-06;
  for(int i=0; i<std::min(9,nbr_ref_values); i++){
    double ref;
    ref_file >> ref;
    if (std::fabs(ref-detJac[i]) > eps) {
      std::cout << "Error in quadratic quadrilateral mapping on point " << i 
                << " : " << detJac[i] << " vs " << ref << std::endl;
      works_fine = false;
    }
  }
  
  return works_fine ? 0:1;
}
