// -*- C++ -*-
// -------------------------------------------------------------------
// MAdLib - Copyright (C) 2019-2019 Cenaero
//
// See the Copyright.txt and License.txt files for license information. 
// -------------------------------------------------------------------


#include <string>
#include <fstream>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <cmath>
#include <cstring>
#ifdef _HAVE_MKL_
#include "mkl_service.h"
#endif

#include "TransformationInterface.h"

using namespace MAd;

int main(int argc, char* argv[])
{

#ifdef _HAVE_MKL_                                                                         
  atexit(mkl_free_buffers);                                                               
#endif
  
  double center[3] = {1,0,0};
  double angle[3]  = {22,6,130};
  double translation[3] = {1,4,9};

  pTransform pt1 = T_create(center,angle,translation);
  if (!pt1) {
    std::cout << "Could not create the initial transformation" << std::endl;
    return 1;
  }
  
  double src0[3] = {0,0,0};
  double src1[3] = {1,0,0};
  double src2[3] = {0,1,0};

  double tgt0[3];
  double tgt1[3];
  double tgt2[3];

  std::memcpy(tgt0,src0,3*sizeof(double));
  std::memcpy(tgt1,src1,3*sizeof(double));
  std::memcpy(tgt2,src2,3*sizeof(double));

  T_transformCoordinates(pt1,tgt0);
  T_transformCoordinates(pt1,tgt1);
  T_transformCoordinates(pt1,tgt2);
  
  pTransform pt2 = T_create(src0,src1,src2,tgt0,tgt1,tgt2);

  if (!pt2) {
    std::cout << "Could not reconstruct the transformation from 3 points" 
              << std::endl;
    T_delete(pt1);
    return 1;
  }

  T_delete(pt1);
  T_delete(pt2);

  return 0;  
}
