#!/usr/bin/python

# If no argument given, use the the 'PLATFORM' environment variable
# If argument given, first one is used to determine the platform
# If additional arguments, perform installation for these configurations only

# Examples:
# python install_new_revision.py
# python install_new_revision.py zenobe64
# python install_new_revision.py zenobe64 zenobe_64_open64_4.2.4.cmake
# python install_new_revision.py zenobe64 zenobe_64_open64_4.2.4.cmake zenobe_64_openmpi_1.4.4-1_intel_10.1.026.cmake

import os
import sys
import re
import fileinput
import platform
import subprocess

# Get configurations list according to PLATFORM environment variable
# ------------------------------------------------------------------

# Extract platf from PLATFORM or argument
if( len(sys.argv) == 1 ):
  platf = os.environ.get( "PLATFORM" )
else:
  platf = sys.argv[1]

if( ( platf[-2:] == '32' ) or ( platf[-2:] == '64' ) ): # Check last 2 char
  platf = platf[:-2] # Remove last 2 char

platf = platf.lower() # Use lower cases only
if( platf == 'cenaero' ):
  platf = 'macaw'
if( platf == 'BPC' ):
  platf = 'emu'

# Get directory where this script is located
script_dir = os.path.dirname(os.path.abspath(sys.argv[0]))

# Get configuration files list
full_list = os.listdir(script_dir)

config_files_list = []
extended_config_files_list = []  # list for replace revision number

if( len(sys.argv) > 1 ):
  for i in range(2, len(sys.argv)):
    config_files_list.append(os.path.normpath(os.path.join(script_dir, sys.argv[i])))

for file in full_list:
  file = os.path.normpath(os.path.join(script_dir, file))
  # Start with platf, followed by '_32/64/64-*_',  and ext is cmake
  file_name = os.path.basename(file)  # Remove folder
  substr_file_name = str.split(file_name, '_') # 'zenobe', '64-gm', 'gcc412.cmake'
  if( substr_file_name[0] == platf ):
    if( len(substr_file_name) > 1 ):
      if( os.path.splitext(file_name)[1] == '.cmake' ):
        if( ( '32' in substr_file_name[1] ) or ( '64' in substr_file_name[1] ) ):
          if( len(sys.argv) <= 2 ):
            config_files_list.append(file)
        if( 'nightly' in substr_file_name[1] ):
          extended_config_files_list.append(file)

extended_config_files_list.append(config_files_list)

# Get revision number from installation folder name
# -------------------------------------------------
path_list = str.split(os.path.abspath(sys.argv[0]), '/')
tmp = path_list[len(path_list)-4]  # e.g.: MAdLib_192 on Linux; r192 on Win
rev = 'head'
if(re.search('r(\d+)$', tmp)):
  rev = (re.search(r'(\d+)$', tmp)).group()  # Extract numbers at the end of tmp
print "\nInstall MAdLib revision '" + rev + "'"

# Change 'head' into revision number
# ----------------------------------
for file in extended_config_files_list:
  for line in fileinput.FileInput(file,inplace=1):
    line = line.replace('head', rev )
    line = line.replace('\n', '' )
    print (line)

# Launch scripts
# --------------
for config_file in config_files_list:
  log_file = os.path.splitext(os.path.basename(config_file))[0] + ".log"
  if( platf == 'zenobe' or platf == 'theophile' ): 
    tmp = os.path.splitext(config_file)[0]  
    name_split = str.split(tmp, '_')
    if( len( name_split ) <= 5 ):      # zenobe_64_gcc_4.1.2 - 5 args if sp1 (zenobe_64_intel_2013_sp1.1.106)
      compiler = name_split[2]         # gcc
      compiler_version = name_split[3] # 4.1.2
      if( len( name_split ) == 5 ):
        compiler_version = name_split[3] + "_" + name_split[4]
      if( compiler_version == "2011.7.256" ):  # Correction to add composerxe
        compiler_version = "composerxe/2011.7.256"
      if( platf == 'zenobe' ):
        command = ". /usr/share/Modules/init/sh; module load cmake/2.8.9; module load compiler/" + compiler + "/" + compiler_version + "; CC=$CC CXX=$CXX ctest -S " + config_file + " -V >& " + log_file
      else:
        command = ". /usr/share/Modules/init/sh; module load cmake/2.8.9/64/gcc/4.1.2; module load compiler/" + compiler + "/" + compiler_version + "; CC=$CC CXX=$CXX ctest -S " + config_file + " -V >& " + log_file
    else:                              # zenobe_64_openmpi_1.4.4.1-1_gcc_4.1.2
      mpi = name_split[2]              # openmpi
      mpi_version = name_split[3]      # 1.4.4-1
      compiler = name_split[4]         # gcc
      compiler_version = name_split[5] # 4.1.2
      if( len( name_split ) == 7 ):    
        compiler_version = name_split[5] + "_" + name_split[6]
      if( platf == 'zenobe' ):
        command = ". /usr/share/Modules/init/sh; module load cmake/2.8.9; module load compiler/" + compiler + "/" + compiler_version + "; module load " + mpi + "/" + mpi_version + "/64; CC=$CC CXX=$CXX ctest -S " + config_file + " -V >& " + log_file
      else:
        command = ". /usr/share/Modules/init/sh; module load cmake/2.8.9/64/gcc/4.1.2; module load compiler/" + compiler + "/" + compiler_version + "; module load " + mpi + "/" + mpi_version + "/64; CC=$CC CXX=$CXX ctest -S " + config_file + " -V >& " + log_file

  elif os.path.isfile("/home/cenaero/pierard/bin/ctest"):
    command = "/home/cenaero/pierard/bin/ctest -S " + config_file + " -V >& " + log_file

  elif os.path.isfile("/softs/cmake/2.8.9/64/gcc/4.2.1/bin/ctest"):  # sealion
    command = "/softs/cmake/2.8.9/64/gcc/4.2.1/bin/ctest -S " + config_file + " -V >& " + log_file

  elif os.path.isfile("/home/pierard/bin/ctest"):
    command = "/home/pierard/bin/ctest -S " + config_file + " -V >& " + log_file

  else:   
    command = "ctest -S " + config_file + " -V >& " + log_file


  print "\n*** Start '" + command + "'"
  p = os.system( command )
  print "*** End "

# chgrp, chmod
# ------------
if(platform.system() == 'Linux'):
  dir = "/softs/madlib/" + rev
  if os.path.isdir(dir):
    command = "chgrp -R cenaero /softs/madlib/" + rev
    os.system( command )
    command = "chmod -R 755 /softs/madlib/" + rev
    os.system( command )
  else:
    print "No chgrp/chmod performed as " + dir + " does not exist"


