# -- Find AUTOPACK --
# Cenaero - 10/06/2010
#
# Inspired from:
# - http://www.itk.org/Wiki/CMake:How_To_Find_Libraries
# - http://github.com/temporaer/seralign/blob/master/CMakeModules/FindAUTOPACK.cmake
#
# Find the native AUTOPACK headers and libraries.
#
# Input parameters which could influence the search:
# - AUTOPACK_ROOT - root folder where AUTOPACK is installed
#
# Once done, this will define:
# - AUTOPACK_INCLUDE_DIRS - where to find AUTOPACK headers
# - AUTOPACK_LIBRARIES - List of libraries when using AUTOPACK.
# - AUTOPACK_FOUND - True if AUTOPACK found.
#

if(AUTOPACK_INCLUDE_DIR AND AUTOPACK_LIBRARY)
  set(AUTOPACK_FIND_QUIETLY TRUE)
endif(AUTOPACK_INCLUDE_DIR AND AUTOPACK_LIBRARY)

include(LibFindMacros)

# Check input parameters
if(AUTOPACK_ROOT)
  set( _AUTOPACK_INCLUDE_SEARCH_DIRS
       ${AUTOPACK_ROOT}/include ${AUTOPACK_ROOT} ${_AUTOPACK_INCLUDE_SEARCH_DIRS})
  set( _AUTOPACK_LIBRARY_SEARCH_DIRS
       ${AUTOPACK_ROOT}/lib ${AUTOPACK_ROOT} ${LIB_BASE}/lib 
       ${_AUTOPACK_LIBRARY_SEARCH_DIRS})
endif(AUTOPACK_ROOT)

# Dependencies
# ! none !

# Use pkg-config to get hints about paths
# libfind_pkg_check_modules(AUTOPACK_PKGCONF AUTOPACK)

# Look for the header file
find_path(AUTOPACK_INCLUDE_DIR NAMES autopack.h
          PATHS ${_AUTOPACK_INCLUDE_SEARCH_DIRS} ${AUTOPACK_PKGCONF_INCLUDE_DIRS})

# Look for the library
find_library(AUTOPACK_LIBRARY NAMES autopack autopack-O
             PATHS ${_AUTOPACK_LIBRARY_SEARCH_DIRS} ${AUTOPACK_PKGCONF_LIBRARY_DIRS})

# Set the include dir variables and the libraries and let libfind_process do the rest
set(AUTOPACK_PROCESS_INCLUDES AUTOPACK_INCLUDE_DIR)
set(AUTOPACK_PROCESS_LIBS AUTOPACK_LIBRARY)
libfind_process(AUTOPACK)
