# -- Find BLASLAPACK --
# Extracted from Gmsh main CMakeLists - 10/06/2010
# Different than default FindLAPACK.cmake to avoid requirement of Fortran
#
# Find the Lapack libraries.
#
# Input parameters which could influence the search:
# - MKL_ROOT: Root path for MKL library (/lib/em64t automatically added)
#
# Once done, this will define:
# - BLASLAPACK_LIBS - List of libraries when using ITL - empty
# - BLASLAPACK_FOUND - True if ITL found.
# - BLASLAPACK_FLAGS - Additional link flags
# - BLASLAPACK_INCLUDE_DIRS - Directory where the header files are located - only defined when needed
#

########################################################### 
# Gmsh macro for FindBlas
########################################################### 
macro(find_all_libraries VARNAME LISTNAME SUFFIX)
  set(${VARNAME})
  list(LENGTH ${LISTNAME} NUM_LIST)
  foreach(LIB ${${LISTNAME}})
    foreach(SUFF ${${SUFFIX}})
      find_library(FOUND_LIB ${LIB} PATHS ${SUFF})
    endforeach()
    if(FOUND_LIB)
      list(APPEND ${VARNAME} ${FOUND_LIB})
    endif()
    set(FOUND_LIB FOUND_LIB-NOTFOUND CACHE INTERNAL "")
    # unset(FOUND_LIB CACHE) # cleaner, but only available in cmake >= 2.6.4
  endforeach(LIB)
  list(LENGTH ${VARNAME} NUM_FOUND_LIBRARIES)
  if(NUM_FOUND_LIBRARIES LESS NUM_LIST)
    set(${VARNAME})
  endif(NUM_FOUND_LIBRARIES LESS NUM_LIST)
endmacro(find_all_libraries)

# For LibFindProcess.cmake
include(LibFindMacros)

set(MADLIB_USE_MKL FALSE CACHE INTERNAL "")

if(MSVC)
  # on Windows with Visual C++ try really hard to find blas/lapack
  # *without* requiring a Fortran compiler: 1) try to find the Intel
  # MKL libs using the standard search path; if not found 2) try to
  # get the reference blas/lapack libs (useful for users with no
  # Fortran compiler and no MKL license, who can just download our
  # precompiled "gmsh-dep" package)
  if(REGISTERS_SIZE EQUAL 64)
    set(MKL_PATH ${MKL_ROOT}/em64t/lib ${MKL_ROOT}/lib/intel64 ${MKL_ROOT}/../lib/intel64)
    set(MKL_LIBS_REQUIRED mkl_sequential mkl_core mkl_intel_lp64)
  else()
    set(MKL_PATH ${MKL_ROOT}/ia32/lib)
    set(MKL_LIBS_REQUIRED mkl_sequential mkl_core mkl_intel_c)
  endif()
  find_all_libraries(BLASLAPACK_LIBS MKL_LIBS_REQUIRED MKL_PATH )
  if(BLASLAPACK_LIBS)
    set(MADLIB_USE_MKL TRUE CACHE INTERNAL "")
    set(HAVE_BLAS TRUE)
    set(HAVE_LAPACK TRUE)
    set(BLASLAPACK_INCLUDE_DIRS "${MKL_ROOT}/include")
  else()
    set(REFLAPACK_LIBS_REQUIRED lapack blas g2c gcc)
    set(MKL_PATH "")
    find_all_libraries(BLASLAPACK_LIBS REFLAPACK_LIBS_REQUIRED MKL_PATH)
    if(BLASLAPACK_LIBS)
      set(HAVE_BLAS TRUE)
      set(HAVE_LAPACK TRUE)
    endif()
  endif()
elseif(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
  # on Linux also try to find the Intel MKL without a Fortran
  # compiler
  if(REGISTERS_SIZE EQUAL 64)
    set(MKL_PATH ${MKL_ROOT}/lib/em64t ${MKL_ROOT}/lib/intel64 ${MKL_ROOT}/../lib/intel64) 
  else()
    set(MKL_PATH ${MKL_ROOT}/lib/32)
  endif()
  set(MKL_LIBS_REQUIRED mkl_sequential mkl_core mkl_intel_lp64)
  find_all_libraries(BLASLAPACK_LIBS MKL_LIBS_REQUIRED MKL_PATH)
  if(BLASLAPACK_LIBS)
    set(MADLIB_USE_MKL TRUE CACHE INTERNAL "")
    find_package(Threads)
    if(Threads_FOUND)
      list(APPEND BLASLAPACK_LIBS ${CMAKE_THREAD_LIBS_INIT})
    endif()
    set(HAVE_BLAS TRUE)
    set(HAVE_LAPACK TRUE)
  endif()
elseif(${CMAKE_SYSTEM_NAME} MATCHES "SunOS")
  # on SunOS we know blas and lapack are available in sunperf
  set(HAVE_BLAS TRUE)
  set(HAVE_LAPACK TRUE)
  set(BLASLAPACK_FLAGS -library=sunperf)
elseif(${CMAKE_SYSTEM_NAME} MATCHES "BlueGeneQ-static" OR ${CMAKE_SYSTEM_NAME} MATCHES "BlueGeneQ-dynamic" )
  message("ESSL_PATH: ${ESSL_PATH}")
  set(ESSL_LIBS_REQUIRED esslbg xlopt xlf90_r xlfmath xl)
  find_all_libraries(BLASLAPACK_LIBS ESSL_LIBS_REQUIRED ESSL_PATH)
  if(BLASLAPACK_LIBS)
    find_package(Threads)
    if(Threads_FOUND)
      list(APPEND BLASLAPACK_LIBS ${CMAKE_THREAD_LIBS_INIT})
    endif()
    set(HAVE_BLAS TRUE)
    set(HAVE_LAPACK TRUE)
  endif()
elseif(APPLE)
  # on Mac we also know that blas and lapack are available
  set(BLASLAPACK_LIBS "-framework accelerate")
  set(HAVE_BLAS TRUE)
  set(HAVE_LAPACK TRUE)
endif()
if(NOT HAVE_BLAS OR NOT HAVE_LAPACK)
  # if we haven't found blas and lapack without using the standard
  # cmake tests, do it (this requires a working Fortran compiler)
  enable_language(Fortran)
  find_package(BLAS)
  if(BLAS_FOUND)
    list(APPEND BLASLAPACK_LIBS ${BLAS_LIBRARIES})
    set(HAVE_BLAS TRUE)
    find_package(LAPACK)
    if(LAPACK_FOUND)
      set(HAVE_LAPACK TRUE)
      list(APPEND BLASLAPACK_LIBS ${LAPACK_LIBRARIES})
    endif()
    if(CMAKE_Fortran_COMPILER_ID MATCHES "GNU")
      if(CMAKE_Fortran_COMPILER MATCHES "gfortran")
        list(APPEND BLASLAPACK_LIBS gfortran)
      elseif(CMAKE_Fortran_COMPILER MATCHES "f95")
        list(APPEND BLASLAPACK_LIBS gfortran)
      elseif(CMAKE_Fortran_COMPILER MATCHES "g77")
        list(APPEND BLASLAPACK_LIBS g2c)
      endif()
    endif()
  endif()
endif()
set(BLASLAPACK_PROCESS_INCLUDES )
set(BLASLAPACK_PROCESS_LIBS BLASLAPACK_LIBS)
libfind_process(BLASLAPACK)
