# -- Find GMP --
# Cenaero - 26/08/2010
#
# Inspired from:
# - http://www.itk.org/Wiki/CMake:How_To_Find_Libraries
#
# Find the native GMP headers and libraries.
#
# Input parameters which could influence the search:
# - GMP_ROOT - root folder where GMP is installed
#
# Once done, this will define:
# - GMP_INCLUDE_DIRS - where to find GMP headers
# - GMP_LIBRARIES - List of libraries when using GMP..
# - GMP_FOUND - True if GMP found.
#
# See FindGMP.cmake module for some explanations
#

if(GMP_INCLUDE_DIR AND GMP_LIBRARY)
  set(GMP_FIND_QUIETLY TRUE)
endif(GMP_INCLUDE_DIR AND GMP_LIBRARY)

include(LibFindMacros)


# Check input parameters
if(GMP_ROOT)
  set( _GMP_INCLUDE_SEARCH_DIRS
       ${GMP_ROOT}/include ${GMP_ROOT}
       ${_GMP_INCLUDE_SEARCH_DIRS})
  set( _GMP_LIBRARY_SEARCH_DIRS
       ${LIB_BASE}/lib
       ${_GMP_LIBRARY_SEARCH_DIRS})
endif(GMP_ROOT)

# Look for the header file
find_path(GMP_INCLUDE_DIR NAMES gmp.h
          PATHS ${_GMP_INCLUDE_SEARCH_DIRS} ${GMP_PKGCONF_INCLUDE_DIRS})

# Look for the library
find_library(GMP_LIBRARY NAMES gmp
             PATHS ${_GMP_LIBRARY_SEARCH_DIRS} ${GMP_PKGCONF_LIBRARY_DIRS})

# Set the include dir variables and the libraries and let libfind_process do the rest
set(GMP_PROCESS_INCLUDES GMP_INCLUDE_DIR)
set(GMP_PROCESS_LIBS GMP_LIBRARY)
libfind_process(GMP)
