# -- Find GMSH --
# Cenaero - 10/06/2010
#
# Inspired from:
# - http://www.itk.org/Wiki/CMake:How_To_Find_Libraries
# - http://github.com/temporaer/seralign/blob/master/CMakeModules/FindGMSH.cmake
#
# Find the native GMSH headers and libraries.
#
# Input parameters which could influence the search:
# - GMSH_ROOT - root folder where GMSH is installed
#
# Once done, this will define:
# - GMSH_INCLUDE_DIRS - where to find GMSH headers
# - GMSH_LIBRARIES - List of libraries when using GMSH.
# - GMSH_FOUND - True if GMSH found.
#
# See FindGmsh.cmake module for some explanations
#

include(LibFindMacros)

# Dependencies
libfind_package(Gmsh GMP)

if(GMSH_INCLUDE_DIR AND GMSH_LIBRARY)
  set(GMSH_FIND_QUIETLY TRUE)
endif(GMSH_INCLUDE_DIR AND GMSH_LIBRARY)


# Check input parameters
if(GMSH_ROOT)
  set( _GMSH_INCLUDE_SEARCH_DIRS
       ${GMSH_ROOT}/include-occ ${GMSH_ROOT}/include ${GMSH_ROOT}
       ${_GMSH_INCLUDE_SEARCH_DIRS})
  set( _GMSH_LIBRARY_SEARCH_DIRS
       ${GMSH_ROOT}/lib-occ ${GMSH_ROOT}/lib ${LIB_BASE}/lib
       ${_GMSH_LIBRARY_SEARCH_DIRS})
endif(GMSH_ROOT)

# Dependencies
# ! none !

# Use pkg-config to get hints about paths
# libfind_pkg_check_modules(GMSH_PKGCONF GMSH)

# Look for the header file
find_path(GMSH_INCLUDE_DIR NAMES Gmsh.h
          PATHS ${_GMSH_INCLUDE_SEARCH_DIRS} ${GMSH_PKGCONF_INCLUDE_DIRS})

# Look for the library
find_library(GMSH_LIBRARY NAMES Gmsh
             PATHS ${_GMSH_LIBRARY_SEARCH_DIRS} ${GMSH_PKGCONF_LIBRARY_DIRS})

# Set the include dir variables and the libraries and let libfind_process do the rest
set(GMSH_PROCESS_INCLUDES GMSH_INCLUDE_DIR)
set(GMSH_PROCESS_LIBS GMSH_LIBRARY)
libfind_process(GMSH)
