# -- Find OpenCasCade --
#
# Find the OpenCasCade headers and libraries.
# Inspired from:
#   http://panda-wiki.gsi.de/pub/Computing/CadConverter/FindOpenCascade.cmake
#
# Input parameters which could influence the search:
# - OCC_ROOT - root folder where OCC is installed
#
# Once done, this will define:
# OCC_FOUND - OCC is installed
# OCC_INCLUDE_DIR
# OCC_LIB_DIR
# OCC_LIBRARIES
# LIBNAME_LIBRARY - full OCC lib 'LIBNAME' path (e.g.: TKSTEP_LIBRARY)
# LIBNAME_FOUND - If OCC lib 'LIBNAME' is found

if(OCC_INCLUDE_DIR AND OCC_LIB_DIR)
  set(OCC_FIND_QUIETLY TRUE)
endif()

include(LibFindMacros)

# -- Dependencies --
libfind_package(OCC X11)
libfind_package(OCC OpenGL)

# Check input parameters
if(OCC_ROOT)
  set( _OCC_INCLUDE_SEARCH_DIRS
       ${OCC_ROOT}/include ${OCC_ROOT}/inc 
       ${_OCC_INCLUDE_SEARCH_DIRS})
  set( _OCC_LIBRARY_SEARCH_DIRS
       ${OCC_ROOT}/lib ${OCC_ROOT}/win32/lib
       ${_OCC_LIBRARY_SEARCH_DIRS})
endif(OCC_ROOT)

# Look for the header file
find_path(OCC_INCLUDE_DIR NAMES STEPCAFControl_Reader.hxx 
          PATHS ${_OCC_INCLUDE_SEARCH_DIRS}
                ${OCC_PKGCONF_INCLUDE_DIRS}
)

find_path(OCC_LIB_DIR
  NAMES libTKSTEP.so libTKSTEP.a TKSTEP.lib
  PATHS ${_OCC_LIBRARY_SEARCH_DIRS}
        ${OCC_PKGCONF_LIBRARY_DIRS}
)

if(OCC_LIB_DIR)
  list(APPEND OCC_REQUIRED_LIBS 
              TKernel TKMath  TKG2d  TKG3d TKGeomBase TKBRep TKGeomAlgo 
              TKTopAlgo TKPrim TKBool TKFeat TKFillet TKOffset TKHLR TKService
              TKV2d TKV3d TKMesh TKPCAF TKLCAF TKPLCAF TKCDF TKCAF PTKernel
              TKIGES TKSTEP TKSTEPBase TKSTEPAttr TKSTEP209 TKXCAF TKXDESTEP
              TKXDEIGES TKSTL TKVRML TKShHealing TKXSBase TKPShape TKBO
  )
endif(OCC_LIB_DIR)

set(OCC_FOUND TRUE)
foreach(libname ${OCC_REQUIRED_LIBS})
  find_library(${libname}_LIBRARY 
               NAMES ${libname} 
               PATHS ${_OCC_LIBRARY_SEARCH_DIRS} ${OCC_PKGCONF_LIBRARY_DIRS})
  if(${libname}_LIBRARY)
    mark_as_advanced(${libname}_LIBRARY)
    list(APPEND OCC_LIBRARIES ${${libname}_LIBRARY})
  else()
    set(OCC_FOUND FALSE)
  endif()
endforeach()

if(OCC_INCLUDE_DIR)
  mark_as_advanced(OCC_INCLUDE_DIR)
else()
  set(OCC_FOUND FALSE)
endif()

if(OCC_LIB_DIR)
  mark_as_advanced(OCC_LIB_DIR)
else()
  set(OCC_FOUND FALSE)
endif()

if (OCC_FOUND)
  if(NOT OCC_FIND_QUIETLY)
    message(STATUS "Looking for OCC... - found ${OCC_LIB_DIR}")
  endif()
else(OCC_FOUND)
  if(OCC_FIND_REQUIRED)
      message(FATAL_ERROR "Looking for OCC... - Not found")
  endif (OCC_FIND_REQUIRED)
endif(OCC_FOUND)
